#include <stdio.h>
#include <sys/types.h>
#include <memory.h>
#include "defines.h"

#define SCENE1ENABLE
#define SCENE2ENABLE
#define ENABLEJUMPS

#define TOTALFRAMES		(1985 + 1000 + 1400)

FILE * fp = NULL;

#define ALPHA_NORMAL   (0 + (1 << 2) + (0 << 4) + (1 << 6))
#define ALPHA_SUBTRACT (1 + (0 << 2) + (2 << 4) + (2 << 6))
#define ALPHA_ADD      (0 + (2 << 2) + (2 << 4) + (1 << 6))
#define ALPHA_MUL      (1 + (2 << 2) + (0 << 4) + (2 << 6))
#define ALPHA_ONE      (0 + (1 << 2) + (2 << 4) + (1 << 6))
#define ALPHA_DISABLE  (2 + (2 << 2) + (2 << 4) + (0 << 6))

typedef union
{
	u_long128     ul128;
	u_long64	  ul64[2];
	u_int         ui32[4];
	float         f32[4];
} qword_t;

void TexGenEnd()
{
	qword_t QW; 
	QW.ul128 = 0; 
	fwrite(&QW, sizeof(u_long128), 1, fp);
}

void TexGenNoise(u_int iDest, float R = 255.0f, float G = 255.0f, float B = 255.0f)
{
	qword_t QW; 
	QW.ui32[0] = 1;
	QW.f32[1] = (float)iDest;
	fwrite(&QW, sizeof(u_long128), 1, fp);
	QW.f32[0] = R;
	QW.f32[1] = G;
	QW.f32[2] = B;
	QW.f32[3] = 255.0f;
	fwrite(&QW, sizeof(u_long128), 1, fp);
}

void TexGenCopy(u_int iDest, u_int iSrc, u_int R = 0x80, u_int G = 0x80, u_int B = 0x80, u_int A = 0x80, float offsetx = 0, float offsety = 0)
{
	qword_t QW; 
	QW.ui32[0] = 2;
	QW.f32[1] = (float)iDest;
	QW.f32[2] = (float)iSrc;
	fwrite(&QW, sizeof(u_long128), 1, fp);
	QW.ui32[0] = R;
	QW.ui32[1] = G;
	QW.ui32[2] = B;
	QW.ui32[3] = A;
	fwrite(&QW, sizeof(u_long128), 1, fp);
	QW.f32[0] = offsetx;
	QW.f32[1] = offsety;
	fwrite(&QW, sizeof(u_long128), 1, fp);
}

void TexGenBlur(u_int iDest, u_int iSrc)
{
	qword_t QW; 
	QW.ui32[0] = 3;
	QW.f32[1] = (float)iDest;
	QW.f32[2] = (float)iSrc;
	fwrite(&QW, sizeof(u_long128), 1, fp);
}

void TexGenFill(u_int iDest, u_int R, u_int G, u_int B, u_int A)
{
	qword_t QW; 
	QW.ui32[0] = 4;
	QW.f32[1] = (float)iDest;
	fwrite(&QW, sizeof(u_long128), 1, fp);
	QW.ui32[0] = R;
	QW.ui32[1] = G;
	QW.ui32[2] = B;
	QW.ui32[3] = A;
	fwrite(&QW, sizeof(u_long128), 1, fp);
}

void TexGenAlphaMode(u_int iMode, u_int iFix)
{
	qword_t QW; 
	QW.ui32[0] = 5;
	QW.ui32[1] = iMode;
	QW.ui32[2] = iFix;
	fwrite(&QW, sizeof(u_long128), 1, fp);
}

void TexGenSinDistort(u_int iDest, u_int iSrc, float fHorizSines, float fHorizAmp, float fVertSines, float fVertAmp)
{
	qword_t QW; 
	QW.ui32[0] = 6;
	QW.f32[1] = (float)iDest;
	QW.f32[2] = (float)iSrc;
	fwrite(&QW, sizeof(u_long128), 1, fp);
	QW.f32[0] = fHorizSines * (2.0f * M_PI) / 128.0f;
	QW.f32[1] = fHorizAmp;
	QW.f32[2] = fVertSines * (2.0f * M_PI) / 128.0f;
	QW.f32[3] = fVertAmp;
	fwrite(&QW, sizeof(u_long128), 1, fp);
}

void TexGenRadial(u_int iDest)
{
	qword_t QW; 
	QW.ui32[0] = 7;
	QW.f32[1] = (float)iDest;
	fwrite(&QW, sizeof(u_long128), 1, fp);
}

void TexGenPlasma(u_int iDest, float fHorizSines, float fVertSines, float R = 255.0f, float G = 255.0f, float B = 255.0f)
{
	qword_t QW; 
	QW.ui32[0] = 8;
	QW.f32[1] = (float)iDest;
	fwrite(&QW, sizeof(u_long128), 1, fp);
	QW.f32[0] = fHorizSines * (2.0f * M_PI) / 128.0f;
	QW.f32[2] = fVertSines * (2.0f * M_PI) / 128.0f;
	fwrite(&QW, sizeof(u_long128), 1, fp);
	QW.f32[0] = R / 255.0f;
	QW.f32[1] = G / 255.0f;
	QW.f32[2] = B / 255.0f;
	QW.f32[3] = 1.0f;
	fwrite(&QW, sizeof(u_long128), 1, fp);
}

void TexGenPingPongBlur(u_int iDest, u_int iSrc, u_int iCount)
{
	qword_t QW; 
	QW.ui32[0] = 9;
	QW.f32[1] = (float)iDest;
	QW.f32[2] = (float)iSrc;
	QW.ui32[3] = iCount;
	fwrite(&QW, sizeof(u_long128), 1, fp);
}

void TexGenRGtoBA(u_int iDest, u_int iSrc)
{
	qword_t QW; 
	QW.ui32[0] = 10;
	QW.f32[1] = (float)iDest;
	QW.f32[2] = (float)iSrc;
	fwrite(&QW, sizeof(u_long128), 1, fp);
}

#define WRITEJUMP(mintime, maxtime, jumpto) {QW.ui32[0] = 4; QW.ui32[1] = mintime; QW.ui32[2] = maxtime; QW.ui32[3] = jumpto; \
								fwrite(&QW, sizeof(u_long128), 1, fp);}

#define END_OF_GEOMETRY		0
#define PLANE				1

#define WRITESPLINEPOS(time, x, y, z)	{QW.f32[0] = x;	QW.f32[1] = y; QW.f32[2] = z; QW.ui32[3] = time; \
								fwrite(&QW, sizeof(u_long128), 1, fp);}

#define WRITEEND			   {QW.ul128 = 0; fwrite(&QW, sizeof(u_long128), 1, fp);}
#define WRITEPLANE(resolution, tex, side) {QW.ui32[0] = PLANE; QW.ui32[1] = resolution; QW.f32[2] = tex; \
								QW.ui32[3] = side; \
								fwrite(&QW, sizeof(u_long128), 1, fp);}

#define WRITESPHERE(resolution, tex) {QW.ui32[0] = 2; QW.ui32[1] = resolution; QW.f32[2] = tex; \
								fwrite(&QW, sizeof(u_long128), 1, fp);}

#define WRITEHEMISPHERE(resolution, tex) {QW.ui32[0] = 5; QW.ui32[1] = resolution; QW.f32[2] = tex; \
								fwrite(&QW, sizeof(u_long128), 1, fp);}

#define WRITECYLINDER(resolution, tex) {QW.ui32[0] = 3; QW.ui32[1] = resolution; QW.f32[2] = tex; \
								fwrite(&QW, sizeof(u_long128), 1, fp);}

#define WRITEVERT(x,y,z,w) {QW.f32[0] = x;	QW.f32[1] = y;	QW.f32[2] = z;	QW.f32[3] = w; \
							fwrite(&QW, sizeof(u_long128), 1, fp);}

#define WRITECOLR(r,g,b) {QW.ui32[0] = r;	QW.ui32[1] = g;	QW.ui32[2] = b;	QW.ui32[3] = 0xFF; \
							fwrite(&QW, sizeof(u_long128), 1, fp);}

#define WRITESTATES(spheremapping, revnormals) {QW.ui32[0] = 6; QW.ui32[1] = spheremapping; QW.ui32[2] = revnormals; \
								fwrite(&QW, sizeof(u_long128), 1, fp);}

#define WRITEBOX(resolution, tex) {QW.ui32[0] = 7; QW.ui32[1] = resolution; QW.f32[2] = tex; \
								fwrite(&QW, sizeof(u_long128), 1, fp);}

#define WRITESCALE(x, y, z) {QW.ui32[0] = 8; QW.f32[1] = x; QW.f32[2] = y; QW.f32[3] = z;\
								fwrite(&QW, sizeof(u_long128), 1, fp);}

#define WRITEPRETRANS(x, y, z) {QW.ui32[0] = 9; QW.f32[1] = x; QW.f32[2] = y; QW.f32[3] = z;\
								fwrite(&QW, sizeof(u_long128), 1, fp);}

#define WRITEPOSTTRANS(x, y, z) {QW.ui32[0] = 10; QW.f32[1] = x; QW.f32[2] = y; QW.f32[3] = z;\
								fwrite(&QW, sizeof(u_long128), 1, fp);}

#define WRITEMULTIPLANE(resolution, tex) {QW.ui32[0] = 12; QW.ui32[1] = resolution; QW.f32[2] = tex; \
								fwrite(&QW, sizeof(u_long128), 1, fp);}

#define WRITESCREENPOLYIN(frame, colour, delta) {QW.ui32[0] = 13; QW.ui32[1] = frame; QW.ui32[2] = colour; QW.f32[3] = delta;\
								fwrite(&QW, sizeof(u_long128), 1, fp);}

void WriteEulerQuat(float roll, float pitch, float yaw, qword_t * QW)
{
	float cr, cp, cy, sr, sp, sy, cpcy, spsy;

	cr = cosf(roll/2);
	cp = cosf(pitch/2);
	cy = cosf(yaw/2);

	sr = sinf(roll/2);
	sp = sinf(pitch/2);
	sy = sinf(yaw/2);
	
	cpcy = cp * cy;
	spsy = sp * sy;

	QW->f32[0] = sr * cpcy - cr * spsy;
	QW->f32[1] = cr * sp * cy + sr * cp * sy;
	QW->f32[2] = cr * cp * sy - sr * sp * cy;
	QW->f32[3] = cr * cpcy + sr * spsy;

	fwrite(QW, sizeof(u_long128), 1, fp);
}

void WriteAnimEulerQuat(float roll, float pitch, float yaw, qword_t * QW)
{
	QW->ui32[0] = 11; QW->ui32[1] = 0; QW->ul64[1] = 0;
	fwrite(QW, sizeof(u_long128), 1, fp);
	
	WriteEulerQuat(roll, pitch, yaw, QW);
}

void CamPosSplineStart()
{
	int SplineStart = (ftell(fp) >> 4) + 1;
	int SavePos = ftell(fp);

	// w of 32
	// x of 33
	fseek(fp, (32 << 4) + (3 << 2), SEEK_SET);
	fwrite(&SplineStart, sizeof(u_int), 1, fp);

	fseek(fp, (33 << 4) + 0, SEEK_SET);
	fwrite(&SplineStart, sizeof(u_int), 1, fp);

	fseek(fp, SavePos, SEEK_SET);
}

void CamAtSplineStart()
{
	int SplineStart = (ftell(fp) >> 4) + 1;
	int SavePos = ftell(fp);

	// w of 32
	// x of 33
	fseek(fp, (33 << 4) + (1 << 2), SEEK_SET);
	fwrite(&SplineStart, sizeof(u_int), 1, fp);

	fseek(fp, (33 << 4) + (3 << 2), SEEK_SET);
	fwrite(&SplineStart, sizeof(u_int), 1, fp);

	fseek(fp, SavePos, SEEK_SET);
}

struct Scene
{
	int iMinTime;
	int iMaxTime;
	int iAddr;
};

Scene g_Scenes[20];

void SceneMarker(int iScene)
{
	if(g_Scenes[iScene].iAddr == 0)
		return;

	int iFilePos = ftell(fp);

	qword_t QW;

	QW.ui32[0] = 4; 
	QW.ui32[1] = g_Scenes[iScene].iMinTime; 
	QW.ui32[2] = g_Scenes[iScene].iMaxTime; 
	QW.ui32[3] = (iFilePos >> 4);

	fseek(fp, g_Scenes[iScene].iAddr, SEEK_SET);

	fwrite(&QW, sizeof(u_long128), 1, fp);

	fseek(fp, iFilePos, SEEK_SET);
}

void Jump(int mintime, int maxtime, int iScene)
{
	g_Scenes[iScene].iAddr = ftell(fp);
	g_Scenes[iScene].iMinTime = mintime;
	g_Scenes[iScene].iMaxTime = maxtime;

	qword_t QW;
	QW.ul128 = 0;
	fwrite(&QW, sizeof(u_long128), 1, fp);	// Reserve the qword
}

int main(int argc, char **argv) 
{
	if(argc != 2)
	{
		printf("usage: gen_data <filename>\n");
		return -1;
	}

	memset(g_Scenes, 0, sizeof(Scene) * 20);

	fp = fopen(argv[1], "wb");

	qword_t QW;
	int prim;

	// 0 /////////////
	// An empty giftag for synchronisation
	QW.ul128 = GIF_SET_TAG(NLOOP(0), EOP(1), PRE(0), 0, 0, NREG(0));
	fwrite(&QW, sizeof(u_long128), 1, fp);

	// 1 /////////////
	// State, set to 0: "Initialise"
	QW.ui32[0] = 0;		// State
	QW.ui32[1] = 0;		// Spheremapping enabled
	QW.ui32[2] = 0;		// Reverse normals
	QW.ui32[3] = 0;		// Repeat Count
	fwrite(&QW, sizeof(u_long128), 1, fp);

	// 2 /////////////
	QW.f32[0] = 2048.0f;	QW.f32[1] = 2048.0f;	QW.f32[2] = (float)(((2 << 24) - 1) / 2);		QW.f32[3] = 1.0f;
	fwrite(&QW, sizeof(u_long128), 1, fp);

	// 3 /////////////
	// WorldSpace -> ProjSpace 
	// (nearplane: z = 1.0f, farplane: z = -1.0f)
	// (left: x = -1.0f, right: x = 1.0f)
	// (top: y = 1.0f, bottom: y = -1.0f)
	float n = 0.1f;
	float f = 1000.0f;
	float fovparam = 0.08f;
	float l = -fovparam * (2048.0f / (640.0f * 0.5f));
	float r =  fovparam * (2048.0f / (640.0f * 0.5f));
	float t =  fovparam * (2048.0f / (350.0f * 0.5f));
	float b = -fovparam * (2048.0f / (350.0f * 0.5f));

	float projmat[4][4] =
	{
		(2*n) / (r-l)	, 0.0f				, 0.0f				, 0.0f,
		0.0f			, -(2*n) / (t-b)	, 0.0f				, 0.0f,
		((r+l) / (r-l)) , ((t+b) / (t-b))	, ((f+n) / (f-n))	, -1.0f,
		0.0f			, 0.0f				, ((2*f*n) / (f-n))	, 0.0f
	};

	fwrite(projmat, sizeof(float), 16, fp);

	// 7 /////////////
	// Camera position
	QW.f32[0] = 0.0f;	QW.f32[1] = 0.0f;	QW.f32[2] = 0.0f;	QW.f32[3] = 0.0f;
	fwrite(&QW, sizeof(u_long128), 1, fp);

	// 8 /////////////
	// Camera lookat
	QW.f32[0] = 0.0f;	QW.f32[1] = 0.0f;	QW.f32[2] = 0.0f;	QW.f32[3] = 0.0f;
	fwrite(&QW, sizeof(u_long128), 1, fp);

	// 9 /////////////
	prim = GS_SET_PRIM(GS_PRIM_TRISTRIP, GOURAUD(1), TEXTURED(1), FOG(0), ABE(1), AA(0), 0, CTXT(0), FIX(0));
	QW.ul64[0] = GIF_SET_TAG(0, EOP(1), PRE(1), prim, GIF_PACKED, NREG(3));
	QW.ul64[1] = REGDESC_STQ + (REGDESC_RGBAQ << 4) + (REGDESC_XYZ2 << 8);
	fwrite(&QW, sizeof(u_long128), 1, fp);

	// 10 ////////////
	// Light colour
	WRITEVERT(0.5f, 0.5f, 0.5f, 1.0f);

	// 11 ////////////
	// Light position
	WRITEVERT(10.0f, 10.0f, 10.0f, 0.0f);

	// 12 ////////////
	// Specular colour
	WRITEVERT(1.0f, 1.0f, 1.0f, 1.0f);


	// 13 ///////////// XYOFFSET TOP LEFT
	QW.ul64[0] = GIF_SET_TAG(NLOOP(1), EOP(1), PRE(1), 0, GIF_PACKED, NREG(1));
	QW.ul64[1] = REGDESC_AD;
	fwrite(&QW, sizeof(u_long128), 1, fp);

	// 14 /////////////
	QW.ui32[0] = 0;	QW.ui32[1] = 0;	QW.ui32[2] = 0x18;	QW.ui32[3] = 0;
	fwrite(&QW, sizeof(u_long128), 1, fp);

	// 15 ///////////// FRAME TAG
	QW.ul64[0] = GIF_SET_TAG(NLOOP(1), EOP(1), PRE(1), 0, GIF_PACKED, NREG(1));
	QW.ul64[1] = REGDESC_AD;
	fwrite(&QW, sizeof(u_long128), 1, fp);

	// 16 /////////////
	QW.ui32[0] = 280 + (4 << 16);	QW.ui32[1] = 0;	QW.ui32[2] = 0x4C;	QW.ui32[3] = 0;
	fwrite(&QW, sizeof(u_long128), 1, fp);

	// 17 /////////////
	// Point Giftag
	prim = GS_SET_PRIM(GS_PRIM_POINT, GOURAUD(0), TEXTURED(0), FOG(0), ABE(0), ALPHA(0), AA(0), CTXT(0), FIX(0));
	QW.ul64[0] = GIF_SET_TAG(NLOOP(1), EOP(1), PRE(1), prim, GIF_PACKED, NREG(2));
	QW.ul64[1] = REGDESC_RGBAQ + (REGDESC_XYZ2 << 4);
	fwrite(&QW, sizeof(u_long128), 1, fp);

	// 18 /////////////
	QW.ul64[0] = GIF_SET_TAG(NLOOP(3), EOP(1), PRE(1), 0, GIF_PACKED, NREG(1));
	QW.ul64[1] = REGDESC_AD;
	fwrite(&QW, sizeof(u_long128), 1, fp);

	// 19 ///////////// TEX0
	QW.ul64[0] = GS_SET_TEX0(280 * 32, 2, 0, 7, 7, 0, 0, 0, 0, 0, 0, 0); QW.ul64[1] = 0x06;
	fwrite(&QW, sizeof(u_long128), 1, fp);

	// 20 ///////////// TEX1
	QW.ul64[0] = 1 + (1 << 5) + (1 << 6) + (1 << 9);	QW.ui32[2] = 0x14;	QW.ui32[3] = 0;
	fwrite(&QW, sizeof(u_long128), 1, fp);

	// 21 ///////////// CLAMP
	QW.ul64[0] = 0;	QW.ul64[1] = 0x8;
	fwrite(&QW, sizeof(u_long128), 1, fp);

	// 22 ///////////// SPRITE PRIM
	prim = GS_SET_PRIM(GS_PRIM_SPRITE, GOURAUD(0), TEXTURED(0), FOG(0), ABE(1), ALPHA(0), 1, 0, 0);
	QW.ul64[0] = GIF_SET_TAG(NLOOP(1), EOP(1), PRE(1), prim, GIF_PACKED, NREG(6));
	QW.ul64[1] = REGDESC_TEX0 + (REGDESC_RGBAQ << 4) + (REGDESC_UV << 8) + (REGDESC_XYZ2 << 12) + 
				(REGDESC_UV << 16) + (REGDESC_XYZ2 << 20);
	fwrite(&QW, sizeof(u_long128), 1, fp);

	// 23 /////////////
	QW.ul128 = GS_SET_TEX0(0, 10, 0, 10, 10, 1, 0, 0, 0, 0, 0, 0);
	fwrite(&QW, sizeof(u_long128), 1, fp);

	// 24 /////////////
	QW.ui32[0] = 0xFF;	QW.ui32[1] = 0xFF;	QW.ui32[2] = 0xFF;	QW.ui32[3] = 0x60;
	fwrite(&QW, sizeof(u_long128), 1, fp);

	// 25 ///////////// U,V = 0,0
	QW.ui32[0] = 8;   QW.ui32[1] = 8; QW.ul64[1] = 0;
	fwrite(&QW, sizeof(u_long128), 1, fp);

	// 26 ///////////// Pos
	QW.ui32[0] = ((2048 - 320) << 4);	QW.ui32[1] = ((2048 - 112) << 4);	QW.ui32[2] = 16777216;	QW.ui32[3] = 0;
	fwrite(&QW, sizeof(u_long128), 1, fp);

	// 27 ///////////// U,V = 0,0
	QW.ui32[0] = (640 << 4) + 8;   QW.ui32[1] = (224 << 4) + 8; QW.ul64[1] = 0;
	fwrite(&QW, sizeof(u_long128), 1, fp);

	// 28 ///////////// Pos
	QW.ui32[0] = ((2048 + 320) << 4);	QW.ui32[1] = ((2048 + 112) << 4);	QW.ui32[2] = 16777216;	QW.ui32[3] = 0;
	fwrite(&QW, sizeof(u_long128), 1, fp);

	// 29 ///////////// TEX0 Tag
	QW.ul64[0] = GIF_SET_TAG(NLOOP(1), EOP(1), PRE(1), 0, GIF_PACKED, NREG(1));
	QW.ul64[1] = REGDESC_AD;
	fwrite(&QW, sizeof(u_long128), 1, fp);

	// 30 ///////////// TEX0
	QW.ul64[0] = GS_SET_TEX0(0x118 * 32, 2, 0, 7, 7, 1, 0, 0, 0, 0, 0, 0);
	QW.ul64[1] = 0x06;
	fwrite(&QW, sizeof(u_long128), 1, fp);

	// 31 ///////////// TEX0 Constants
	QW.ui32[0] = (u_int)GS_SET_TEX0(0x118 * 32, 2, 0, 7, 7, 1, 0, 0, 0, 0, 0, 0); 
	QW.f32[1] = 256; 
	QW.ui32[2] = (u_int)GS_SET_TEX0(0, 10, 0, 10, 10, 1, 0, 0, 0, 0, 0, 0);
	fwrite(&QW, sizeof(u_long128), 1, fp);

	// 32 ///////////// FRAME TIME
	QW.ui32[0] = 0;		QW.ui32[1] = TOTALFRAMES;	// Last frame
	QW.ui32[2] = 0;		// Campos countdown
	QW.ui32[3] = 0;
	fwrite(&QW, sizeof(u_long128), 1, fp);

	// 33 ///////////// SPLINE STUFF
	QW.ui32[0] = 0;	// Campos spline start
	QW.ui32[1] = 0;	// CamAt spline start
	QW.ui32[2] = 0;	// CamAt spline counter
	QW.ui32[3] = 0;	// CamAt spline ptr
	fwrite(&QW, sizeof(u_long128), 1, fp);

	// 34 ///////////// SPLINE MATRIX
	float splinemat[4][4] =
	{
		-1, 3,-3, 1,
		 3,-6, 3, 0,
	    -3, 0, 3, 0,
	     1, 4, 1, 0
	};

	for(int i = 0; i < 4; i++)
	{
		for(int j = 0; j < 4; j++)
		{
			splinemat[i][j] /= 6.0f;
		}
	}

	fwrite(splinemat, sizeof(u_long128), 4, fp);

	// 38 ///////////// SCALE
	WRITEVERT(1, 1, 1, 1);

	// 39 ///////////// PRETRANS
	WRITEVERT(0, 0, 0, 1);

	// 40 ///////////// ROTATE ANIM
	WriteEulerQuat(0, 0, 0, &QW);

	// 41 ///////////// POSTTRANS
	WRITEVERT(0, 0, 0, 1);

	// 42 ///////////// TEXSCALE
	WRITEVERT(0.2f, 0.2f, 0.2f, 1);

	// 43 ///////////// ALPHA GIFTAG
	QW.ul64[0] = GIF_SET_TAG(NLOOP(1), EOP(1), PRE(1), 0, GIF_PACKED, NREG(1));
	QW.ul64[1] = REGDESC_AD;
	fwrite(&QW, sizeof(u_long128), 1, fp);

	// 44 ///////////// ALPHA
	QW.ui32[0] = ALPHA_NORMAL;
	QW.ui32[1] = 0x80;
	QW.ul64[1] = 0x42;
	fwrite(&QW, sizeof(u_long128), 1, fp);

	// 45 ///////////// Alpha Restore
	QW.ul64[0] = GIF_SET_TAG(NLOOP(1), EOP(1), PRE(1), 0, GIF_PACKED, NREG(1));
	QW.ul64[1] = REGDESC_AD;
	fwrite(&QW, sizeof(u_long128), 1, fp);

	// 46 ///////////// ALPHA
	QW.ui32[0] = ALPHA_ONE;
	QW.ui32[1] = 0x80;
	QW.ul64[1] = 0x42;
	fwrite(&QW, sizeof(u_long128), 1, fp);

	// 47 /////////////
	// Geom commands
	
#ifdef ENABLEJUMPS
	Jump(1984, 2985, 1);
	Jump(2984, 6000, 2);
#endif

	SceneMarker(0);

#ifdef SCENE1ENABLE
	WriteAnimEulerQuat(0, 0, 0, &QW);

	/// Walls
	WRITESTATES(0,1);
	WRITEPOSTTRANS(0, 10, 0);
	WRITESCALE(20, 10, 20);
	WRITEBOX(14, 2);
	WriteEulerQuat(0, 0, 0, &QW);

	// Cylindrical bottoms
	WRITESTATES(1,0);
	WRITEPOSTTRANS(-10.0f, 2, -10.0f);
	WRITESCALE(1.5, 4, 1.5);
	WRITECYLINDER(8, 0);
	WriteEulerQuat(0, 0, 0, &QW);

	WRITEPOSTTRANS(10.0f, 2, -10.0f);
	WRITECYLINDER(8, 0);
	WriteEulerQuat(0, 0, 0, &QW);

	WRITEPOSTTRANS(-10.0f, 2, 10.0f);
	WRITECYLINDER(8, 0);
	WriteEulerQuat(0, 0, 0, &QW);

	WRITEPOSTTRANS(10.0f, 2, 10.0f);
	WRITECYLINDER(8, 0);
	WriteEulerQuat(0, 0, 0, &QW);

	// Sphere tops
	WRITESTATES(1,0);
	WRITESCALE(1, 1.5, 1.5);
	WRITEPOSTTRANS(-10, 6, -10);
	WRITEHEMISPHERE(20, 0);
	WriteEulerQuat(0, 0, M_PI / 2.0f, &QW);

	WRITEPOSTTRANS(10, 6, -10);
	WRITEHEMISPHERE(20, 0);
	WriteEulerQuat(0, 0, M_PI / 2.0f, &QW);

	WRITEPOSTTRANS(-10, 6, 10);
	WRITEHEMISPHERE(20, 0);
	WriteEulerQuat(0, 0, M_PI / 2.0f, &QW);

	WRITEPOSTTRANS(10, 6, 10);
	WRITEHEMISPHERE(20, 0);
	WriteEulerQuat(0, 0, M_PI / 2.0f, &QW);

	// Big corner boxes
	WRITESTATES(0,0);
	WRITESCALE(1, 10, 1);
	WRITEPOSTTRANS(-19.1, 9.9, -19.1);
	WRITEBOX(1, 2);
	WriteEulerQuat(0, 0, 0, &QW);

	WRITEPOSTTRANS(19.1, 9.9, -19.1);
	WRITEBOX(1, 2);
	WriteEulerQuat(0, 0, 0, &QW);

	WRITEPOSTTRANS(-19.1, 9.9, 19.1);
	WRITEBOX(1, 2);
	WriteEulerQuat(0, 0, 0, &QW);

	WRITEPOSTTRANS(19.1, 9.9, 19.1);
	WRITEBOX(1, 2);
	WriteEulerQuat(0, 0, 0, &QW);

	// Shiny Boxes
	WriteAnimEulerQuat(0.04f, 0.05f, 0.02f, &QW);

	WRITESTATES(1,0);
	WRITESCALE(1, 1, 1);
	WRITEPOSTTRANS(-10, 10, -10);
	WRITEBOX(8, 2);
	WriteEulerQuat(0, 0, 0, &QW);

	WRITEPOSTTRANS(10, 10, -10);
	WRITEBOX(8, 2);
	WriteEulerQuat(0, 0, 0, &QW);

	WRITEPOSTTRANS(-10, 10, 10);
	WRITEBOX(8, 2);
	WriteEulerQuat(0, 0, 0, &QW);

	WRITEPOSTTRANS(10, 10, 10);
	WRITEBOX(8, 2);
	WriteEulerQuat(0, 0, 0, &QW);

	// Floor spheres
	WriteAnimEulerQuat(0, 0.1f, 0, &QW);
	WRITEPOSTTRANS(0, 0, 0);
	WRITESCALE(0.5, 8, 8);
	WRITEHEMISPHERE(20, 0);
	WriteEulerQuat(0, 0, M_PI / 2.0f, &QW);
	WriteAnimEulerQuat(0, -0.1f, 0, &QW);
	WRITESCALE(1, 5, 5);
	WRITEHEMISPHERE(20, 0);
	WriteEulerQuat(0, 0, M_PI / 2.0f, &QW);
	WriteAnimEulerQuat(0, 0.1f, 0, &QW);
	WRITESCALE(2, 2, 2);
	WRITEHEMISPHERE(20, 0);
	WriteEulerQuat(0, 0, M_PI / 2.0f, &QW);

	WRITESCREENPOLYIN(1960, 0xFF, 0.1f)
	
	WRITEEND;

	SceneMarker(1);

#endif

#ifdef SCENE2ENABLE

	WriteAnimEulerQuat(0, 0, 0.1f, &QW);

	WRITESTATES(0,1);
	WRITEPOSTTRANS(0, 0, 0);
	WRITESCALE(20, 100, 20);
	WRITECYLINDER(20, 2);
	WriteEulerQuat(M_PI / 2.0f, 0, 0, &QW);		

	WRITESCALE(20, 20, 20);
	WRITEPOSTTRANS(0, 0, -99);
	WRITEHEMISPHERE(20, 2);
	WriteEulerQuat(0, M_PI / 2.0f, 0, &QW);	

	WRITEPOSTTRANS(0, 0,  99);
	WRITEHEMISPHERE(20, 2);
	WriteEulerQuat(0, 3*M_PI / 2.0f, 0, &QW);

	WriteAnimEulerQuat(0.04f, 0.03f, 0.01f, &QW);
	WRITESTATES(0,0);

	WRITESCALE(2, 2, 2);
	WRITEPOSTTRANS(0, 0, 0);
	WRITEPRETRANS(-2, 5, -10);
	WRITESPHERE(20, 1);
	WriteEulerQuat(0, 0, 0, &QW);

	WriteAnimEulerQuat(0.01f, 0.04f, 0.02f, &QW);	

	WRITEPRETRANS(4, 6, -5);
	WRITESPHERE(20, 1);
	WriteEulerQuat(0, 0, 0, &QW);

	WriteAnimEulerQuat(0.05f, 0.022f, 0.01f, &QW);	

	WRITEPRETRANS(-10, 2, -13);
	WRITESPHERE(20, 1);
	WriteEulerQuat(0, 0, 0, &QW);

	WriteAnimEulerQuat(0.03f, 0.03f, 0.05f, &QW);	

	WRITEPRETRANS(2, 10, 5);
	WRITESPHERE(20, 1);
	WriteEulerQuat(0, 0, 0, &QW);

	WRITEPRETRANS(0, 0, 0);

	WRITESCREENPOLYIN(2960, 0xFF, 0.1f)

	WRITEEND;

	SceneMarker(2);

#endif

	WriteAnimEulerQuat(0, 0, 0, &QW);

	/// Sphere
	WRITESTATES(0,1);
	WRITEPRETRANS(0, 0, 0);
	WRITEPOSTTRANS(0, 0, 0);
	WRITESCALE(30, 30, 30);
	WRITESPHERE(16, 2);
	WriteEulerQuat(0, 0, 0, &QW);

	// Top pole
	WRITESTATES(1,0);
	WRITEPRETRANS(0, 0, 0);
	WRITEPOSTTRANS(0, 18, 0);
	WRITESCALE(2.5f, 10, 2.5f);
	WRITECYLINDER(15, 1);
	WriteEulerQuat(0, 0, 0, &QW);

	// Bottom pole
	WRITEPRETRANS(0, 0, 0);
	WRITEPOSTTRANS(0, -18, 0);
	WRITESCALE(2.5f, 12, 2.5f);
	WRITECYLINDER(15, 1);
	WriteEulerQuat(0, 0, 0, &QW);

	// Left pole
	WRITEPRETRANS(0, 0, 0);
	WRITEPOSTTRANS(18, 0, 0);
	WRITESCALE(2.5f, 12, 2.5f);
	WRITECYLINDER(15, 1);
	WriteEulerQuat(0, 0, M_PI / 2.0f, &QW);

	// Right pole
	WRITEPRETRANS(0, 0, 0);
	WRITEPOSTTRANS(-18, 0, 0);
	WRITESCALE(2.5f, 12, 2.5f);
	WRITECYLINDER(15, 1);
	WriteEulerQuat(0, 0, M_PI / 2.0f, &QW);

	// Far pole
	WRITEPRETRANS(0, 0, 0);
	WRITEPOSTTRANS(0, 0, -18);
	WRITESCALE(2.5f, 12, 2.5f);
	WRITECYLINDER(20, 1);
	WriteEulerQuat(M_PI / 2.0f, 0, 0, &QW);

	// Near pole
	WRITEPRETRANS(0, 0, 0);
	WRITEPOSTTRANS(0, 0, 18);
	WRITESCALE(2.5f, 12, 2.5f);
	WRITECYLINDER(15, 1);
	WriteEulerQuat(M_PI / 2.0f, 0, 0, &QW);

	// Sphere thingy
	WriteAnimEulerQuat(0.14f, 0.02f, 0.11f, &QW);

	//WRITESTATES(1,0);
	WRITEPOSTTRANS(0, 0, 0);
	WRITESCALE(4, 1, 1);
	WRITESPHERE(7, 12);
	WriteEulerQuat(0, 0, 0, &QW);

	WRITESCALE(1, 4, 1);
	WRITESPHERE(7, 12);
	WriteEulerQuat(0, 0, 0, &QW);

	WRITESCALE(1, 1, 4);
	WRITESPHERE(7, 12);
	WriteEulerQuat(0, 0, 0, &QW);

	WriteAnimEulerQuat(0.04f, 0.05f, 0.02f, &QW);
	WRITESTATES(1,0);
	WRITESCALE(1, 1, 1);
	WRITEPOSTTRANS(-10, 10, -10);
	WRITEBOX(8, 1);
	WriteEulerQuat(0, 0, 0, &QW);
	WRITEPOSTTRANS(10, 10, -10);
	WRITEBOX(8, 1);
	WriteEulerQuat(0, 0, 0, &QW);
	WRITEPOSTTRANS(-10, 10, 10);
	WRITEBOX(8, 1);
	WriteEulerQuat(0, 0, 0, &QW);
	WRITEPOSTTRANS(10, 10, 10);
	WRITEBOX(8, 1);
	WriteEulerQuat(0, 0, 0, &QW);
	WRITEPOSTTRANS(-10, -10, -10);
	WRITEBOX(8, 1);
	WriteEulerQuat(0, 0, 0, &QW);
	WRITEPOSTTRANS(10, -10, -10);
	WRITEBOX(8, 1);
	WriteEulerQuat(0, 0, 0, &QW);
	WRITEPOSTTRANS(-10, -10, 10);
	WRITEBOX(8, 1);
	WriteEulerQuat(0, 0, 0, &QW);
	WRITEPOSTTRANS(10, -10, 10);
	WRITEBOX(8, 1);
	WriteEulerQuat(0, 0, 0, &QW);

	WRITESCREENPOLYIN(4200, 0x00, 0.01f)

	CamPosSplineStart();
		
#ifdef SCENE1ENABLE

	// CAMERA POS SPLINE POINTS							
	/*WRITESPLINEPOS(0  , -100,  60,-100);						
	WRITESPLINEPOS(100, -80,  50, -80);						
	WRITESPLINEPOS(100, -50,  40, -40);						
	WRITESPLINEPOS(100, -10,  20, -20);*/

	WRITESPLINEPOS(0  ,  10,  20,  0);						
	WRITESPLINEPOS(100,   0,  15,  0);						
	WRITESPLINEPOS(100,  -5,  10,  -5);						
	WRITESPLINEPOS(100, -10,  10, -10);

	WRITESPLINEPOS(100,  4,   10,-5.5);				
	WRITESPLINEPOS(100,	 7,   5,   -5);/////////////////////////////						
	WRITESPLINEPOS(100,  4,   4, -5.5);						
	WRITESPLINEPOS(100,  0, 2.5,   -4);	
	WRITESPLINEPOS(100, -3,   3,    3);	
	WRITESPLINEPOS(100, -3,   3,    3);	
	WRITESPLINEPOS(100,	 0,   2,    3);	
	WRITESPLINEPOS(100,  3,   3,    3);	
	WRITESPLINEPOS(100,  3,   3,    3);	
	WRITESPLINEPOS(100,  4, 2.5,   -3);	
	WRITESPLINEPOS(100,  4, 5.0, -3.5);	
	WRITESPLINEPOS(100,  4, 4.5, -4.6);	
	WRITESPLINEPOS(100,  0, 2.5,   -4);	
	WRITESPLINEPOS(100,  0, 2.5,   -4);	
	WRITESPLINEPOS(100, -5, 1.5,    0);						
	WRITESPLINEPOS(100, -3,   2,    4);
	WRITESPLINEPOS(50,	 4,   5,    0);//////////////////////////////
	WRITESPLINEPOS(35,	 0,   3,    0);//////////////////////////////
	WRITESPLINEPOS(0,	 0,   0.3f, 0);/////////////////////////////
	WRITESPLINEPOS(0,	 0, -10,    0);/////////////////////////////

#endif

#ifdef SCENE2ENABLE

	// SCENE 2! POS
	WRITESPLINEPOS(0  ,  -10,  0, -40);						
	WRITESPLINEPOS(100,   5,  0, -40);						
	WRITESPLINEPOS(0  ,  0,  0,  40);						
	WRITESPLINEPOS(0  ,	 5,  0,  40);

	WRITESPLINEPOS(0  ,  -10, -10, -40);						
	WRITESPLINEPOS(100,  0,  5,  -40);						
	WRITESPLINEPOS(0  ,  0,  -4,  40);						
	WRITESPLINEPOS(0  ,	 0,  0,   40);

	WRITESPLINEPOS(0  ,  -10,  0, -40);						
	WRITESPLINEPOS(100,   5,  0, -40);						
	WRITESPLINEPOS(0  ,  0,  0,  40);						
	WRITESPLINEPOS(0  ,	 5,  0,  40);

	WRITESPLINEPOS(0  ,  -10, -10, -40);						
	WRITESPLINEPOS(100,  0,  5,  -40);						
	WRITESPLINEPOS(0  ,  0,  -4,  40);						
	WRITESPLINEPOS(0  ,	 0,  0,   40);

	WRITESPLINEPOS(0  ,  -10, -10, -10);						
	WRITESPLINEPOS(100,   5, -8, -8);						
	WRITESPLINEPOS(100,   0,  6,  8);
	WRITESPLINEPOS(100,  -3,  4,  0);											
	WRITESPLINEPOS(100,   5, -8, -8);						
	WRITESPLINEPOS(100,   0,  6,  8);
	WRITESPLINEPOS(0,  -3,  4,  0);						
	WRITESPLINEPOS(0  ,	  0, -8, -8);

	WRITESPLINEPOS(0  ,  0,  0, -40);						
	WRITESPLINEPOS(100,  0,  0, -40);						
	WRITESPLINEPOS(0  ,  0,  0,  60);						
	WRITESPLINEPOS(0  ,	 0,  0,  60);

#endif

	WRITESPLINEPOS(0  ,  0,  0, -10);
	WRITESPLINEPOS(100,  0,  0, -10);	
	WRITESPLINEPOS(0  ,  0,  0, -4);											
	WRITESPLINEPOS(0,	 0,  0, -4);
	WRITESPLINEPOS(100,	 2,  5, -3);
	WRITESPLINEPOS(0  ,	 4,  7, -3);
	WRITESPLINEPOS(0  ,	 5,  7, 0);

	WRITESPLINEPOS(0,	 -0,  -7, -4);
	WRITESPLINEPOS(100,	 -2,  -5, -3);
	WRITESPLINEPOS(0  ,	 -4,  0, -3);
	WRITESPLINEPOS(0  ,	 -5,  3, 0);

	WRITESPLINEPOS(0,	 -4,  7, -4);
	WRITESPLINEPOS(100,	 -3,  5, -3);
	WRITESPLINEPOS(0  ,	 -3,  3, -3);
	WRITESPLINEPOS(0  ,	  0,  3, 0);

	WRITESPLINEPOS(0,	  5,  5, 0);
	WRITESPLINEPOS(200,	  0,  5, 5);
	WRITESPLINEPOS(0,	 -5,  5, 0);
	WRITESPLINEPOS(0  ,	  0,  5, -5);

	WRITESPLINEPOS(0,	  5,  5, 0);
	WRITESPLINEPOS(200,	  0,  5, -5);
	WRITESPLINEPOS(0,	 -5,  5, 0);
	WRITESPLINEPOS(0  ,	  0,  5, 5);

	WRITESPLINEPOS(0,	  5,  0, 0);
	WRITESPLINEPOS(100,	  0,  0, -5);
	WRITESPLINEPOS(100,	 -5,  0, 0);
	WRITESPLINEPOS(100,	  0,  0, 5);
	WRITESPLINEPOS(100,	  5,  0, 0);
	WRITESPLINEPOS(100,	  0,  0, -5);
	WRITESPLINEPOS(100,	  0,  0, -5);
	WRITESPLINEPOS(0,	  0,  0, -25);
	WRITESPLINEPOS(0  ,	  0,  0, -25);

	CamAtSplineStart();

	// CAMERA AT SPLINE POINTS

#ifdef SCENE1ENABLE

	WRITESPLINEPOS(0,	3, 2, 0);//////////////////////////
	WRITESPLINEPOS(200, 3, 2, 0);
	WRITESPLINEPOS(200,-3, 2, 3);
	WRITESPLINEPOS(200,-3.5, 2, 3.5);
	WRITESPLINEPOS(400,-3.5, 3.5f, 3.5);
	WRITESPLINEPOS(200,-3.5, 3.5f, 3.5);
	WRITESPLINEPOS(200, 0, 1, 0);
	WRITESPLINEPOS(200, 3, 2, -4);
	WRITESPLINEPOS(100, 0, 2, 0);
	WRITESPLINEPOS(100, 3, 2, 0);
	WRITESPLINEPOS(100,-3, 2, 3);
	WRITESPLINEPOS(85,	0, 0, 0);////////////////////////////
	WRITESPLINEPOS(0,	0, 0, 0);////////////////////////////
	WRITESPLINEPOS(0,	0, 0, 0);////////////////////////////

#endif

#ifdef SCENE2ENABLE

	// SCENE2! AT
	WRITESPLINEPOS(0  ,  0,  0,  0);						
	WRITESPLINEPOS(100,  0, -2,  0);						
	WRITESPLINEPOS(0  ,  0,  2,  0);						
	WRITESPLINEPOS(0  ,	 0,  0,  0);

	WRITESPLINEPOS(0  ,  0,  0,  0);						
	WRITESPLINEPOS(100,  0,  0,  0);						
	WRITESPLINEPOS(0  ,  0,  0,  0);						
	WRITESPLINEPOS(0  ,	 0,  0,  0);

	WRITESPLINEPOS(0  ,  0,  0,  0);						
	WRITESPLINEPOS(100,  0, -2,  0);						
	WRITESPLINEPOS(0  ,  0,  2,  0);						
	WRITESPLINEPOS(0  ,	 0,  0,  0);

	WRITESPLINEPOS(0  ,  0,  0,  0);						
	WRITESPLINEPOS(600,  0,  0,  0);						
	WRITESPLINEPOS(0  ,  0,  0,  0);						
	WRITESPLINEPOS(0  ,	 0,  0,  0);

	WRITESPLINEPOS(0  ,  0,  0,  0);						
	WRITESPLINEPOS(100,  0,  0, 100);						
	WRITESPLINEPOS(0  ,  0,  0, 100);						
	WRITESPLINEPOS(0  ,	 0,  0,  0);

#endif

	WRITESPLINEPOS(0  ,  0,  0, 0);						
	WRITESPLINEPOS(10000, 0,  0, 0);						
	WRITESPLINEPOS(0  ,  0,  0, 0);						
	WRITESPLINEPOS(0  ,	 0,  0, 0);

	int iQWord = (ftell(fp) >> 4);
	printf("Geometry: %d qwords\n", iQWord);
	int iNumSpace = 700 - iQWord;
	qword_t * QWA = new qword_t[iNumSpace];
	fwrite(QWA, sizeof(u_long128), iNumSpace, fp);

	// 700 ///////////// SPRITE GIFTAG
	prim = GS_SET_PRIM(GS_PRIM_SPRITE, GOURAUD(0), TEXTURED(1), FOG(0), ABE(1), ALPHA(0), 1, 0, 0);
	QW.ul64[0] = GIF_SET_TAG(NLOOP(1), EOP(1), PRE(1), prim, GIF_PACKED, NREG(6));
	QW.ul64[1] = REGDESC_TEX0 + (REGDESC_RGBAQ << 4) + (REGDESC_UV << 8) + (REGDESC_XYZ2 << 12) + 
					(REGDESC_UV << 16) + (REGDESC_XYZ2 << 20);
	fwrite(&QW, sizeof(u_long128), 1, fp);

	// 701 ///////////// TEX0
	QW.ul128 = GS_SET_TEX0(0x118 * 32, 2, 0, 7, 7, 1, 0, 0, 0, 0, 0, 0);
	fwrite(&QW, sizeof(u_long128), 1, fp);

	// 702 ///////////// COLOUR
	QW.ul128 = 0;
	fwrite(&QW, sizeof(u_long128), 1, fp);

	// 703 ///////////// U,V 0
	QW.ul128 = 0;
	fwrite(&QW, sizeof(u_long128), 1, fp);

	// 704 ///////////// Pos 0
	QW.ul128 = 0;
	fwrite(&QW, sizeof(u_long128), 1, fp);

	// 705 ///////////// U,V 1
	QW.ul128 = 0;
	fwrite(&QW, sizeof(u_long128), 1, fp);

	// 706 ///////////// Pos 1
	QW.ul128 = 0;
	fwrite(&QW, sizeof(u_long128), 1, fp);

	// 707 ///////////// XYOFFSET A+D
	QW.ul64[0] = GIF_SET_TAG(NLOOP(1), EOP(1), PRE(1), 0, GIF_PACKED, NREG(1));
	QW.ul64[1] = REGDESC_AD;
	fwrite(&QW, sizeof(u_long128), 1, fp);

	// 708 ///////////// XYOFFSET
	QW.ui32[0] = 0;	QW.ui32[1] = 0;	QW.ui32[2] = 0x18;	QW.ui32[3] = 0;
	fwrite(&QW, sizeof(u_long128), 1, fp);

	// 709 ///////////// FLOATXYOFFSET
	QW.f32[0] = 0.0f;	QW.f32[1] = 0.0f;	QW.f32[2] = 0;	QW.f32[3] = 0;
	fwrite(&QW, sizeof(u_long128), 1, fp);

	// 710 ///////////// SPRITESIZE
	QW.f32[0] = 256.0f;	QW.f32[1] = 128.0f;	QW.f32[2] = 0;	QW.f32[3] = 0;
	fwrite(&QW, sizeof(u_long128), 1, fp);

	// 711 ///////////// TEX0 Constants
	QW.ui32[0] = (u_int)GS_SET_TEX0(0x118 * 32, 2, 0, 7, 7, 1, 0, 0, 0, 0, 0, 0); 
	QW.f32[1] = 256; QW.ul64[1] = 0;
	fwrite(&QW, sizeof(u_long128), 1, fp);

	// 712 ///////////// POINT GIFTAG
	prim = GS_SET_PRIM(GS_PRIM_POINT, GOURAUD(0), TEXTURED(0), FOG(0), ABE(1), ALPHA(0), AA(0), CTXT(0), FIX(0));
	QW.ul64[0] = GIF_SET_TAG(NLOOP(1), EOP(1), PRE(1), prim, GIF_PACKED, NREG(2));
	QW.ul64[1] = REGDESC_RGBAQ + (REGDESC_XYZ2 << 4);
	fwrite(&QW, sizeof(u_long128), 1, fp);

	// 713 ///////////// POINT COLOUR
	QW.ui32[0] = 0xFF;	QW.ui32[1] = 0xFF;	QW.ui32[2] = 0xFF;	QW.ui32[3] = 0xFF;
	fwrite(&QW, sizeof(u_long128), 1, fp);

	// 714 ///////////// POINT POSITION
	QW.ui32[0] = 0;	QW.ui32[1] = 0;	QW.ui32[2] = 0;	QW.ui32[3] = 0;
	fwrite(&QW, sizeof(u_long128), 1, fp);

	// 715 ///////////// FRAME TAG
	QW.ul64[0] = GIF_SET_TAG(NLOOP(1), EOP(1), PRE(1), 0, GIF_PACKED, NREG(1));
	QW.ul64[1] = REGDESC_AD;
	fwrite(&QW, sizeof(u_long128), 1, fp);

	// 716 ///////////// FRAME
	QW.f32[0] = 280 + (2 << 16);	QW.ui32[1] = 0;	QW.ui32[2] = 0x4C;	QW.ui32[3] = 0;
	fwrite(&QW, sizeof(u_long128), 1, fp);

	// 717 ///////////// COLOUR SCALE
	QW.f32[0] = 255.0f;	QW.f32[1] = 255.0f;	QW.f32[2] = 255.0f;	QW.f32[3] = 255.0f;
	fwrite(&QW, sizeof(u_long128), 1, fp);

	// 718 ///////////// U,V = 0.5, 0.5
	QW.ui32[0] = (128 << 4) + 8;   QW.ui32[1] = (128 << 4) + 8; QW.ul64[1] = 0;
	fwrite(&QW, sizeof(u_long128), 1, fp);

	// 719 ///////////// U,V = 128.5, 128.5
	QW.ui32[0] = (256 << 4) + 8;   QW.ui32[1] = (256 << 4) +8 ; QW.ul64[1] = 0;
	fwrite(&QW, sizeof(u_long128), 1, fp);

	// 720 ///////////// Unit Colour
	QW.ui32[0] = 0x80;	QW.ui32[1] = 0x80;	QW.ui32[2] = 0x80;	QW.ui32[3] = 0x80;
	fwrite(&QW, sizeof(u_long128), 1, fp);

	// 721 ///////////// TEX1 Tag
	QW.ul64[0] = GIF_SET_TAG(NLOOP(1), EOP(1), PRE(1), 0, GIF_PACKED, NREG(1));
	QW.ul64[1] = REGDESC_AD;
	fwrite(&QW, sizeof(u_long128), 1, fp);

	// 722 ///////////// TEX1
	QW.ul64[0] = 1 + (1 << 5) + (1 << 6);	QW.ui32[2] = 0x14;	QW.ui32[3] = 0;
	fwrite(&QW, sizeof(u_long128), 1, fp);

	// 723 ///////////// FRAME Constants
	QW.f32[0] = (float)(280 + (2 << 16));
	QW.f32[1] = 8.0f;
	fwrite(&QW, sizeof(u_long128), 1, fp);

	// 724 ///////////// TEXTURESIZE
	QW.f32[0] = 128.0f;	QW.f32[1] = 128.0f;	QW.f32[2] = 0;	QW.f32[3] = 0;
	fwrite(&QW, sizeof(u_long128), 1, fp);

	// 725 ///////////// Blur Colour
	QW.ui32[0] = 0x80;	QW.ui32[1] = 0x80;	QW.ui32[2] = 0x80;	QW.ui32[3] = 0x60;
	fwrite(&QW, sizeof(u_long128), 1, fp);

	// 726 ///////////// COLOURED SPRITE
	prim = GS_SET_PRIM(GS_PRIM_SPRITE, GOURAUD(0), TEXTURED(0), FOG(0), ABE(1), ALPHA(0), 1, 0, 0);
	QW.ul64[0] = GIF_SET_TAG(NLOOP(1), EOP(1), PRE(1), prim, GIF_PACKED, NREG(3));
	QW.ul64[1] = (REGDESC_RGBAQ) + (REGDESC_XYZ2 << 4) + (REGDESC_XYZ2 << 8);
	fwrite(&QW, sizeof(u_long128), 1, fp);

	// 727 ///////////// COLOURED SPRITE COLOUR
	QW.ui32[0] = 0x80;	QW.ui32[1] = 0x80;	QW.ui32[2] = 0x80;	QW.ui32[3] = 0x80;
	fwrite(&QW, sizeof(u_long128), 1, fp);

	// 728 ///////////// COLOURED SPRITE POS1
	QW.ui32[0] = 0;	QW.ui32[1] = 0;	QW.ui32[2] = 0;	QW.ui32[3] = 0;
	fwrite(&QW, sizeof(u_long128), 1, fp);

	// 729 ///////////// COLOURED SPRITE POS2
	QW.ui32[0] = 128 << 4;	QW.ui32[1] = 128 << 4;	QW.ui32[2] = 0;	QW.ui32[3] = 0;
	fwrite(&QW, sizeof(u_long128), 1, fp);

	// 730 ///////////// ALPHA GIFTAG
	QW.ul64[0] = GIF_SET_TAG(NLOOP(1), EOP(1), PRE(1), 0, GIF_PACKED, NREG(1));
	QW.ul64[1] = REGDESC_AD;
	fwrite(&QW, sizeof(u_long128), 1, fp);

	// 731 ///////////// ALPHA
	QW.ul64[0] = 0 + (1 << 2) + (0 << 4) + (1 << 6);		 QW.ul64[1] = 0x42;
	fwrite(&QW, sizeof(u_long128), 1, fp);

	// 732 ///////////// TEXTURED POINT GIFTAG
	prim = GS_SET_PRIM(GS_PRIM_POINT, GOURAUD(0), TEXTURED(1), FOG(0), ABE(0), ALPHA(0), 1, 0, 0);
	QW.ul64[0] = GIF_SET_TAG(NLOOP(1), EOP(1), PRE(1), prim, GIF_PACKED, NREG(2));
	QW.ul64[1] = REGDESC_UV + (REGDESC_XYZ2 << 4);
	fwrite(&QW, sizeof(u_long128), 1, fp);

	// 733 ///////////// TEXTURED POINT UV
	QW.ui32[0] = (64 << 4) + 8;	QW.ui32[1] = (64 << 4) + 8;	QW.ui32[2] = 0;	QW.ui32[3] = 0;
	fwrite(&QW, sizeof(u_long128), 1, fp);

	// 734 ///////////// TEXTURED POINT POSITION
	QW.ui32[0] = 0;	QW.ui32[1] = 0;	QW.ui32[2] = 0;	QW.ui32[3] = 0;
	fwrite(&QW, sizeof(u_long128), 1, fp);

	// 735 ///////////// TEX0 Tag
	QW.ul64[0] = GIF_SET_TAG(NLOOP(1), EOP(1), PRE(1), 0, GIF_PACKED, NREG(1));
	QW.ul64[1] = REGDESC_AD;
	fwrite(&QW, sizeof(u_long128), 1, fp);

	// 736 ///////////// TEX0
	QW.ul64[0] = GS_SET_TEX0(0x118 * 32, 2, 0, 7, 7, 1, 1, 0, 0, 0, 0, 0);
	QW.ul64[1] = 0x06;
	fwrite(&QW, sizeof(u_long128), 1, fp);

	// 737 ///////////// RADIAL CONSTANTS
	QW.f32[0] = 64.0f;
	QW.f32[1] = 64.0f;
	QW.f32[2] = 4.0f;
	fwrite(&QW, sizeof(u_long128), 1, fp);

	// 738 ///////////// Texflush
	QW.ul64[0] = GIF_SET_TAG(NLOOP(1), EOP(1), PRE(1), 0, GIF_PACKED, NREG(1));
	QW.ul64[1] = REGDESC_AD;
	fwrite(&QW, sizeof(u_long128), 1, fp);

	// 739 /////////////
	QW.ul64[0] = 0;
	QW.ul64[1] = 0x3F;
	fwrite(&QW, sizeof(u_long128), 1, fp);

	// 740 ///////////// RGtoBA A+D Megamix HOOHOO!
	QW.ul64[0] = GIF_SET_TAG(NLOOP(6), EOP(1), PRE(1), 0, GIF_PACKED, NREG(1));
	QW.ul64[1] = REGDESC_AD;
	fwrite(&QW, sizeof(u_long128), 1, fp);

	// 741 ///////////// FRAME PSMCT16
	QW.ui32[0] = 280 + (4 << 16) + (2 << 24);	QW.ui32[1] = 0;	QW.ui32[2] = 0x4C;	QW.ui32[3] = 0;
	fwrite(&QW, sizeof(u_long128), 1, fp);

	// 742 ///////////// TEX0
	QW.ul64[0] = GS_SET_TEX0(0x118 * 32, 4, 2, 10, 10, 1, 1, 0, 0, 0, 0, 0);
	QW.ul64[1] = 0x06;
	fwrite(&QW, sizeof(u_long128), 1, fp);

	// 743 ///////////// ALPHA
	QW.ui32[0] = ALPHA_DISABLE;
	QW.ui32[1] = 0x80;
	QW.ul64[1] = 0x42;
	fwrite(&QW, sizeof(u_long128), 1, fp);

	// 744 ///////////// TEXFLUSH
	QW.ul64[0] = 0;		QW.ul64[1] = 0x3F;
	fwrite(&QW, sizeof(u_long128), 1, fp);

	// 745 ///////////// TEXA
	QW.ui32[0] = 0;
	QW.ui32[1] = 0x80;
	QW.ul64[1] = 0x3B;
	fwrite(&QW, sizeof(u_long128), 1, fp);

	// 746 ///////////// TEX1
	QW.ul64[0] = 1;	QW.ui32[2] = 0x14;	QW.ui32[3] = 0;
	fwrite(&QW, sizeof(u_long128), 1, fp);

	// 747 ///////////// RGBAUV1
	QW.f32[0] = 8; QW.f32[1] = 8; QW.ul64[1] = 0;
	fwrite(&QW, sizeof(u_long128), 1, fp);

	// 748 ///////////// RGBAPOS1
	QW.f32[0] = (8 << 4); QW.f32[1] = 0; QW.ul64[1] = 0;
	fwrite(&QW, sizeof(u_long128), 1, fp);

	// 749 ///////////// RGBAUV2
	QW.f32[0] = 8 + (8 << 4); QW.f32[1] = 8 + (128 << 4); QW.ul64[1] = 0;
	fwrite(&QW, sizeof(u_long128), 1, fp);

	// 750 ///////////// RGBAPOS2
	QW.f32[0] = (8 << 4) + (8 << 4); QW.f32[1] = (128 << 4); QW.ul64[1] = 0;
	fwrite(&QW, sizeof(u_long128), 1, fp);

	// 751 ///////////// RESET TEX0
	QW.ul128 = GS_SET_TEX0(0x118 * 32, 2, 0, 7, 7, 1, 0, 0, 0, 0, 0, 0);
	fwrite(&QW, sizeof(u_long128), 1, fp);

	// 752 ///////////// Texgen Script
	/*TexGenAlphaMode(ALPHA_ONE, 0x80);
	TexGenRadial(3);
	TexGenCopy(0, 3, 255 / 2, 255 / 2, 200 / 2);
	TexGenPlasma(1, 4, 4, 255, 255, 255);
	TexGenFill(2, 128, 128, 128, 128);
	TexGenSinDistort(3, 1, 8, 5, 8, 5);
	TexGenNoise(10, 10, 10, 10);
	TexGenAlphaMode(ALPHA_ADD, 0x80);
	TexGenCopy(3, 10);
	TexGenAlphaMode(ALPHA_ONE, 0x80);
	TexGenCopy(1, 3, 200 / 2, 200 / 2, 255 / 2);*/

	TexGenRadial(1);
	TexGenCopy(0, 1, 255 / 2, 255 / 2, 255 / 2);

	TexGenRadial(3);
	TexGenSinDistort(2, 3, 2, 69 / 2, 2, 78 / 2);
	TexGenSinDistort(3, 2, 2, 69 / 2, 2, 78 / 2);

	TexGenCopy(2, 3, 200 / 2, 200 / 2, 255 / 2);

	TexGenPlasma(1, 4, 4, 255, 255, 255);
	TexGenSinDistort(4, 1, 8, 5, 8, 5);
	TexGenNoise(10, 10, 10, 10);
	TexGenAlphaMode(ALPHA_ADD, 0x80);
	TexGenCopy(4, 10);
	TexGenAlphaMode(ALPHA_ONE, 0x80);
	TexGenCopy(1, 4, 200 / 2, 200 / 2, 255 / 2);

	TexGenAlphaMode(ALPHA_ONE, 0x80);

	TexGenRadial(5);
	TexGenSinDistort(6, 5, 2, 100 / 2, 2, 78 / 2);

	TexGenFill(7, 255, 255, 255, 128);
	TexGenAlphaMode(ALPHA_SUBTRACT, 0x80);
	TexGenCopy(7, 6);

	TexGenAlphaMode(ALPHA_ONE, 0x80);

	TexGenFill(8, 0, 0, 0, 128);
	TexGenRGtoBA(8, 6);

	TexGenNoise(10, 200, 200, 255);

	TexGenAlphaMode(ALPHA_NORMAL, 0x80);

	TexGenPingPongBlur(10, 11, 100);

	TexGenAlphaMode(ALPHA_MUL, 0x80);
	TexGenCopy(10, 8);

	TexGenAlphaMode(ALPHA_ADD, 0x80);
	TexGenCopy(10, 2);

	TexGenSinDistort(11, 10, 2, 100 / 2, 2, 78 / 2);

	TexGenAlphaMode(ALPHA_ONE, 0x80);

	TexGenRadial(13);
	TexGenCopy(12, 13, 128, 128, 128, 255);


	TexGenEnd();
	printf("Wrote %d qword(s) to %s\n", ftell(fp) >> 4, argv[1]);

	fclose(fp);

	return 0;
}