; bubble.vcl
; Fabian Roth, fin@envoy.ch 2003
; for VU2 coding contest 2003 ps2linux
;

.include "vcl_sml.i"
.include "rays.i"
.include "bubble.i"



; pad const
NOPAD 			.assign		32767		; 0x7FFF

START			.assign		8			; 0x0008
UP			.assign		16			; 0x0010
RIGHT			.assign		32			; 0x0020
DOWN			.assign		64			; 0x0040
LEFT			.assign		128			; 0x0080
TRIANGLE		.assign		4096		; 0x1000
CIRCLE			.assign		8192		; 0x2000
X			.assign		16384		; 0x4000
SQUARE			.assign		32767		; 0x7FFF
R2			.assign		512			; 0x0200
R1			.assign		2048		; 0x0800

LastQW			.assign		1023		; end of memory



.init_vf_all
.init_vi_all
.syntax new

.vu

--enter
--endenter

START:

; init GS: clear zbuffer

	; disable z-test
	iaddiu		zbufTag,vi00,ZBufTag
	lq		test,2(zbufTag)
	loi		0x00030000
	max.x		test,vf00,i
	sq.x		test,2(zbufTag)
	
--barrier
	xgkick		zbufTag
--barrier

	iaddiu		nopTag, vi00, NopTag
	xgkick		nopTag

	iaddiu		clrTag,vi00,ClrTag
	
	loi		2048.0
	add.xy		cent,vf00,i
	
	sub.zw		tl,vf00,vf00
	sub.zw		br,vf00,vf00

	loi		320.0
	max.x		side,vf00,I
	loi		112.0
	max.y		side,vf00,I

	sub.xy		tl,cent,side
	add.xy		br,cent,side

	ftoi4		tl,tl
	ftoi4		br,br

	sq		tl,2(clrTag)
	sq		br,3(clrTag)

--barrier
	xgkick		clrTag
--barrier
	iaddiu		nopTag, vi00, NopTag
	xgkick		nopTag

	; enable z-test
	loi		0x00070000 	; GREATER than in Zbuf
	max.x		test,vf00,i
	sq.x		test,2(zbufTag)
--barrier
	xgkick		zbufTag
--barrier
	iaddiu		nopTag, vi00, NopTag
	xgkick		nopTag


	; initalize textures
	iaddiu		texinit, vi00, TexInitTag
--barrier
	xgkick		texinit
--barrier

	iaddiu		texinit, vi00, TexImage
--barrier
	xgkick		texinit
--barrier

	iaddiu		texinit, vi00, SetTexTag
--barrier
	xgkick		texinit
--barrier

	

	
; read specs

	lq.xyzw		fconstants,aconsts(vi00)	; constants (tau, f, beta)

	MatrixLoad	s1, ascrmat, vi00
	iaddiu		edgecount,vi00,nedges		; # of edges

	iaddiu		startsurfs,vi00,startgeom	; surfs in startsurfs
	iaddiu  	startedges,startsurfs,nsurfs	; add 2*nsurfs:
	iaddiu  	startedges,startedges,nsurfs	; edges in startedges
	iaddiu		startverts,startedges,nedges	; verts in startverts

	iaddiu		startvertnormals,startverts,nverts
	iaddiu		startvels, startvertnormals, nverts
	iaddiu		startaccels, startvels, nverts

	iaddiu 		edgeptr, startedges, 0		; prepare pointer to edges

	ilw.z		random, LastQW(vi00)


; PAD
	ilw.w		pad, LastQW(vi00)				

	; if R1 is pressed, we don't need to do any calculations!
	iaddiu		start,vi00,START
	iand		input,pad,start
	ibeq		input,vi00,freeze 	; head of to freeze


forceloop:
	ilwr.x		vert1, (edgeptr)x		; load verts into vert1 and vert
	ilwr.y		vert2, (edgeptr)y		;
	
	iadd		vert1ptr, startverts, vert1
	iadd 		vert2ptr, startverts, vert2

	lq.xyz		coord1,(vert1ptr)		; load coord of vert1
	lq.xyz		coord2,(vert2ptr)		; load coord of vert2
	
	sub.xyz		diff, coord1, coord2		; diff of coords

	; edge	connects two nodes, force acts on both nodes, actio == reactio
	; force is proportional to distance with spring const fconstants.y.

	mul.xyz		diff, diff, fconstants[y]	; multiply by f

	iadd		paccel1, startaccels, vert1	; 
	iadd		paccel2, startaccels, vert2

	lq.xyzw		accel1, (paccel1)		; load accels
	lq.xyzw		accel2, (paccel2)		; 

	sub.xyz		accel1, accel1, diff		; add force 
	add.xyz		accel2, accel2, diff

	add.w		accel1, accel1, vf00		; increment count of forces
	add.w		accel2, accel2, vf00		; increment count of forces

	sq.xyzw		accel1, (paccel1)		; store
	sq.xyzw		accel2, (paccel2)		; store


	iaddiu		edgeptr,edgeptr,1		; advance edgeptr
	ibne		edgeptr,startverts,forceloop	; repeat until verts data starts



; at this point the forces have been calculated. now we need to integrate the accels
; and velocities. to do that, we loop through all vertices...
; and while looping through all the verts, might as well calculate center of gravity

	iaddiu	vert, vi00, nverts			; this will be the index into the verts
eachvert:
	isubiu		vert,vert,1
	iadd		paccel, startaccels, vert	; pointer into accel
	lq.xyzw		int,(paccel)			; get force
	sub.xyzw	zero,vf00,vf00			; clear accel
	sq.xyzw		zero,(paccel)			; reset force to zero for next frame

	div		q, vf00[w],int[w]		; divide forces by number of forces
	mulq.xyz	int,int,q			; (edges) at this vertex


	iadd		pvertnormal, startvertnormals, vert
	lq.xyz		vertnormal,(pvertnormal)	; read out vertex normal

	VectorNormalizeXYZ norm, vertnormal
	sq.xyz		vf00,(pvertnormal)

; PRESSURE
;	iadd		pvert, startverts, vert
;	lq.xyz		pos,(pvert)
;	VectorNormalize norm, pos
	mul.xyz		press,norm,fconstants[w]	; multiply normal by pressure constant
	add.xyz		int,int,press			; add pressure
; END PRESSURE

; ORIGIN
;	iadd		pvert, startverts, vert
;	lq.xyz		pos,(pvert)
;	loi		0.002	
;	muli.xyz	originf,pos,i
;	sub.xyz		int,int,originf
; END ORIGiN

; GRAVITATION
;	loi		0.1
;	sub.y		int,int,i
; END GRAVITATION

; FRICTION
; here we add friction
	iadd		pvel, startvels, vert
	lq.xyz		vel,(pvel)
	mul.xyz		vel,vel,fconstants[z]		; friction
	sub.xyz		int,int,vel	
; END FRICTION

; RANDOM MOVEMENTS
	iaddiu		rvert, vi00, H'fff		; with little probability jolt bubble
	iand		rvert, rvert, random
	ibne		rvert, vert, nojolt
	loi		6.0
	mul.xyz		fluct,press,i
	add.xyz		int,int,fluct		
	b 	randcont
nojolt:
	iaddiu		rvert, vi00, H'ff		; with higher probability wiggle a little bit
	iand		rvert, rvert, random
	ibne		rvert, vert, randcont
	add.xyz		int,int,press	
	add.xyz		int,int,press	

randcont:


;	b cont

	ibne		vert,vi00,sdone			; we're only pulling at vertex 0
	iaddiu		r1,vi00,R1

	
	loi 		0.6
	iaddiu		xbutton,vi00,X
	iand		input,pad,xbutton
	ibne		input,vi00,xdone
	iand		input,pad,r1			; check whether R1 is pushed
	ibne		input,vi00,nor1x		
	sub.z		int,int,i			; if so, pull along the z-axis
	b xdone:	
nor1x:
	sub.y		int,int,i

xdone:
	iaddiu		sbutton,vi00,CIRCLE
	iand		input,pad,sbutton
	ibne		input,vi00,cdone
	add.x		int,int,i
;	iaddiu		svert, vi00, 1
;	ibne		vert,svert, cdone
;	add.xyz		int,int,press
cdone:
	iaddiu		tbutton,vi00,TRIANGLE
	iand		input,pad,tbutton
	ibne		input,vi00,tdone
	iand		input,pad,r1
	ibne		input,vi00,nor1t
	add.z		int,int,i
	b tdone:	
nor1t:
	add.y		int,int,i
;	iaddiu		tvert, vi00, 2
;	ibne		vert,tvert, tdone
;	add.xyz		int,int,press
tdone:
	iaddiu		sqbutton,vi00,SQUARE
	iaddiu		sqbutton,sqbutton,1
	iand		input,pad,sqbutton
	ibne		input,vi00,sdone
	sub.x		int,int,i

sdone:
	
; END PAD
	
	mul.xyz		int,int,fconstants[x]		; integrate with tau
	
	iadd		pvel, startvels, vert		; into vels
	lq.xyz		vel,(pvel)			; v load
	add.xyz		int,vel,int			; v = v + dt*a
	sq.xyz		int,(pvel)			; v store
	
	mul.xyz		int,int,fconstants[x]		; integrate with tau, int = dt*v

	iadd		pvert, startverts, vert		; pointer into verts
	lq.xyz		coord,(pvert)			; x load
	add.xyz		int,coord,int			; x = x + dt*v

;	iaddiu		sqbutton,vi00,SQUARE
;	iaddiu		sqbutton,sqbutton,1
;	iand		input,pad,sqbutton
;	ibne		input,vi00,cont
;	iaddiu		sqvert, vi00, 3
;	ibne		vert,sqvert, cont
;	loi		2.0
;	add.y		int, vf00, i
;	add.xyz		int,int,press
;	sub.xyz		int,vf00,vf00

cont:


	; new coord calculated, now store it back
	sq.xyz		int,(pvert)			; x store
	ibne		vert,vi00,eachvert

freeze:

	; calculate face normals

	iaddiu		surfptr, startsurfs, 0
	iaddiu		facecount,vi00,nsurfs
eachface:
	ilwr.x		vert1, (surfptr)x
	ilwr.y		vert2, (surfptr)y
	ilwr.z		vert3, (surfptr)z
	
	iadd		vert1ptr, startverts, vert1
	iadd 		vert2ptr, startverts, vert2
	iadd 		vert3ptr, startverts, vert3

	lq.xyz		base,(vert1ptr)
	lq.xyz		vec1,(vert2ptr)
	lq.xyz		vec2,(vert3ptr)
	
	sub.xyz		vec1,vec1,base	
	sub.xyz		vec2,vec2,base	
	
	VectorCrossProduct	cross,vec1,vec2	
	VectorNormalize		norm,cross		; face normal of this face

	sq.xyz		norm,nsurfs(surfptr)		; save it in the face normal

	; now load vertex normals of verts, this face is connected to, and add this face normal
	lq.xyz		vnorm,nverts(vert1ptr)
	add.xyz		vnorm,vnorm,norm
	sq.xyz		vnorm,nverts(vert1ptr)
	lq.xyz		vnorm,nverts(vert2ptr)
	add.xyz		vnorm,vnorm,norm
	sq.xyz		vnorm,nverts(vert2ptr)
	lq.xyz		vnorm,nverts(vert3ptr)
	add.xyz		vnorm,vnorm,norm
	sq.xyz		vnorm,nverts(vert3ptr)
	

	iaddiu		surfptr,surfptr,1
	isubiu		facecount,facecount,1
	ibne		facecount,vi00,eachface




; rotate camera
	; angle.x = theta
	; angle.y = \dot{theta}	
	; angle.z = phi (eventually)
	; angle.w = \dot{phi}

	lq.xyzw		camconsts, acamconsts(vi00)

	sub.xz		cam,vf00,vf00


	; check for pad
	ilw.w		pad, LastQW(vi00)
	iaddiu		key,vi00,LEFT
	iand		input,pad,key
	ibne		input,vi00,right
	add.x		cam,cam,camconsts[w]
right:
	iaddiu		key,vi00,RIGHT
	iand		input,pad,key
	ibne 		input,vi00, up
	sub.x		cam,cam,camconsts[w]
up:
	iaddiu		key,vi00,UP
	iand		input,pad,key
	ibne 		input,vi00, down
	add.z		cam,cam,camconsts[w]
down:
	iaddiu		key,vi00,DOWN
	iand		input,pad,key
	ibne 		input,vi00, camcont
	sub.z		cam,cam,camconsts[w]
camcont:

	lq.xyzw		angle, aangles(vi00)
	
	sub.xz		accel,vf00,angle		; spring 
	mul.xz		accel, accel, camconsts[y]	
	add.xz		accel,accel,cam			; add attractor
	mr32.xyzw	fric,angle
	mul.xz		fric,fric,camconsts[z]		; multiply by beta (friction)
	sub.xz		accel,accel,fric		; add friction
	mul.xz		accel,accel,camconsts[x]
	add.y		angle,angle,accel[x]
	add.w		angle,angle,accel[z]

	mr32		vel,angle
	mul.xz		vel,vel,camconsts[x]	
	add.xz		angle, angle, vel
	sq.xyzw		angle, aangles(vi00)
	


	sq.xy		angle,aangles(vi00)

	AngleSinCos	angle,sincos		; sin(angle.x) in sincos.x and cos(angle.x) in sincos.y


	MatrixIdentity	rottheta

	add.y		rottheta[1],vf00,sincos[w]
	add.z		rottheta[1],vf00,sincos[z]
	sub.y		rottheta[2],vf00,sincos[z]
	add.z		rottheta[2],vf00,sincos[w]
	
	MatrixIdentity	rotphi

	add.x		rotphi[0],vf00,sincos[y]
	add.z		rotphi[0],vf00,sincos[x]
	sub.x		rotphi[2],vf00,sincos[x]
	add.z		rotphi[2],vf00,sincos[y]

	loi		8
	add.z		rotphi[3],angle,i

	MatrixMultiply	rot, rottheta, rotphi


; DRAWING

; DRAW BACKWALL

	iaddiu		backWall, vi00, BackWall

	lq.xyz		color,acol(vi00)
	loi		0.4
	mul.xyz		color,color,i
	sub.w		color,vf00,vf00
	ftoi0		color,color


	lq.xyz		coord,WallCoords(vi00)
	move.w		coord,vf00

	MatrixMultiplyVertexXYZ1 camcoord1,rot,coord
	MatrixMultiplyVertexXYZ1 coord1out,s1,camcoord1
	div		q, vf00[w],coord1out[w]
	mulq.xyz	coord1, coord1out, q
	ftoi4.xyz	coordf, coord1
	lq.xyz		stq, WallTexCoords+0(vi00)
	mulq.xyz	stq, stq, q
	sq.xyz		stq,BackWall+1(vi00)
	sq.xyz		coordf,BackWall+3(vi00)

	lq.xyzw		coord,WallCoords+1(vi00)
	move.w		coord,vf00
	MatrixMultiplyVertexXYZ1 camcoord1,rot,coord
	MatrixMultiplyVertexXYZ1 coord1out,s1,camcoord1
	div		q, vf00[w],coord1out[w]
	mulq.xyz	coord1, coord1out, q
	ftoi4.xyz	coordf, coord1
	lq.xyz		stq, WallTexCoords+1(vi00)
	mulq.xyz	stq, stq, q
	sq.xyz		stq,BackWall+4(vi00)
	sq.xyz		coordf,BackWall+6(vi00)

	lq.xyzw		coord,WallCoords+2(vi00)
	move.w		coord,vf00
	MatrixMultiplyVertexXYZ1 camcoord1,rot,coord
	MatrixMultiplyVertexXYZ1 coord1out,s1,camcoord1
	div		q, vf00[w],coord1out[w]
	mulq.xyz	coord1, coord1out, q
	ftoi4.xyz	coordf, coord1
	lq.xyz		stq, WallTexCoords+2(vi00)
	mulq.xyz	stq, stq, q
	sq.xyz		stq,BackWall+7(vi00)
	sq.xyz		coordf,BackWall+9(vi00)

	lq.xyzw		coord,WallCoords+3(vi00)
	move.w		coord,vf00
	MatrixMultiplyVertexXYZ1 camcoord1,rot,coord
	MatrixMultiplyVertexXYZ1 coord1out,s1,camcoord1
	div		q, vf00[w],coord1out[w]
	mulq.xyz	coord1, coord1out, q
	ftoi4.xyz	coordf, coord1
	lq.xyz		stq, WallTexCoords+3(vi00)
	mulq.xyz	stq, stq, q
	sq.xyz		stq,BackWall+10(vi00)
	sq.xyz		coordf,BackWall+12(vi00)

	sq		color,BackWall+2(vi00)
	sq		color,BackWall+5(vi00)
	sq		color,BackWall+8(vi00)
	sq		color,BackWall+11(vi00)
	xgkick 		backWall

	iaddiu		nopTag, vi00, NopTag
	xgkick		nopTag


; DRAW BUBBLE

	iaddiu		faceptr, startsurfs, 0
	iaddiu		facecount, vi00, nsurfs

drawface:

	ilwr.x		vert1, (faceptr)x
	ilwr.y		vert2, (faceptr)y
	ilwr.z		vert3, (faceptr)z

	iadd		vert1ptr, startverts, vert1
	iadd 		vert2ptr, startverts, vert2
	iadd 		vert3ptr, startverts, vert3
			
	lq.xyz		coord1,(vert1ptr);		; load coord of vert1
	move.w		coord1,vf00
	lq.xyz		coord2,(vert2ptr);		; load coord of vert2
	move.w		coord2,vf00
	lq.xyz		coord3,(vert3ptr);		; load coord of vert3
	move.w		coord3,vf00

	lq.xyz		normal,nsurfs(faceptr)

	MatrixMultiplyVector  scrnormal,rot,normal

	lq		refrconsts, arefract(vi00)


	; draw shadow of bubble


	sub		color,vf00,vf00
	loi		100.0
	add.w		color,color,i

	lq.xyz		light, alight(vi00)

	lq		shadowprim, atags(vi00)		; PRIM for solid/wireframe shadow at zw
	mr32.yzw	shadowprim,shadowprim		; PRIM for solid now in shadowprim[y]

	iaddiu		r2,vi00,R2
	iand		input,pad,r2
	ibne		input,vi00,contshadow

	mr32.yz		shadowprim,shadowprim		; PRIM for wireframe now in shadorpim[y]

contshadow:

	; disable z-test (shadow on the wall -- wall has been drawn already)
	iaddiu		zbufTag,vi00,ZBufTag
	lq		test,2(zbufTag)
	loi		0x00030000
	max.x		test,vf00,i
	sq.x		test,2(zbufTag)
	
	xgkick		zbufTag


	add.z		dist,vf00,refrconsts[y]


	shadow		shad, coord1, light, dist
	MatrixMultiplyVertexXYZ1 camshad1,rot,shad
	MatrixMultiplyVertexXYZ1 shad1,s1,camshad1
	div		q, vf00[w],shad1[w]
	mulq.xyz	scrshad1, shad1, q
	ftoi4.xyz	scrshad1, scrshad1

	shadow		shad, coord2, light, dist
	MatrixMultiplyVertexXYZ1 camshad2,rot,shad
	MatrixMultiplyVertexXYZ1 shad2,s1,camshad2
	div		q, vf00[w],shad2[w]
	mulq.xyz	scrshad2, shad2, q
	ftoi4.xyz	scrshad2, scrshad2

	shadow		shad, coord3, light, dist
	MatrixMultiplyVertexXYZ1 camshad3,rot,shad
	MatrixMultiplyVertexXYZ1 shad3,s1,camshad3
	div		q, vf00[w],shad3[w]
	mulq.xyz	scrshad3, shad3, q
	ftoi4.xyz	scrshad3, scrshad3

	sq.y		shadowprim, triangGifpacket(vi00)
	
	ftoi0		color,color

	sq		color,triangGifpacket+2(vi00)
	sq		color,triangGifpacket+5(vi00)
	sq		color,triangGifpacket+8(vi00)

	sq.xyz		scrshad1,triangGifpacket+3(vi00)
	sq.xyz		scrshad2,triangGifpacket+6(vi00)
	sq.xyz		scrshad3,triangGifpacket+9(vi00)

	

	iaddiu 		iKick, vi00,triangGifpacket 		; fill packet 
	xgkick 		iKick				; kick off
	iaddiu		nopTag, vi00, NopTag
	xgkick		nopTag

	; enable z-test
	loi		0x00070000 	; GREATER than in Zbuf
	max.x		test,vf00,i
	sq.x		test,2(zbufTag)
--barrier
	xgkick		zbufTag
--barrier
	iaddiu		nopTag, vi00, NopTag
	xgkick		nopTag



;	drawing of bubble

	; test for R2 and conditional branch to wireframe

	iaddiu		r2,vi00,R2
	iand		input,pad,r2
	ibeq		input,vi00,wireframe

	lq.xyzw		color,acol(vi00)
	lq.xyz		light, alight(vi00)
	sub.w		color1,vf00,vf00

	lq.xyz		vert1normal,nverts(vert1ptr);
	VectorNormalizeXYZ v1normal,vert1normal
	VectorDotProduct	dot,v1normal,light
	loi		0.2				; lowerlimit intensity 
	maxi.x		dot, dot, I
	mini.x		dot, dot, vf00[w]
	mul.xyzw	color1,color,dot[x]		; scale color
	ftoi0		color1,color1
	sq		color1,triangGifpacket+2(vi00)

	lq.xyz		vert2normal,nverts(vert2ptr);
	VectorNormalizeXYZ v2normal,vert2normal
	VectorDotProduct	dot,v2normal,light
	maxi.x		dot, dot, I
	mini.x		dot, dot, vf00[w]
	mul.xyzw	color1,color,dot[x]		; scale color
	ftoi0		color1,color1
	sq		color1,triangGifpacket+5(vi00)

	lq.xyz		vert3normal,nverts(vert3ptr);
	VectorNormalizeXYZ v3normal,vert3normal
	VectorDotProduct	dot,v3normal,light
	maxi.x		dot, dot, I	
	mini.x		dot, dot, vf00[w]
	mul.xyzw	color1,color,dot[x]		; scale color
	ftoi0		color1,color1
	sq		color1,triangGifpacket+8(vi00)

	lq.x		gif,atags(vi00)			; load PRIM for texture mapped triangles
	add.y		gif,vf00,gif[x]

	b 		draw
	
wireframe:

		
	lq.xyz		color,acol(vi00)
	sub.w		color,vf00,vf00
	lq.xyz		light, alight(vi00)
	VectorDotProduct	dot,normal,light
	loi		0.1
	maxi.x		dot, dot, I

	mul.xyz		color,color,dot[x]		; scale color
	ftoi0		color,color
	sq		color,triangGifpacket+2(vi00)
	sq		color,triangGifpacket+5(vi00)
	sq		color,triangGifpacket+8(vi00)

	lq.y		gif,atags(vi00)			; load PRIM for flat shaded linestrip

draw:

	sq.y		gif,triangGifpacket(vi00)

	; take care of refraction
	
	; create (0,0,-1) vector for viewing direction
	add.z		tincident,vf00,vf00[w]
	sub.xy		tincident,vf00,vf00
	move.w		tincident,vf00

	; rotate it
	MatrixMultiplyVertexXYZ1 incident,rot,tincident


	lq.xyz		vert1normal,nverts(vert1ptr);
	VectorNormalizeXYZ v1normal,vert1normal
	refract		refracted1,incident,v1normal,refrconsts
	mul.xy		refracted1,refracted1,refrconsts[x]
	lq.xyz		vert2normal,nverts(vert2ptr);
	VectorNormalizeXYZ v2normal,vert2normal
	refract		refracted2,incident,v2normal,refrconsts
	mul.xy		refracted2,refracted2,refrconsts[x]
	lq.xyz		vert3normal,nverts(vert3ptr);
	VectorNormalizeXYZ v3normal,vert3normal
	refract		refracted3,incident,v3normal,refrconsts
	mul.xy		refracted3,refracted3,refrconsts[x]


	MatrixMultiplyVertexXYZ1 camcoord1,rot,coord1
	MatrixMultiplyVertexXYZ1 coord1out,s1,camcoord1
	div		q, vf00[w],coord1out[w]
	mulq.xyz	scrcoord1, coord1out, q
	ftoi4.xyz	scrcoord1, scrcoord1

	MatrixMultiplyVertexXYZ1 camcoord2,rot,coord2
	MatrixMultiplyVertexXYZ1 coord2out,s1,camcoord2
	div		q, vf00[w],coord2out[w]
	mulq.xyz	scrcoord2, coord2out, q
	ftoi4.xyz	scrcoord2, scrcoord2

	MatrixMultiplyVertexXYZ1 camcoord3,rot,coord3
	MatrixMultiplyVertexXYZ1 coord3out,s1,camcoord3
	div		q, vf00[w],coord3out[w]
	mulq.xyz	scrcoord3, coord3out, q
	ftoi4.xyz	scrcoord3, scrcoord3
	
	sub.y		dist, refrconsts, coord2[z]		; calculate distance from bubble to wall
	

	iaddiu 		iKick, vi00,triangGifpacket 		; fill packet 
	iaddiu 		firstv, iKick, 0
	add.xy		refracted1,camcoord1,refracted1
	map		tex, refracted1, incident, dist
	sq.xy		tex,1(firstv)
	sq.xyz 		scrcoord1,3(firstv)
	iaddiu 		firstv, firstv, 3
	add.xy		refracted2,camcoord2,refracted2
	map		tex, refracted2, incident, dist
	sq.xy		tex,1(firstv)
	sq.xyz 		scrcoord2,3(firstv)
	iaddiu 		firstv, firstv, 3
	add.xy		refracted3,camcoord3,refracted3
	map		tex, refracted3, incident, dist
	sq.xy		tex,1(firstv)
	sq.xyz 		scrcoord3,3(firstv)

	xgkick 		iKick				; kick off
	iaddiu		nopTag, vi00, NopTag
	xgkick		nopTag

next:
	iaddiu		faceptr, faceptr,1
	isubiu		facecount, facecount, 1
	ibne		facecount, vi00, drawface



--exit
--endexit


