
#include <stdio.h>
#include "defines.h"
#include "packet.h"
#include "geommath.h"
#include <math.h>

#define NUM_VERTS 1


void printMatrix(const Mat44& mat)
{
  printf("[%8f %8f %8f %8f]\n", mat.m_fMat[0], mat.m_fMat[1], mat.m_fMat[2], mat.m_fMat[3]);
  printf("[%8f %8f %8f %8f]\n", mat.m_fMat[4], mat.m_fMat[5], mat.m_fMat[6], mat.m_fMat[7]);
  printf("[%8f %8f %8f %8f]\n", mat.m_fMat[8], mat.m_fMat[9], mat.m_fMat[10], mat.m_fMat[11]);
  printf("[%8f %8f %8f %8f]\n", mat.m_fMat[12], mat.m_fMat[13], mat.m_fMat[14], mat.m_fMat[15]);
}

Mat44 makePerspectiveMatrix()
{
	Mat44 perspective;

	float n = 0.1f;
	float f = 1000.0f;
	float l = -0.06f * (2048.0f / (640.0f * 0.5f));
	float r =  0.06f * (2048.0f / (640.0f * 0.5f));
	float t =  0.06f * (2048.0f / (350.0f * 0.5f));
	float b = -0.06f * (2048.0f / (350.0f * 0.5f));

	LoadIdentity(&perspective);
	SetRow(&perspective,0,Vec4((2*n) / (r-l)	, 0.0f				, 0.0f				, 0.0f));
	SetRow(&perspective,1,Vec4(0.0f				, -(2*n) / (t-b)	, 0.0f				, 0.0f));
	SetRow(&perspective,2,Vec4(((r+l) / (r-l))	, ((t+b) / (t-b))	, ((f+n) / (f-n))	, -1.0f));
	SetRow(&perspective,3,Vec4(0.0f				, 0.0f				, ((2*f*n) / (f-n))	, 0.0f));
//	printMatrix(perspective);
/* was originally making my own perspective matrix, but for now the default will work
	float zNear = 1.0f;
	float zFar = 8000.0f;
	float fovY = 45.0f * (3.1415926535 / 180.0f);
	float fd = 1 / tan(fovY / 2);
	float a = 1.0f; 

	SetColumn(&perspective, 0, Vec4( (fd / a), 0.0f, 0.0f,                            0.0f )); 
	SetColumn(&perspective, 1, Vec4( 0.0f,     fd,   0.0f,                            0.0f )); 
	SetColumn(&perspective, 2, Vec4( 0.0f,     0.0f, (zNear + zFar) / (zNear - zFar), (2 * zNear * zFar) / (zNear - zFar) )); 
	SetColumn(&perspective, 3, Vec4( 0.0f,     0.0f, -1.0f,                           0.0f )); 
	printMatrix(perspective);
*/
	return (perspective);
}

void printEndInfo(FILE* output, char* name, gifpacket_t* packet)
{
  fprintf(output, "%s .assign %u\n", name, gifpacket_size(packet) - 1);
}

void printInfo(FILE* output, char* name, gifpacket_t* packet)
{
  fprintf(output, "%s .assign %u\n", name, gifpacket_size(packet));
}

int main(int argc, char **argv)
{
	FILE *output;
	FILE *infoFile;
	int bytes=0;
	int prim=0;
	qword_t colour;
	qword_t scales;
	qword_t reg;
	int i,j;

	u_long128 packet_data[1024];

	gifpacket_t gifpacket;

	for (i=0; i<1024; i++)
	{
		packet_data[i] = 0;
	}

	if (argc!=3)
	{
		fprintf(stderr, "Usage: gen_data output_file info_file\n");
		return -1;
	}

	infoFile=fopen(argv[2], "wb");

	if (infoFile==NULL)
	{
		fprintf(stderr, "Could not open file \"%s\"\n", argv[2]);
		return -1;
	}

	gifpacket_init(&gifpacket, packet_data);

	/*
	printInfo(infoFile, "kAlphaTag", &gifpacket);
        gifpacket_addgsdata(&gifpacket, GIF_SET_TAG(NLOOP(1), EOP(1), PRE(0), 0, GIF_PACKED, NREG(1)));
        gifpacket_addgsdata(&gifpacket, 0x0e);
        // setup alpha blending
        reg.ul64[0] = GS_SET_ALPHA_1(0, 1, 0, 1, 0);
        reg.ul64[1] = ALPHA_1_REG;
        gifpacket_addgspacked(&gifpacket, reg.ul128);
	*/

	printInfo(infoFile, "kPointTag", &gifpacket);
	prim=GS_SET_PRIM(GS_PRIM_POINT, GOURAUD(1), TEXTURED(0), FOG(0), ABE(0), AA(0), FST(0), CTXT(0), FIX(0));
	gifpacket_addgsdata(&gifpacket, GIF_SET_TAG(NLOOP(NUM_VERTS), EOP(1), PRE(1), prim, GIF_PACKED, NREG(2)));
	gifpacket_addgsdata(&gifpacket, 0x51);

	printInfo(infoFile, "kPointStart", &gifpacket);
	for (i=0; i<NUM_VERTS; i++)
	{
		colour.ui32[0] = 255;
		colour.ui32[1] = 255;
		colour.ui32[2] = 255;
		colour.ui32[3] = 0x80;
		gifpacket_addgspacked(&gifpacket, colour.ul128);
		gifpacket_addgspacked(&gifpacket, 0); // Leave a gap for transformed vertex co-ords
	}
	printEndInfo(infoFile, "kPointEnd", &gifpacket);

	Mat44 objectToWorld;
	LoadIdentity(&objectToWorld);
	printInfo(infoFile, "kObjectToWorld", &gifpacket);
	gifpacket_addgsmat44(&gifpacket, &objectToWorld);

	Mat44 worldToCamera;
	LoadIdentity(&worldToCamera);
	printInfo(infoFile, "kWorldToCamera", &gifpacket);
	gifpacket_addgsmat44(&gifpacket, &worldToCamera);

	Mat44 cameraToScreen = makePerspectiveMatrix();
	printInfo(infoFile, "kCameraToScreen", &gifpacket);
	gifpacket_addgsmat44(&gifpacket, &cameraToScreen);

	scales.f32[0] = 2048.0f;
	scales.f32[1] = 2048.0f;
	scales.f32[2] = (float)((2<<24)-1)/2;
	scales.f32[3] = 1.0f;
	printInfo(infoFile, "kScales", &gifpacket);
	gifpacket_addgspacked(&gifpacket,scales.ul128);

	// this is where we will store info about the particles
	printInfo(infoFile, "kParticleStart", &gifpacket);

	gifpacket_terminate(&gifpacket);

	//close info file
	fclose(infoFile);

	output=fopen(argv[1], "wb");

	if (output==NULL)
	{
		fprintf(stderr, "Could not open file \"%s\"\n", argv[1]);
		return -1;
	}

	fwrite(packet_data, sizeof(packet_data)/sizeof(qword_t), sizeof(qword_t), output);

	bytes=ftell(output);
	fclose(output);

	printf("Written %d bytes to \"%s\"\n", bytes, argv[1]);

	return 0;
}
