;------------------------------------------------------------------------------
; Demo by vostorus  	(http://vostorus.home.mchsi.com)
;
;	Thanks to:
;	- Eratosthenes for the tutorials
;	- Stewart for random number generator (found from Kirk Benders Fluff)
;	- Everyone on the PS2 forums
;------------------------------------------------------------------------------

.init_vf_all
.init_vi_all
.syntax new

.vu

kPointTag .assign 0
kPointStart .assign 1
kPointEnd .assign 2

kObjectToWorld .assign 3
kWorldToCamera .assign 7
kCameraToScreen .assign 11
kScales .assign 15

kParticleStart .assign 16		; This is the starting point of where
					; all particle info is stored

kCameraZoom .assign 1021
kInitDone .assign 1022
kLastQWord .assign 1023

kSquare .assign H'7FFF
kCross .assign H'4000
kTriangle .assign H'1000
kCircle .assign H'2000
kR1 .assign H'0800
kR2 .assign H'0200
kStart .assign H'0008
kLeft .assign H'0080
kDown .assign H'0040
kUp .assign H'0010
kRight .assign H'0020

kNUM_PARTICLES .assign 500
;------------------------------------------------------------------------------


	.macro	loixyzw	reg,xval,yval,zval,wval
	loi		\xval
	addi.x		\reg,vf00,i

	loi		\yval
	addi.y		\reg,vf00,i

	loi		\zval
	addi.z		\reg,vf00,i

	loi		\wval
	muli.w		\reg,vf00,i
	.endm


;------------------------------------------------------------------------------


	.include "vcl_sml.i"

--enter
--endenter


;------------------------------------------------------------------------------
START:
;------------------------------------------------------------------------------
	; convert fLastQWord to 0..1 range (and Z has the random number generated 
	;   by the harness program)
	VectorLoad fLastQWord, kLastQWord, vi00
	ftoi0 iLastQWord, fLastQWord
	itof0 iLastQWord, iLastQWord
	sub fLastQWord, fLastQWord, iLastQWord

	ilw.x iInitDone, kInitDone(vi00)
	ibeq iInitDone, vi00, INITIALIZATION:
	
NO_INIT_NEEDED:
	VectorLoad fScales, kScales, vi00

	MatrixLoad mObjectToWorld, kObjectToWorld, vi00
	MatrixLoad mWorldToCamera, kWorldToCamera, vi00	
	MatrixLoad mCameraToScreen, kCameraToScreen, vi00

	; create fixed rotation matrix for x axis (15 degrees)
	MatrixIdentity mRotate
	loi 0.9659258
	muli.y mRotate[1], mRotate[1], I			; cos into 1,1
	muli.z mRotate[2], mRotate[2], I			; cos into 2,2
	loi 0.2588190
	addi.z mRotate[1], mRotate[1], I			; sin into 1,2
	subi.y mRotate[2], mRotate[2], I			; -sin into 2,1

	; SEE IF THE VIEWER IS ZOOMING IN/OUT
	lq.z fCameraZoom, (kCameraZoom)(vi00)
	ilw.w iPad, kLastQWord(vi00)
	iaddiu iMask, vi00, kR1
	iand iCheck, iPad, iMask
	ibeq iCheck, vi00, ZOOM_OUT:
	iaddiu iMask, vi00, kR2
	iand iCheck, iPad, iMask
	ibeq iCheck, vi00, ZOOM_IN:
	b ZOOM_DONE:
ZOOM_OUT:
	loi 5.0
	sub.z fCameraZoom, fCameraZoom, i 
	sq.z fCameraZoom, (kCameraZoom)(vi00)
	b ZOOM_DONE:
ZOOM_IN:
	loi 5.0
	add.z fCameraZoom, fCameraZoom, i 
	sq.z fCameraZoom, (kCameraZoom)(vi00)
ZOOM_DONE:

	; translate the world by the zoom amount
	add.z mRotate[3], vf00, fCameraZoom

	; modify world to camera by rotating it
	MatrixMultiply mWorldToCamera, mWorldToCamera, mRotate

	; CREATE OBJECT TO SCREEN MATRIX
	; (Object to World * World to Camera * Camera to Screen)
	MatrixMultiply mObjectToScreen, mObjectToWorld, mWorldToCamera
	MatrixMultiply mObjectToScreen, mObjectToScreen, mCameraToScreen
	
	; keep a count of number of particles finished with phase 2
	iaddiu iPhase2FinishCount, vi00, 0

	; loop through transforming each model vertex, storing transformed verts into GIFtag.
	iaddiu iParticlePointer, vi00, kParticleStart
	iaddiu iStoreTo, vi00, kPointStart

	; setup the loop counter
	iaddiu iParticleIndex, vi00, kNUM_PARTICLES

;-------------------------------------------------------------------------------
; Loop done for each particle
PARTICLE_LOOP:
	; load the previous position
	VertexLoad vVert, 0, iParticlePointer
	VertexLoad vDestVert, 1, iParticlePointer


	; determine which phase we are on
	ftoi0 vPhaseVert, vVert
	mtir iPhase, vPhaseVert[w]

	iaddiu iCheckPhase, vi00, 2
	ibeq iPhase, iCheckPhase, PHASE_2:

; code handles both phase 0 and phase 1 since they are just moving from source
;   to destination location.  Once the reach destination, then the code is slightly
;   different to setup the particle for the next phase
PHASE_0_1:
	; determine vector direction to destination
	sub vDir, vDestVert, vVert

	; determine length
	eleng P, vDir
	mfp.w vLength, P
	; convert floating value to integer
	ftoi0.w vLength, vLength
	mtir iLength, vLength[w]

	; determine if we are close to the destination
	isubiu iLength, iLength, 2	
	iblez iLength, PHASE_0_1_COMPLETE:

	; particle is not within destination range, so move it
	VectorNormalize vDir, vDir

	; move towards the destination
	loi 1.5
	mul.xyz vDir, vDir, I
	add.xyz vVert, vVert, vDir
	; save the new position
	VertexSave vVert, 0, iParticlePointer

	b RENDER_PARTICLE:

PHASE_0_1_COMPLETE:
	ibgtz iPhase, INIT_FOR_PHASE_2:

INIT_FOR_PHASE_1:
	; set the y component to zero for the destination
	add.y vDestVert, vf00, vf00
	b INIT_FOR_PHASE_1_2_COMPLETE:

INIT_FOR_PHASE_2:
	; determine R (and place in z of vVert)
	eleng P, vVert
	mfp.z vVert, P
	; clear other fields
	add.xy vVert, vf00, vf00
	; place random number into x,y (dr, dtheta)
	add.xy vVert, vVert, fLastQWord[z]
	; generate random dr (0.30..1.30)
	loi 0.30
	addi.x vVert, vVert, I
	; generate random dtheta (0..0.0000002)
	loi 0.0000001
	muli.y vVert, vVert, I
	addi.y vVert, vVert, I
	; generate random end r (0..5)
	add.x vDestVert, vf00, fLastQWord[z]
	loi 5.0
	muli.x vDestVert, vDestVert, I

INIT_FOR_PHASE_1_2_COMPLETE:
	; move to the next phase
	add.w vVert, vVert, vf00

	; save the new position (and phase)
	VertexSave vVert, 0, iParticlePointer
	; save destination vertex
	VertexSave vDestVert, 1, iParticlePointer

	b RENDER_PARTICLE:


PHASE_2:
	; determine if r < endr
	ftoi0 vTempVert, vVert
	mtir iCurrentR, vTempVert[z]
	ftoi0 vTempVert, vDestVert
	mtir iEndR, vTempVert[x]
	isub iRDiff, iCurrentR, iEndR
	iblez iRDiff, PHASE_2_COMPLETE:
	
	; r -= dr
	sub.z vVert, vVert, vVert[x] 
	; theta -= dtheta
	sub.w vDestVert, vDestVert, vVert[y]

	; save the new information (r and theta in particular)
	VertexSave vVert, 0, iParticlePointer
	VertexSave vDestVert, 1, iParticlePointer
	b PHASE_2_CONVERT_TO_CARTESIAN:

PHASE_2_COMPLETE:
	; bump the count that are finished with phase 2
	iaddiu iPhase2FinishCount, iPhase2FinishCount, 1

PHASE_2_CONVERT_TO_CARTESIAN:
	; calculate x,z coords
	add vTheta, vf00, vf00
	add.x vTheta, vTheta, vDestVert[w]
	;  vCalcTheta:  x=sin(theta),y=cos(theta)
	AngleSinCos vTheta, vCalcTheta
	; clear out fields except for R (value of w will be set later)
	add.xy vVert, vf00, vf00
	; place R into X field also
	add.x vVert, vVert, vVert[z]
	; x = rcos(theta)
	mul.x vVert, vVert, vCalcTheta[y]
	; z = rsin(theta)
	mul.z vVert, vVert, vCalcTheta[x]

	b RENDER_PARTICLE:


;-------------------------------------------------------------------------------
RENDER_PARTICLE:
	; set the w coord to 1.0 for use in rendering
	sub.w vVert, vf00, vf00[x]

	; place object in screen coords
	MatrixMultiplyVertex vVert, mObjectToScreen, vVert

	; determine color based on phase
	iaddiu iCheckPhase, vi00, 2
	ibeq iPhase, iCheckPhase, DETERMINE_PHASE_2_COLOR:

DETERMINE_PHASE_0_1_COLOR:
	loi 255.0
	add tColor, vf00, I
	sub.w tColor, vf00, vf00
	b COLOR_SET:


DETERMINE_PHASE_2_COLOR:
	; setup color of the vertex based on it's x axis position
	add tColor, vf00, vVert[x]
	; make the RGB values -255..255
	loi 15.0
	muli.xyz tColor, tColor, I
	; move the color to a temporary integer register that we
	;   will use to check which side of the x axis the vertex is on
	ftoi0 tiColor, tColor
	mtir  iColor, tiColor[x]

	; another temporary that will be used to invert the colors (from 0-255 to 255-0)
	loi 255.0
	addi.xyz vMaxColor, vf00, I

	; determine which side the vertex is on
	ibgez iColor, RED_SIDE:

BLUE_SIDE:
	abs.xyz tColor, tColor	
	; make sure all are within 255
	minii.xyz tColor, tColor, I
	; force Blue to be 255
	max.z tColor, vf00, I
	; Make Red/Green 0 at furthest from center, and 255 at closest
	sub.xy tColor, vMaxColor, tColor
	b COLOR_SET:

RED_SIDE:
	abs.xyz tColor, tColor	
	; make sure all are within 255
	minii.xyz tColor, tColor, I
	; force Red to be 255
	max.x tColor, vf00, I
	; make Blue/Green 0 at furthest from center, and 255 at closest
	sub.yz tColor, vMaxColor, tColor

COLOR_SET:
	; set alpha to 255
	sub.w tColor, vf00, vf00
	loi 255.0
	addi.w tColor, tColor, I

	; now make the float values to integer values
	ftoi0 tColor, tColor
	; save the color into the vertex
	VertexSave tColor, 0, iStoreTo 

	; alternative perspective correction, to make use of fScales
	div q, vf00[w], vVert[w]
	mul.xyz acc, fScales, vVert[w]
	madd.xyz vVert, vVert, fScales
	mul.xyz vVert, vVert, q

	; convert to screen coords
	VertexFptoGsXYZ2 vVert, vVert 
	VertexSave vVert, 1, iStoreTo

	; send the point to the GS
	iaddiu iPointTag, vi00, kPointTag
	xgkick iPointTag

	; move on to the next particle
	iaddiu iParticlePointer, iParticlePointer, 2

	; decrement loop counter and see if it's time to stop
	isubiu iParticleIndex, iParticleIndex, 1
	ibgtz iParticleIndex, PARTICLE_LOOP:

	; see if all particles are done with phase 2
	iaddiu iNumParticlesLeft, vi00, kNUM_PARTICLES
	isub iNumParticlesLeft, iNumParticlesLeft, iPhase2FinishCount
	ibgtz iNumParticlesLeft, END

	; initialize for explosion (if all particles are done with phase 2)
	;   which will in turn branch to END
	b INIT_EXPLOSION_PHASE:
;------------------------------------------------------------------------------


;------------------------------------------------------------------------------
INIT_EXPLOSION_PHASE:
;------------------------------------------------------------------------------
	; random seed init values (thanks to Kirk Bender and his fluff demo)
	loixyzw		rSeed,1.31357231,1.79512412,17.2341241,3.214341
	loixyzw		rMul,1.31357231,1.79512412,17.2341241,3.214341
	loixyzw		rAdd,0.71234121,0.1245121,0.313815147,0.6185812
	
	; make a little more random
	mul rAdd, rAdd, fLastQWord[z]

	iaddiu	iTemp, vi00, 200
	mfir fRadiusScale, iTemp
	itof0 fRadiusScale, fRadiusScale

	loi 100.0
	add fRadiusOffset, vf00, I

	loi 6.283185307
	add fPiScale, vf00, I

	iaddiu	iTemp, vi00, 10
	mfir fSourceScale, iTemp
	itof0 fSourceScale, fSourceScale

	iaddiu	iTemp, vi00, 5
	mfir fSourceOffset, iTemp
	itof0 fSourceOffset, fSourceOffset


	; initialize all our points
	iaddiu iParticlePointer, vi00, kParticleStart
	iaddiu iParticleCounter, vi00, kNUM_PARTICLES
EXPLOSION_PARTICLE_INIT:
	; Stewart's random number generator (thanks to Kirk Bender and his fluff demo)
	mula		rSeed,rSeed,rMul   
	maddw		rSeed,rAdd,vf00
	ftoi0		rI,rSeed
	itof0		rI,rI
	sub		rSeed,rSeed,rI			; rSeed now [0-1.0)

	; start with all zero (which means the particles phase is explosion)
	add.xyz vParticle, vf00, vf00
	sub.w vParticle, vf00, vf00

	; setup source xyz
	mul.xyz vParticle, rSeed, fSourceScale
	sub.xyz vParticle, vParticle, fSourceOffset
	
	; save the particle
	VertexSave vParticle, 0, iParticlePointer


	; create another random number (so the angle isn't used as the same as the x,z pos)
	mula		rSeed,rSeed,rMul   
	maddw		rSeed,rAdd,vf00
	ftoi0		rI,rSeed
	itof0		rI,rI
	sub		rSeed,rSeed,rI			; rSeed now [0-1.0)

	; set all to zero
	add vParticle, vf00, vf00[x]

	; calculate random r (and place in vDistance.x)
	mul.x vDistance, rSeed, fRadiusScale
	add.x vDistance, vDistance, fRadiusOffset
	; calculate random theta (x), phi (z)
	mul vAngles, rSeed, fPiScale
	; save theta
	add.w vParticle, vParticle, vAngles[x]

	; calculate sin and cos of theta and phi
	;   x=sin(theta),y=cos(theta)   z=sin(phi),w=cos(phi)
	AngleSinCos vAngles, vCalcAngles

	; calculate destination x,y,z
	; start with putting R in x,y,z
	add.xyz vParticle, vf00, vDistance[x]
	; calculate final x (multiply by sin(phi)*cos(theta))
	mul.x vParticle, vParticle, vCalcAngles[z]
	mul.x vParticle, vParticle, vCalcAngles[y]
	; calculate final z (multiply by sin(phi)*sin(theta))
	mul.z vParticle, vParticle, vCalcAngles[z]
	mul.z vParticle, vParticle, vCalcAngles[x]
	; calculate final y (multiply by cos(phi))
	mul.y vParticle, vParticle, vCalcAngles[w]
	
	; save the particle destination
	VertexSave vParticle, 1, iParticlePointer

	; move to the next particle
	iaddiu iParticlePointer, iParticlePointer, 2

	isubiu iParticleCounter, iParticleCounter, 1
	ibgtz iParticleCounter, EXPLOSION_PARTICLE_INIT:

	b END:
;-------------------------------------------------------------------------------




;-------------------------------------------------------------------------------
INITIALIZATION:
;-------------------------------------------------------------------------------
	; init camera zoom
	move.xyzw fCameraZoom, vf00
	loi -800.0
	addi.z fCameraZoom, fCameraZoom, I
	sq fCameraZoom, (kCameraZoom)(vi00)

	iaddiu iInitDone, iInitDone, 1
	isw.x iInitDone, kInitDone(vi00)
	b INIT_EXPLOSION_PHASE:
;-------------------------------------------------------------------------------


;-------------------------------------------------------------------------------
END:
;-------------------------------------------------------------------------------

--exit
--endexit

.END
