
#include <stdio.h>
#include <math.h>
#include "packet.h"
#define kHistoryStart 	12
#define kHistoryEnd 	112
#define kTicker 	5

typedef union
{
	u_long128     ul128;
	u_long        u64[2];
	u_int         u32[4];
	float         f32[4];
} qword_t;

int main(int argc, char **argv)
{
	FILE *output;
	int bytes=0;
	int prim=0;
	int m=0;
	float angle;
	int historySize = kHistoryEnd-kHistoryStart;
	u_long128 packet_data[1024];

	gifpacket_t gifpacket;

qword_t xydata, angles, colour0,vertex0,velocity,joydata;
qword_t Origin, window, X,Y,Z;
float da=0.025f;
float ca=(float)cos(da), sa=(float)sin(da);

angles.f32[0]=ca;	// positive rotation angles
angles.f32[1]=sa;
angles.f32[2]=ca;	// negative rotation angles	
angles.f32[3]=-sa;

xydata.f32[0]=10.0;	// dx distance
xydata.f32[1]=10.0;	// dy distance
xydata.u32[2]=24;	// number of vertices on x side	
xydata.u32[3]=23;	// not used

joydata.f32[0]=0;
joydata.f32[1]=0;
joydata.f32[2]=0;
joydata.f32[3]=0;

velocity.f32[0]=0.0;
velocity.f32[1]=0.0;
velocity.f32[2]=0.001;
velocity.f32[3]=0.0;

colour0.f32[0]=255;	// white
colour0.f32[1]=255;
colour0.f32[2]=255;
colour0.f32[3]=1.0;

Origin.f32[0]=0;	
Origin.f32[1]=0;
Origin.f32[2]=20;
Origin.f32[3]=0;

window.f32[0]=2048;	// x centre
window.f32[1]=2048;	// y centre
window.f32[2]=320;	// x half width
window.f32[3]=112;	// y half width

X.f32[0]=1;		// Frame[0]
X.f32[1]=0;
X.f32[2]=0;
X.f32[3]=0;

Y.f32[0]=0;		// Frame[1]
Y.f32[1]=1;
Y.f32[2]=0;
Y.f32[3]=0;

Z.f32[0]=0;		// Frame[2]
Z.f32[1]=0;
Z.f32[2]=1;
Z.f32[3]=0;

	if (argc!=2)
	{
		fprintf(stderr, "Usage: gen_data output_file\n");
		return -1;
	}

	gifpacket_init(&gifpacket, packet_data);

	prim=GS_SET_PRIM(0, GOURAUD(1), TEXTURED(0), FOG(0), ABE(0), ALPHA(0), AA(0), CTXT(0), FIX(0));
/*0*/	gifpacket_addgsdata(&gifpacket, GIF_SET_TAG(NLOOP(1), EOP(1), PRE(1), prim, GIF_PACKED, NREG(2)));
	gifpacket_addgsdata(&gifpacket, 0x51);
/*1*/	gifpacket_addgspacked(&gifpacket, Origin.ul128); // not used, was a null tag
/*2*/	gifpacket_addgspacked(&gifpacket, Origin.ul128);
/*3*/	gifpacket_addgspacked(&gifpacket, window.ul128);
/*4*/	gifpacket_addgspacked(&gifpacket, X.ul128);
/*5*/	gifpacket_addgspacked(&gifpacket, Y.ul128);
/*6*/	gifpacket_addgspacked(&gifpacket, Z.ul128);
/*7*/	gifpacket_addgspacked(&gifpacket, xydata.ul128);
/*8*/	gifpacket_addgspacked(&gifpacket, colour0.ul128);
/*9*/	gifpacket_addgspacked(&gifpacket, velocity.ul128);
/*10*/	gifpacket_addgspacked(&gifpacket, angles.ul128);
	

	vertex0.u32[0]=0xffffffff;      // previous Pad data
	vertex0.f32[1]=0;               // not used
	vertex0.u32[2]=kTicker;	        // frame ticker
	vertex0.u32[3]=kHistoryStart; 	// iHistory is at start

/*11*/	gifpacket_addgspacked(&gifpacket,vertex0.ul128);

	vertex0.f32[0]=0;      // previous Pad data
	vertex0.f32[1]=0;               // not used
	vertex0.f32[2]=0;	        // frame ticker
	vertex0.f32[3]=0; 	// iHistory is at start

	for(m=0;m<historySize;m++)
		{
		vertex0.f32[2]+=0.1;
		gifpacket_addgspacked(&gifpacket,vertex0.ul128);
		}

	// sines
	da=6.283184/48.0;
	angle=0;
	
	for(m=0;m<12;m++)
	  {
	    vertex0.f32[0]=sin(angle);
	    angle+=da;printf("sin %f = %f\n",angle,sin(angle));
	    vertex0.f32[1]=sin(angle);
	    angle+=da;printf("sin %f = %f\n",angle,sin(angle));
	    vertex0.f32[2]=sin(angle);
	    angle+=da;printf("sin %f = %f\n",angle,sin(angle));
	    vertex0.f32[3]=sin(angle);
	    angle+=da;printf("sin %f = %f\n",angle,sin(angle));
	    gifpacket_addgspacked(&gifpacket,vertex0.ul128);
	  }

	// cosines
	da=6.283184/48.0;
	angle=0;
	for(m=0;m<12;m++)
	  {
	    vertex0.f32[0]=cos(angle);angle+=da;
	    vertex0.f32[1]=cos(angle);angle+=da;
	    vertex0.f32[2]=cos(angle);angle+=da;
	    vertex0.f32[3]=cos(angle);angle+=da;
	    gifpacket_addgspacked(&gifpacket,vertex0.ul128);
	  }
	gifpacket_terminate(&gifpacket);
	output=fopen(argv[1], "wb");

	if (output==NULL)
	{
		fprintf(stderr, "Could not open file \"%s\"\n", argv[1]);
		return -1;
	}

	fwrite(packet_data, sizeof(packet_data)/sizeof(qword_t), sizeof(qword_t), output);

	bytes=ftell(output);
	fclose(output);

	printf("Written %d bytes to \"%s\"\n", bytes, argv[1]);

	return 0;
}

