// Kalleidoscope Stage 1
// Generate 20 random Triangles, mirror each one to 8 directions
//
// Write the dat file that contains the GifTag and data to draw
// Basic:
// QW 1 - GIFTAG:
//        NLOOP=3,
//        EOP=1,
//        PRE=1,
//        PRIM=tri,gouraud,notex,nofog,noalpha,noaa,N/A,ctxt1,unfixed
//        FLG=PACKED
//        NREG=2
//        REGS=0x51
// QW 2 - COLOR 1 (red)
// QW 3 - VERTEX 1 (2048-100,2048-50,1)
// QW 4 - COLOR 2 (green)
// QW 5 - VERTEX 2 (2048+100,2048-50,1)
// QW 6 - COLOR 3 (blue)
// QW 7 - VERTEX 3 (2048+100,2048+50,1)


#include <stdio.h>
#include <sys/time.h>
#include "packet.h"

// ------------------- Constants ---------------------------------
#define NUM_TRIS    20
#define SCR_WIDTH   600  /* 640 */
#define SCR_HEIGHT  200  /* 224 */
#define STEP (SCR_WIDTH / 18)
#define SCR_CENTER_X 2048
#define SCR_CENTER_Y 2048
// ------------------- Global Definitions -------------------------
typedef union
{
	u_long128     ul128;
	u_long64      ul64[2];
	u_int         ui32[4];
	float         f32[4];
} qword_t;

u_long128 packet_data[1024];
gifpacket_t gifpacket;

u_int clip_x(x) {
  int ret = x;
  if (x < SCR_CENTER_X - SCR_WIDTH/2 + 1) 
    ret = SCR_CENTER_X - SCR_WIDTH/2 + 1;
  if (x > SCR_CENTER_X + SCR_WIDTH/2 - 1) 
    ret = SCR_CENTER_X + SCR_WIDTH/2 - 1;
  /* multiply a X:Y ratio to it */
  ret = SCR_CENTER_X + ((ret - SCR_CENTER_X) * 1) / 1;
  return ret;
}

u_int clip_y(y) {
  if (y < SCR_CENTER_Y - SCR_HEIGHT/2 + 1)
    return  SCR_CENTER_Y - SCR_HEIGHT/2 + 1;
  if (y > SCR_CENTER_Y + SCR_HEIGHT/2 - 1) 
    return SCR_CENTER_Y + SCR_HEIGHT/2 - 1;
  return y;
}

int out_of_scr(x,y) {
  if (x < - SCR_WIDTH/2 + 1 ||
      x >  SCR_WIDTH/2 - 1 ||
      y < - SCR_HEIGHT/2 + 1 ||
      y > SCR_HEIGHT/2 - 1)
    return 1;
  else return 0;
}

int triangle_out_of_scr (x0,y0,x1,y1,x2,y2) {
  if (out_of_scr(x0,y0) || out_of_scr(x1, y1) || out_of_scr(x2,y2)) return 1;
  return 0;
}

int clip_triangle(u_int x0, u_int y0, u_int x1, u_int y1, u_int x2, u_int y2) 
{
  int cnt = 0;

  cnt += triangle_out_of_scr(x0, y0, x1, y1, x2, y2);
  cnt += triangle_out_of_scr(-x0, y0, -x1, y1, -x2, y2);
  cnt += triangle_out_of_scr(x0, -y0, x1, -y1, x2, -y2);
  cnt += triangle_out_of_scr(-x0, -y0, -x1, -y1, -x2, -y2);

  cnt += triangle_out_of_scr(y0, x0, y1, x1, y2, x2);
  cnt += triangle_out_of_scr(-y0, x0, -y1, x1, -y2, x2);
  cnt += triangle_out_of_scr(y0, -x0, y1, -x1, y2, -x2);
  cnt += triangle_out_of_scr(-y0, -x0, -y1, -x1, -y2, -x2);

  if (cnt > 0) return 1;  /* only keep triangles whose 8 mirrors are all in screen */
  return 0;
}

int draw_triangle(u_int x0, u_int y0, u_int x1, u_int y1, u_int x2, u_int y2,
		   u_int c0r, u_int c0g, u_int c0b, 
		   u_int c1r, u_int c1g, u_int c1b, 
		   u_int c2r, u_int c2g, u_int c2b) {
  qword_t color;
  qword_t vertex;

  if (triangle_out_of_scr(x0, y0, x1, y1, x2, y2)) return 0;
  memset(&color, 0, sizeof(color));
  color.ui32[0]=c0r;
  color.ui32[1]=c0g;
  color.ui32[2]=c0b;
  color.ui32[3]=0x80;
  gifpacket_addgspacked(&gifpacket, color.ul128);
  
  vertex.ui32[0]= clip_x(SCR_CENTER_X + x0) <<4; 
  vertex.ui32[1]= clip_y(SCR_CENTER_Y + y0) <<4; 
  vertex.ui32[2]= 1;    // Z coordinate
  vertex.ui32[3]=1;
  gifpacket_addgspacked(&gifpacket, vertex.ul128);
  
  color.ui32[0]=c1r;
  color.ui32[1]=c1g;
  color.ui32[2]=c1b;
  color.ui32[3]=0x80;
  gifpacket_addgspacked(&gifpacket, color.ul128);
  
  vertex.ui32[0]= clip_x(SCR_CENTER_X + x1) <<4; 
  vertex.ui32[1]= clip_y(SCR_CENTER_Y + y1) <<4; 
  vertex.ui32[2]=1;    // Z coordinate
  vertex.ui32[3]=1;
  gifpacket_addgspacked(&gifpacket, vertex.ul128);

  color.ui32[0]=c2r;
  color.ui32[1]=c2g;
  color.ui32[2]=c2b;
  color.ui32[3]=0x80;
  gifpacket_addgspacked(&gifpacket, color.ul128);
  
  vertex.ui32[0]= clip_x(SCR_CENTER_X + x2) <<4; 
  vertex.ui32[1]= clip_y(SCR_CENTER_Y + y2) <<4; 
  vertex.ui32[2]=1;    // Z coordinate
  vertex.ui32[3]=1;
  gifpacket_addgspacked(&gifpacket, vertex.ul128);
  return 1;

}

int main(int argc, char **argv)
{
	FILE *output;
	int bytes=0;
	int prim=0;
	int cnt = 0;
	u_int x0, y0, x1, y1, x2, y2;
	u_int c0r, c0g, c0b, c1r, c1g, c1b, c2r, c2g, c2b;
	unsigned int seed = 0;
	struct timeval tv;
	qword_t damp, doublehalf;

	if (argc!=2)
	{
		fprintf(stderr, "Usage: gen_data output_file\n");
		return -1;
	}

	gifpacket_init(&gifpacket, packet_data);

	prim=GS_SET_PRIM(GS_PRIM_TRI, GOURAUD(1), TEXTURED(0), FOG(0), ABE(0), 
			 ALPHA(0), AA(0), CTXT(0), FIX(0));
	gifpacket_addgsdata(&gifpacket, GIF_SET_TAG(NLOOP(NUM_TRIS*3*8), EOP(1), PRE(1), 
						    prim, GIF_PACKED, NREG(2)));
	gifpacket_addgsdata(&gifpacket, 0x51);

	srand((unsigned int)time(NULL));

	while (cnt < NUM_TRIS*8) {
	  
	  do {
	    x0 = - SCR_WIDTH/2 + (rand()%SCR_WIDTH);
	    y0 = - SCR_HEIGHT/2 + (rand()%SCR_HEIGHT);
	    c0r = rand()%256;
	    c0g = 0;
	    c0b = rand()%256;
	    
	    x1 = x0 + (rand() % STEP);
	    y1 = y0 - STEP/2 + (rand() % STEP);
	    c1r = rand()%256;
	    c1g = rand()%256;
	    c1b = 0;
	    
	    x2 = x0 + (rand() % STEP);
	    y2 = y0 - STEP/2 + (rand() % STEP);
	    c2r = 0;
	    c2g = rand()%256;
	    c2b = rand()%256;
	  } while (clip_triangle(x0,y0,x1,y1,x2,y2));

	  cnt += draw_triangle(x0,y0,x1,y1,x2,y2,c0r,c0g,c0b,c1r,c1g,c1b,c2r,c2g,c2b);
	  if (cnt ==  NUM_TRIS*3*8) break;
	  cnt += draw_triangle(-x0,y0,-x1,y1,-x2,y2,c0r,c0g,c0b,c1r,c1g,c1b,c2r,c2g,c2b);
	  if (cnt ==  NUM_TRIS*3*8) break;
	  cnt += draw_triangle(x0,-y0,x1,-y1,x2,-y2,c0r,c0g,c0b,c1r,c1g,c1b,c2r,c2g,c2b);
	  if (cnt ==  NUM_TRIS*3*8) break;
	  cnt += draw_triangle(-x0,-y0,-x1,-y1,-x2,-y2,c0r,c0g,c0b,c1r,c1g,c1b,c2r,c2g,c2b);
	  if (cnt ==  NUM_TRIS*3*8) break;
	  cnt += draw_triangle(y0,x0,y1,x1,y2,x2,c0r,c0g,c0b,c1r,c1g,c1b,c2r,c2g,c2b);
	  if (cnt ==  NUM_TRIS*3*8) break;
	  cnt += draw_triangle(-y0,x0,-y1,x1,-y2,x2,c0r,c0g,c0b,c1r,c1g,c1b,c2r,c2g,c2b);
	  if (cnt ==  NUM_TRIS*3*8) break;
	  cnt += draw_triangle(y0,-x0,y1,-x1,y2,-x2,c0r,c0g,c0b,c1r,c1g,c1b,c2r,c2g,c2b);
	  if (cnt ==  NUM_TRIS*3*8) break;
	  cnt += draw_triangle(-y0,-x0,-y1,-x1,-y2,-x2,c0r,c0g,c0b,c1r,c1g,c1b,c2r,c2g,c2b);
	  if (cnt ==  NUM_TRIS*3*8) break;
	  printf("cnt = %d\n", cnt);
	}

	damp.f32[0] = damp.f32[1] = damp.f32[2] = damp.f32[3] = 0.995;
	gifpacket_addgspacked(&gifpacket, damp.ul128);

	doublehalf.f32[0] = 2; 
	doublehalf.f32[1] = 0.5;
	doublehalf.f32[2] = 1;
	doublehalf.f32[3] = 1;
	gifpacket_addgspacked(&gifpacket, doublehalf.ul128);

	gifpacket_terminate(&gifpacket);

	output=fopen(argv[1], "wb");
	
	if (output==NULL)
	{
		fprintf(stderr, "Could not open file \"%s\"\n", argv[1]);
		return -1;
	}

	fwrite(packet_data, sizeof(packet_data)/sizeof(qword_t), 
	       sizeof(qword_t), output);

	bytes=ftell(output);
	fclose(output);

	printf("Written %d bytes to \"%s\"\n", bytes, argv[1]);

	return 0;
}

