
// Write the dat file that contains the GifTag and data to draw
// Basic:
// QW 1 - GIFTAG:
//        NLOOP=3,
//        EOP=1,
//        PRE=1,
//        PRIM=tri,gouraud,notex,nofog,noalpha,noaa,N/A,ctxt1,unfixed
//        FLG=PACKED
//        NREG=2
//        REGS=0x51
// QW 2 - COLOR 1 (red)
// QW 3 - VERTEX 1 (2048-100,2048-50,1)
// QW 4 - COLOR 2 (green)
// QW 5 - VERTEX 2 (2048+100,2048-50,1)
// QW 6 - COLOR 3 (blue)
// QW 7 - VERTEX 3 (2048+100,2048+50,1)


#include <stdio.h>
#include <math.h>

#include "packet.h"

// #define GS_PRIM_POINT 0
#define ONE_DEGREE (3.1415926535/180.0)

#define AD_REG 0x0e
#define BITBLTBUF_REG 0x50
#define TRXPOS_REG 0x51
#define TRXREG_REG 0x52
#define TRXDIR_REG 0x53
#define TEXFLUSH_REG 0x3f
#define TEST_1_REG 0x47
#define FBA_1_REG 0x4a
#define ALPHA_1_REG 0x42
#define PSMCT32 0
#define GIF_IMAGE 2
#define TRXDIR_TO_GS 0
#define PS2_GS_ZALWAYS 1

/*
#define GS_SET_BITBLTBUF(SBP, SBW, SPSM, DBP, DBW, DPSM)                                                 (((u_long64)(DPSM)<<56) | ((u_long64)(DBW)<<48) | ((u_long64)(DBP)<<32) | ((u_long64)(SPSM)<<24) | ((u_long64)(SBW)<<16) | ((u_long64)(SBP)<<0))
#define GS_SET_TRXREG(RRW, RRH)                                                                          (((u_long64)(RRH)<<32) | ((u_long64)(RRW)<<0))

#define GS_SET_TRXPOS(SSAX, SSAY, DSAX, DSAY, DIR)                                                       (((u_long64)(DIR)<<59) | ((u_long64)(DSAY)<<48) | ((u_long64)(DSAX)<<32) | ((u_long64)(SSAY)<<16) | ((u_long64)(SSAX)<<0))
#define GS_SET_TRXDIR(XDIR)                                                                              ((u_long64) XDIR&0x3)

*/
FILE *def;

typedef union
{
	u_long128     ul128;
	u_long64        ul64[2];
	u_int         ui32[4];
	float         f32[4];
} qword_t;

const char *pcCurrentElement;
int iCurrentElementStart;

void outputAddress(const char *pcSubElement, int iOffset) {
  if (!pcSubElement) {
    fprintf(def, "k%s .assign %04i\n", pcCurrentElement,
	    iCurrentElementStart+iOffset);
  } else {
    fprintf(def, "k%s_%s .assign %04i\n", pcCurrentElement,
	    pcSubElement, iCurrentElementStart+iOffset);
  }
}

void setCurrentElement(const char *pcName, int iElementStart) {
  pcCurrentElement=pcName;
  iCurrentElementStart=iElementStart;
  fprintf(def,"\n;------------------\n; Start of %s\n;------------------\n",
	  pcCurrentElement);
  outputAddress(0,0);
}

int main(int argc, char **argv)
{
  FILE *output;
  int bytes=0;
  int prim=0;
  qword_t color;
  qword_t vertex;
  qword_t stq;
  qword_t qw;
  qword_t reg;

  int i,j;

  u_long128 packet_data[1024];

  gifpacket_t gifpacket;
  int iElementStart=0;

  if (argc!=3)
    {
      fprintf(stderr, "Usage: gen_data <output_file> <def file>\n");
      return -1;
    }

  def=fopen(argv[2], "wt");

  if (!def) {
    fprintf(stderr, "Can't open def file %s\n", argv[2]);
    return -1;
  }

  /********************************************************
   * WARNING: DO NOT TOUCH ANY OF THESE WITHOUT UPDATING  *
   * sauce_demo.dsm, IF NECESSARY                         *
   *******************************************************/
  setCurrentElement("SCRATCH_QWORD", iElementStart);
  iElementStart++;

  setCurrentElement("STATE", iElementStart);
  qw.ui32[0]=0; qw.ui32[1]=0; qw.ui32[2]=0; qw.ui32[3]=0;
  packet_data[iElementStart]=qw.ul128;
  iElementStart++;

  setCurrentElement("PARAMETERS", iElementStart);
  iElementStart++;

  /********************************************************
   * WARNING: ENDS HERE                                   *
   *******************************************************/

  setCurrentElement("TRANSFORM_MATRIX", iElementStart);
  iElementStart+=4;

  setCurrentElement("LIGHT_1_MATRIX", iElementStart);
  iElementStart+=4;

  setCurrentElement("LIGHT_2_MATRIX", iElementStart);
  iElementStart+=4;

  setCurrentElement("LIGHT_3_MATRIX", iElementStart);
  iElementStart+=4;

  setCurrentElement("LIGHT_AMBIENT", iElementStart);
  iElementStart++;

  // Start section of GIF tags
  setCurrentElement("GIF_TAG", iElementStart);
  gifpacket_init(&gifpacket, packet_data + iElementStart);

  // No-op tag
  outputAddress("NOP", gifpacket_size(&gifpacket));
  gifpacket_addgsdata(&gifpacket, GIF_SET_TAG(NLOOP(0), EOP(1), PRE(0),
					      0, GIF_PACKED, NREG(0)));
  gifpacket_addgsdata(&gifpacket, 0);

  // Color triangle tag
  outputAddress("COLOR_TRI", gifpacket_size(&gifpacket));
  prim=GS_SET_PRIM(GS_PRIM_TRI, GOURAUD(1), TEXTURED(0), FOG(0), ABE(0),
		   ALPHA(1), AA(0), CTXT(0), FIX(0));
  gifpacket_addgsdata(&gifpacket, GIF_SET_TAG(NLOOP(0), EOP(1), PRE(1),
					      prim, GIF_PACKED,
					      NREG(2)));
  gifpacket_addgsdata(&gifpacket, 0x51);

  // Color triangle tag
  outputAddress("COLOR_POINT", gifpacket_size(&gifpacket));
  prim=GS_SET_PRIM(GS_PRIM_POINT, GOURAUD(1), TEXTURED(0), FOG(0), ABE(0),
		   ALPHA(1), AA(0), CTXT(0), FIX(0));
  gifpacket_addgsdata(&gifpacket, GIF_SET_TAG(NLOOP(0), EOP(1), PRE(1),
					      prim, GIF_PACKED,
					      NREG(2)));
  gifpacket_addgsdata(&gifpacket, 0x51);

  // Color tristrip tag
  outputAddress("COLOR_TRISTRIP", gifpacket_size(&gifpacket));
  prim=GS_SET_PRIM(GS_PRIM_TRISTRIP, GOURAUD(1), TEXTURED(0), FOG(0), ABE(0),
		   ALPHA(1), AA(0), CTXT(0), FIX(0));
  gifpacket_addgsdata(&gifpacket, GIF_SET_TAG(NLOOP(0), EOP(1), PRE(1),
					      prim, GIF_PACKED,
					      NREG(2)));
  gifpacket_addgsdata(&gifpacket, 0x51);

  // Color, textured tristrip tag
  outputAddress("COLOR_TEXTURED_TRISTRIP", gifpacket_size(&gifpacket));
  prim=GS_SET_PRIM(GS_PRIM_TRISTRIP, GOURAUD(1), TEXTURED(1), FOG(0), ABE(0),
		   ALPHA(1), AA(0), CTXT(0), FIX(0));
  gifpacket_addgsdata(&gifpacket, GIF_SET_TAG(NLOOP(0), EOP(1), PRE(1),
					      prim, GIF_PACKED,
					      NREG(3)));
  gifpacket_addgsdata(&gifpacket, 0x521);

  // Color, textured tri tag
  outputAddress("COLOR_TEXTURED_TRI", gifpacket_size(&gifpacket));
  prim=GS_SET_PRIM(GS_PRIM_TRI, GOURAUD(1), TEXTURED(1), FOG(0), ABE(0),
		   ALPHA(1), AA(0), CTXT(0), FIX(0));
  gifpacket_addgsdata(&gifpacket, GIF_SET_TAG(NLOOP(0), EOP(1), PRE(1),
					      prim, GIF_PACKED,
					      NREG(3)));
  gifpacket_addgsdata(&gifpacket, 0x521);

  // Image data tag
  outputAddress("IMAGE", gifpacket_size(&gifpacket));
  gifpacket_addgsdata(&gifpacket, GIF_SET_TAG(NLOOP(0), EOP(1), PRE(0),
					      0, GIF_IMAGE, NREG(0)));
  gifpacket_addgsdata(&gifpacket, 0);

  // Color sprite tag
  outputAddress("COLOR_SPRITE", gifpacket_size(&gifpacket));
  prim=GS_SET_PRIM(GS_PRIM_SPRITE, GOURAUD(1), TEXTURED(0), FOG(0), ABE(0),
		   ALPHA(1), AA(0), CTXT(0), FIX(0));
  gifpacket_addgsdata(&gifpacket, GIF_SET_TAG(NLOOP(0), EOP(1), PRE(1),
					      prim, GIF_PACKED,
					      NREG(2)));
  gifpacket_addgsdata(&gifpacket, 0x51);

  // No more tags
  gifpacket_terminate(&gifpacket);
  iElementStart+=gifpacket_size(&gifpacket);

  setCurrentElement("TRIANGLE", iElementStart);
  gifpacket_init(&gifpacket, packet_data + iElementStart);

  outputAddress("TAG", gifpacket_size(&gifpacket));
  prim=GS_SET_PRIM(GS_PRIM_TRI, GOURAUD(1), TEXTURED(1), FOG(0), ABE(0),
		   ALPHA(1), AA(0), CTXT(0), FIX(0));
  gifpacket_addgsdata(&gifpacket, GIF_SET_TAG(NLOOP(3), EOP(1), PRE(1),
					      prim, GIF_PACKED,
					      NREG(3)));
  gifpacket_addgsdata(&gifpacket, 0x521);

  color.ui32[0]=0xff; color.ui32[1]=0xff; color.ui32[2]=0xff;
  color.ui32[3]=0xff;

  outputAddress("COLOR_1",
		gifpacket_size(&gifpacket));
  gifpacket_addgspacked(&gifpacket, color.ul128);

  stq.f32[0]=1.0;
  stq.f32[1]=0.0;
  stq.f32[2]=1.0;
  stq.f32[3]=1.0;

  outputAddress("STQ_1",
		gifpacket_size(&gifpacket));
  gifpacket_addgspacked(&gifpacket, stq.ul128);
	
  vertex.ui32[0]=(2048-100)<<4; vertex.ui32[1]=(2048-50)<<4;
  vertex.ui32[2]=1; vertex.ui32[3]=1;

  outputAddress("XYZ_1",
		gifpacket_size(&gifpacket));
  gifpacket_addgspacked(&gifpacket, vertex.ul128);
	
  color.ui32[0]=0xff; color.ui32[1]=0x00; color.ui32[2]=0xff;
  color.ui32[3]=0xff;

  outputAddress("COLOR_2",
		gifpacket_size(&gifpacket));
  gifpacket_addgspacked(&gifpacket, color.ul128);

  stq.f32[0]=0.0;
  stq.f32[1]=1.0;
  stq.f32[2]=1.0;
  stq.f32[3]=0.0;

  outputAddress("STQ_2",
		gifpacket_size(&gifpacket));
  gifpacket_addgspacked(&gifpacket, stq.ul128);
	
  vertex.ui32[0]=(2048+100)<<4; vertex.ui32[1]=(2048-50)<<4;
  vertex.ui32[2]=1; vertex.ui32[3]=1;

  outputAddress("XYZ_2",
		gifpacket_size(&gifpacket));
  gifpacket_addgspacked(&gifpacket, vertex.ul128);
	
  color.ui32[0]=0xff; color.ui32[1]=0xff; color.ui32[2]=0xff;
  color.ui32[3]=0xff;

  outputAddress("COLOR_3",
		gifpacket_size(&gifpacket));
  gifpacket_addgspacked(&gifpacket, color.ul128);
	


  stq.f32[0]=1.0;
  stq.f32[1]=1.0;
  stq.f32[2]=1.0;
  stq.f32[3]=1.0;

  outputAddress("STQ_3",
		gifpacket_size(&gifpacket));
  gifpacket_addgspacked(&gifpacket, stq.ul128);

  vertex.ui32[0]=(2048+100)<<4; vertex.ui32[1]=(2048+50)<<4;
  vertex.ui32[2]=1; vertex.ui32[3]=1;
  outputAddress("XYZ_3",
		gifpacket_size(&gifpacket));
  gifpacket_addgspacked(&gifpacket, vertex.ul128);

  gifpacket_terminate(&gifpacket);
  iElementStart+=gifpacket_size(&gifpacket);

	// Point
  setCurrentElement("POINT", iElementStart);

  gifpacket_init(&gifpacket, packet_data+iElementStart);
  prim=GS_SET_PRIM(GS_PRIM_POINT, GOURAUD(1),
		   TEXTURED(0), FOG(0), ABE(0), ALPHA(1),
		   AA(0), CTXT(0), FIX(0));
  gifpacket_addgsdata(&gifpacket, GIF_SET_TAG(NLOOP(1), EOP(1),
					      PRE(1), prim, GIF_PACKED, 
					      NREG(2)));
  gifpacket_addgsdata(&gifpacket, 0x51);

  outputAddress("COLOR",
		gifpacket_size(&gifpacket));

  color.ui32[0]=0xFF; color.ui32[1]=0x0; color.ui32[2]=0x0;
  color.ui32[3]=0x0;
  gifpacket_addgspacked(&gifpacket, color.ul128);
	
  outputAddress("XYZ",
		gifpacket_size(&gifpacket));

  vertex.ui32[0]=(2048-100)<<4; vertex.ui32[1]=(2048-50)<<4;
  vertex.ui32[2]=1; vertex.ui32[3]=1;

  gifpacket_addgspacked(&gifpacket, vertex.ul128);

  gifpacket_terminate(&gifpacket);
  iElementStart+=gifpacket_size(&gifpacket);


	// X, Y, Z, W = 2048 * 16 = 32768
  setCurrentElement("CENTER", iElementStart);

  vertex.ui32[0]=2048 << 4;
  vertex.ui32[1]=2048 << 4;
  vertex.ui32[2]=2048 << 4;
  vertex.ui32[3]=1 << 4;

  packet_data[iElementStart]=vertex.ul128;
  iElementStart++;

  setCurrentElement("WHITE", iElementStart);
  color.ui32[0]=0xff; color.ui32[1]=0xFF; color.ui32[2]=0xff;
  color.ui32[3]=0x0;
  packet_data[iElementStart]=color.ul128;
  iElementStart++;

  setCurrentElement("SPHERE_SEGMENTS_RADIUS", iElementStart);
  qw.f32[0]=50;
  qw.f32[1]=20;
  packet_data[iElementStart]=qw.ul128;
  iElementStart++;

  setCurrentElement("TEXTURE_UPLOAD_INIT", iElementStart);
  gifpacket_init(&gifpacket, packet_data + iElementStart);
  gifpacket_addgsdata(&gifpacket, GIF_SET_TAG(NLOOP(4), EOP(1), PRE(0),
					      0, GIF_PACKED, NREG(1)));
  gifpacket_addgsdata(&gifpacket, AD_REG);
  outputAddress("BITBLTBUF", gifpacket_size(&gifpacket));
  reg.ul64[0]=GS_SET_BITBLTBUF(0, 1, PSMCT32, /* 0x118 << 5 */ 0x118, 1, PSMCT32);
  reg.ul64[1]=BITBLTBUF_REG;
  gifpacket_addgspacked(&gifpacket, reg.ul128);
  reg.ul64[0]=GS_SET_TRXPOS(0,0,0,0,0);
  reg.ul64[1]=TRXPOS_REG;
  gifpacket_addgspacked(&gifpacket, reg.ul128);
  reg.ul64[0]=GS_SET_TRXREG(32, 32);
  reg.ul64[1]=TRXREG_REG;
  gifpacket_addgspacked(&gifpacket, reg.ul128);
  reg.ul64[0]=GS_SET_TRXDIR(TRXDIR_TO_GS);
  reg.ul64[1]=TRXDIR_REG;
  gifpacket_addgspacked(&gifpacket, reg.ul128);
  outputAddress("BITBLTBUF_TEMPLATE", gifpacket_size(&gifpacket));
  reg.ul64[0]=GS_SET_BITBLTBUF(0, 1, PSMCT32, 0x118 << 5, 1, PSMCT32);
  reg.ul64[1]=BITBLTBUF_REG;
  gifpacket_addgspacked(&gifpacket, reg.ul128);
  gifpacket_terminate(&gifpacket);
  iElementStart+=gifpacket_size(&gifpacket);

  setCurrentElement("TEXTURE_UPLOAD_DATA", iElementStart);
  gifpacket_init(&gifpacket, packet_data + iElementStart);
  gifpacket_addgsdata(&gifpacket, GIF_SET_TAG(NLOOP(1024/4), EOP(1), PRE(0),
					      0, GIF_IMAGE, NREG(0)));
  gifpacket_addgsdata(&gifpacket, 0);

  /*
  for(i=0, j=0; i<256/4; i++)
    {
      qword_t colour;
      colour.ui32[0] = (j) | (j << 8) | (j << 16) | 0x00000000;
      j++;
      colour.ui32[1] = (j) | (j << 8) | (j << 16) | 0x00000000;
      j++;
      colour.ui32[2] = (j) | (j << 8) | (j << 16) | 0x00000000;
      j++;
      colour.ui32[3] = (j) | (j << 8) | (j << 16) | 0x00000000;
      j++;
      gifpacket_addgspacked(&gifpacket, colour.ul128);
    }
  */
  gifpacket_terminate(&gifpacket);
  iElementStart+=gifpacket_size(&gifpacket);

  setCurrentElement("TEXTURE_ENABLE", iElementStart);
  gifpacket_init(&gifpacket, packet_data + iElementStart);
  gifpacket_addgsdata(&gifpacket, GIF_SET_TAG(NLOOP(4), EOP(1), PRE(0),
					      0, GIF_PACKED, NREG(1)));
  gifpacket_addgsdata(&gifpacket, AD_REG);
  reg.ul64[0] = GS_SET_TEXFLUSH();
  reg.ul64[1] = TEXFLUSH_REG;
  gifpacket_addgspacked(&gifpacket, reg.ul128);
  reg.ul64[0] = GS_SET_TEXA(0xff, TEXA_AEM_NORMAL, 0xff);
  reg.ul64[1] = TEXA_REG;
  gifpacket_addgspacked(&gifpacket, reg.ul128);
  reg.ul64[0] = GS_SET_TEX0(0x118 << 5, 1, 1, 5, 5, TEX0_TCC_RGB,
			    TEX0_TFX_MODULATE, 0, 0, 0, 0, 0);
  reg.ul64[1] = TEX0_1_REG;
  gifpacket_addgspacked(&gifpacket, reg.ul128);
  reg.ul64[0] = GS_SET_TEX1(TEX1_LCM_K, 0, TEX1_MMAG_NEAREST,
			    TEX1_MMIN_NEAREST, TEX1_MTBA_NOAUTO, 0, 0);
  reg.ul64[1] = TEX1_1_REG;
  gifpacket_addgspacked(&gifpacket, reg.ul128);
  gifpacket_terminate(&gifpacket);
  iElementStart+=gifpacket_size(&gifpacket);

  // Alpha enable/disable
  setCurrentElement("ALPHA", iElementStart);
  gifpacket_init(&gifpacket, packet_data + iElementStart);
  outputAddress("ENABLE", gifpacket_size(&gifpacket));
  gifpacket_addgsdata(&gifpacket, GIF_SET_TAG(NLOOP(2), EOP(1), PRE(0),
					      0, GIF_PACKED, NREG(1)));
  gifpacket_addgsdata(&gifpacket, AD_REG);
  reg.ul64[0] = 1;
  reg.ul64[1] = FBA_1_REG;
  gifpacket_addgspacked(&gifpacket, reg.ul128);
  outputAddress("ENABLE_ALPHA_1_REG", gifpacket_size(&gifpacket));
  reg.ui32[0] = (1 << 6) | (2 << 4) | (2 << 2) | (0 << 0);
  reg.ui32[1] =  64;
  reg.ul64[1] = ALPHA_1_REG;
  gifpacket_addgspacked(&gifpacket, reg.ul128);

  outputAddress("DISABLE", gifpacket_size(&gifpacket));
  gifpacket_addgsdata(&gifpacket, GIF_SET_TAG(NLOOP(2), EOP(1), PRE(0),
					      0, GIF_PACKED, NREG(1)));
  gifpacket_addgsdata(&gifpacket, AD_REG);
  reg.ul64[0] = 0;
  reg.ul64[1] = FBA_1_REG;
  gifpacket_addgspacked(&gifpacket, reg.ul128);
  reg.ui32[0] = (0 << 6) | (2 << 4) | (2 << 2) | (0 << 0);
  reg.ui32[1] =  64;
  reg.ul64[1] = ALPHA_1_REG;
  gifpacket_addgspacked(&gifpacket, reg.ul128);

  gifpacket_terminate(&gifpacket);
  iElementStart+=gifpacket_size(&gifpacket);


  // Z Depth test enable/disable
  setCurrentElement("DEPTH_TEST", iElementStart);
  gifpacket_init(&gifpacket, packet_data + iElementStart);
  outputAddress("ENABLE", gifpacket_size(&gifpacket));
  gifpacket_addgsdata(&gifpacket, GIF_SET_TAG(NLOOP(1), EOP(1), PRE(0),
					      0, GIF_PACKED, NREG(1)));
  gifpacket_addgsdata(&gifpacket, AD_REG);
  reg.ul64[0] = (1 << 16) | (PS2_GS_ZALWAYS << 17);
  reg.ul64[1] = TEST_1_REG;
  gifpacket_addgspacked(&gifpacket, reg.ul128);

  outputAddress("DISABLE", gifpacket_size(&gifpacket));
  gifpacket_addgsdata(&gifpacket, GIF_SET_TAG(NLOOP(1), EOP(1), PRE(0),
					      0, GIF_PACKED, NREG(1)));
  gifpacket_addgsdata(&gifpacket, AD_REG);
  reg.ul64[0] = PS2_GS_ZALWAYS << 17;
  reg.ul64[1] = TEST_1_REG;
  gifpacket_addgspacked(&gifpacket, reg.ul128);

  gifpacket_terminate(&gifpacket);
  iElementStart+=gifpacket_size(&gifpacket);






  // Last entry in file
  setCurrentElement("256_QWORDS", iElementStart);
  //  iElementStart+=257;


  if (iElementStart+257>1024) {
    fprintf(stderr, "Too much data!  Element Start is %i\n", iElementStart);
    exit(-1);
  }

  output=fopen(argv[1], "wb");
	
  if (output==NULL)
    {
      fprintf(stderr, "Could not open file \"%s\"\n", argv[1]);
      return -1;
    }

  //	fwrite(packet_data, sizeof(packet_data)/sizeof(qword_t), sizeof(qword_t), output);
  //	iElementStart=1024;
  fwrite(packet_data, iElementStart, sizeof(qword_t), output);

  bytes=ftell(output);
  fclose(output);

  printf("Written %d bytes to \"%s\"\n", bytes, argv[1]);

  return 0;
}






































int main1(int argc, char **argv)
{
  FILE *output;
  int bytes=0;
  int prim=0;
  qword_t color;
  qword_t vertex;
  qword_t stq;
  qword_t qw;
  qword_t reg;

  int i,j;

  u_long128 packet_data[1024];

  gifpacket_t gifpacket;
  int iElementStart=0;

  if (argc!=3)
    {
      fprintf(stderr, "Usage: gen_data <output_file> <def file>\n");
      return -1;
    }

  def=fopen(argv[2], "wt");

  if (!def) {
    fprintf(stderr, "Can't open def file %s\n", argv[2]);
    return -1;
  }

  /********************************************************
   * WARNING: DO NOT TOUCH ANY OF THESE WITHOUT UPDATING  *
   * sauce_demo.dsm, IF NECESSARY                         *
   *******************************************************/
  setCurrentElement("SCRATCH_QWORD", iElementStart);
  iElementStart++;

  setCurrentElement("STATE", iElementStart);
  qw.ui32[0]=0; qw.ui32[1]=0; qw.ui32[2]=0; qw.ui32[3]=0;
  packet_data[iElementStart]=qw.ul128;
  iElementStart++;

  setCurrentElement("PARAMETERS", iElementStart);
  iElementStart++;

  /********************************************************
   * WARNING: ENDS HERE                                   *
   *******************************************************/

  setCurrentElement("TRANSFORM_MATRIX", iElementStart);
  iElementStart+=4;

  setCurrentElement("LIGHT_1_MATRIX", iElementStart);
  iElementStart+=4;

  setCurrentElement("LIGHT_2_MATRIX", iElementStart);
  iElementStart+=4;

  setCurrentElement("LIGHT_3_MATRIX", iElementStart);
  iElementStart+=4;

  setCurrentElement("LIGHT_AMBIENT", iElementStart);
  iElementStart++;

  // Start section of GIF tags
  setCurrentElement("GIF_TAG", iElementStart);
  gifpacket_init(&gifpacket, packet_data + iElementStart);

  // No-op tag
  outputAddress("NOP", gifpacket_size(&gifpacket));
  gifpacket_addgsdata(&gifpacket, GIF_SET_TAG(NLOOP(0), EOP(1), PRE(0),
					      0, GIF_PACKED, NREG(0)));
  gifpacket_addgsdata(&gifpacket, 0);

  // Color triangle tag
  outputAddress("COLOR_TRI", gifpacket_size(&gifpacket));
  prim=GS_SET_PRIM(GS_PRIM_TRI, GOURAUD(1), TEXTURED(0), FOG(0), ABE(0),
		   ALPHA(1), AA(0), CTXT(0), FIX(0));
  gifpacket_addgsdata(&gifpacket, GIF_SET_TAG(NLOOP(0), EOP(1), PRE(1),
					      prim, GIF_PACKED,
					      NREG(2)));
  gifpacket_addgsdata(&gifpacket, 0x51);

  // Color triangle tag
  outputAddress("COLOR_POINT", gifpacket_size(&gifpacket));
  prim=GS_SET_PRIM(GS_PRIM_POINT, GOURAUD(1), TEXTURED(0), FOG(0), ABE(0),
		   ALPHA(1), AA(0), CTXT(0), FIX(0));
  gifpacket_addgsdata(&gifpacket, GIF_SET_TAG(NLOOP(0), EOP(1), PRE(1),
					      prim, GIF_PACKED,
					      NREG(2)));
  gifpacket_addgsdata(&gifpacket, 0x51);

  // Color tristrip tag
  outputAddress("COLOR_TRISTRIP", gifpacket_size(&gifpacket));
  prim=GS_SET_PRIM(GS_PRIM_TRISTRIP, GOURAUD(1), TEXTURED(0), FOG(0), ABE(0),
		   ALPHA(1), AA(0), CTXT(0), FIX(0));
  gifpacket_addgsdata(&gifpacket, GIF_SET_TAG(NLOOP(0), EOP(1), PRE(1),
					      prim, GIF_PACKED,
					      NREG(2)));
  gifpacket_addgsdata(&gifpacket, 0x51);

  // Color, textured tristrip tag
  outputAddress("COLOR_TEXTURED_TRISTRIP", gifpacket_size(&gifpacket));
  prim=GS_SET_PRIM(GS_PRIM_TRISTRIP, GOURAUD(1), TEXTURED(1), FOG(0), ABE(0),
		   ALPHA(1), AA(0), CTXT(0), FIX(0));
  gifpacket_addgsdata(&gifpacket, GIF_SET_TAG(NLOOP(0), EOP(1), PRE(1),
					      prim, GIF_PACKED,
					      NREG(3)));
  gifpacket_addgsdata(&gifpacket, 0x521);

  // Image data tag
  outputAddress("IMAGE", gifpacket_size(&gifpacket));
  gifpacket_addgsdata(&gifpacket, GIF_SET_TAG(NLOOP(0), EOP(1), PRE(0),
					      0, GIF_IMAGE, NREG(0)));
  gifpacket_addgsdata(&gifpacket, 0);

  // Color sprite tag
  outputAddress("COLOR_SPRITE", gifpacket_size(&gifpacket));
  prim=GS_SET_PRIM(GS_PRIM_SPRITE, GOURAUD(1), TEXTURED(0), FOG(0), ABE(0),
		   ALPHA(1), AA(0), CTXT(0), FIX(0));
  gifpacket_addgsdata(&gifpacket, GIF_SET_TAG(NLOOP(0), EOP(1), PRE(1),
					      prim, GIF_PACKED,
					      NREG(2)));
  gifpacket_addgsdata(&gifpacket, 0x51);

  // No more tags
  gifpacket_terminate(&gifpacket);
  iElementStart+=gifpacket_size(&gifpacket);

  setCurrentElement("SPHERE_SEGMENTS_RADIUS", iElementStart);
  qw.f32[0]=50;
  qw.f32[1]=20;
  packet_data[iElementStart]=qw.ul128;
  iElementStart++;

  // Alpha enable/disable
  setCurrentElement("ALPHA", iElementStart);
  gifpacket_init(&gifpacket, packet_data + iElementStart);
  outputAddress("ENABLE", gifpacket_size(&gifpacket));
  gifpacket_addgsdata(&gifpacket, GIF_SET_TAG(NLOOP(2), EOP(1), PRE(0),
					      0, GIF_PACKED, NREG(1)));
  gifpacket_addgsdata(&gifpacket, AD_REG);
  reg.ul64[0] = 1;
  reg.ul64[1] = FBA_1_REG;
  gifpacket_addgspacked(&gifpacket, reg.ul128);
  outputAddress("ENABLE_ALPHA_1_REG", gifpacket_size(&gifpacket));
  reg.ui32[0] = (1 << 6) | (2 << 4) | (2 << 2) | (0 << 0);
  reg.ui32[1] =  64;
  reg.ul64[1] = ALPHA_1_REG;
  gifpacket_addgspacked(&gifpacket, reg.ul128);

  outputAddress("DISABLE", gifpacket_size(&gifpacket));
  gifpacket_addgsdata(&gifpacket, GIF_SET_TAG(NLOOP(2), EOP(1), PRE(0),
					      0, GIF_PACKED, NREG(1)));
  gifpacket_addgsdata(&gifpacket, AD_REG);
  reg.ul64[0] = 0;
  reg.ul64[1] = FBA_1_REG;
  gifpacket_addgspacked(&gifpacket, reg.ul128);
  reg.ui32[0] = (0 << 6) | (2 << 4) | (2 << 2) | (0 << 0);
  reg.ui32[1] =  64;
  reg.ul64[1] = ALPHA_1_REG;
  gifpacket_addgspacked(&gifpacket, reg.ul128);

  gifpacket_terminate(&gifpacket);
  iElementStart+=gifpacket_size(&gifpacket);


  // Z Depth test enable/disable
  setCurrentElement("DEPTH_TEST", iElementStart);
  gifpacket_init(&gifpacket, packet_data + iElementStart);
  outputAddress("ENABLE", gifpacket_size(&gifpacket));
  gifpacket_addgsdata(&gifpacket, GIF_SET_TAG(NLOOP(1), EOP(1), PRE(0),
					      0, GIF_PACKED, NREG(1)));
  gifpacket_addgsdata(&gifpacket, AD_REG);
  reg.ul64[0] = (1 << 16) | (PS2_GS_ZALWAYS << 17);
  reg.ul64[1] = TEST_1_REG;
  gifpacket_addgspacked(&gifpacket, reg.ul128);

  outputAddress("DISABLE", gifpacket_size(&gifpacket));
  gifpacket_addgsdata(&gifpacket, GIF_SET_TAG(NLOOP(1), EOP(1), PRE(0),
					      0, GIF_PACKED, NREG(1)));
  gifpacket_addgsdata(&gifpacket, AD_REG);
  reg.ul64[0] = PS2_GS_ZALWAYS << 17;
  reg.ul64[1] = TEST_1_REG;
  gifpacket_addgspacked(&gifpacket, reg.ul128);

  gifpacket_terminate(&gifpacket);
  iElementStart+=gifpacket_size(&gifpacket);

  setCurrentElement("256_QWORDS", iElementStart);
  //  iElementStart+=257;



  if (iElementStart+257>1024) {
    fprintf(stderr, "Too much data!  Element Start is %i\n", iElementStart);
    exit(-1);
  }

  output=fopen(argv[1], "wb");
	
  if (output==NULL)
    {
      fprintf(stderr, "Could not open file \"%s\"\n", argv[1]);
      return -1;
    }

  //	fwrite(packet_data, sizeof(packet_data)/sizeof(qword_t), sizeof(qword_t), output);
  //	iElementStart=1024;
  fwrite(packet_data, iElementStart, sizeof(qword_t), output);

  bytes=ftell(output);
  fclose(output);

  printf("Written %d bytes to \"%s\"\n", bytes, argv[1]);

  return 0;
}

