.include "vcl_sml.i"

; Bitmasks for various states
kStateInitialized     .assign 1
kStateFadingFromNoise .assign 2
kStateFadingToNoise   .assign 4
kStateSphereLOD	      .assign 8
kStateSphereRotateX   .assign 16
kStateSphereRotateY   .assign 32
kStateSphereRotateZ   .assign 64
kStateSphereFour      .assign 128
kStateTexturedTriangle .assign 256

;kStateGameMenu        .assign 8
;kStateGamePlaying     .assign 16
;kStateGamePaused      .assign 32
;kStateGameWon         .assign 64
;kStateGameLost        .assign 128
;kStateUnused1         .assign 256
;kStateUnused2         .assign 512
;kStateUnused3         .assign 1024
;kStateUnused4         .assign 2048
;kStateUnused5         .assign 4096
;kStateUnused6         .assign 8192
;kStateUnused7         .assign 16384
;kStateUnused8         .assign 32768

kDemoState .assign 1023

.macro KICK iAddress
   iaddiu iKick, vi00, \iAddress
   xgkick iKick
.endm

; Build a rotation about the X axis -- angle is stored in fAngle[x]
.macro MatrixXRotation mTarget, fAngle
    MatrixIdentity \mTarget
    AngleSinCos \fAngle, fSinCos
    add.y \mTarget[1], vf00, fSinCos
    sub.z \mTarget[1], vf00, fSinCos[x]
    add.y \mTarget[2], vf00, fSinCos[x]
    add.z \mTarget[2], vf00, fSinCos[y]
.endm

; Build a rotation about the Y axis -- angle is stored in fAngle[y]
.macro MatrixYRotation mTarget, fAngle
    MatrixIdentity \mTarget
    mr32 fYAngle, \fAngle
    AngleSinCos fYAngle, fSinCos

    add.x \mTarget[0], vf00, fSinCos[y]
    add.z \mTarget[0], vf00, fSinCos[x]
    sub.x \mTarget[2], vf00, fSinCos
    add.z \mTarget[2], vf00, fSinCos[y]
.endm

; Build a rotation about the Z axis -- angle is stored in fAngle[z]
.macro MatrixZRotation mTarget, fAngle
    MatrixIdentity \mTarget
    mr32 fZAngle, \fAngle
    mr32 fZAngle, fZAngle
    AngleSinCos fZAngle, fSinCos

    add.x \mTarget[0], vf00, fSinCos[y]
    sub.y \mTarget[0], vf00, fSinCos[x]
    add.x \mTarget[1], vf00, fSinCos[x]
    add.y \mTarget[1], vf00, fSinCos[y]
.endm

.macro MatrixCorrectAspect mTarget, mTransform
    MatrixCopy \mTarget, \mTransform
    add.x \mTarget[0], \mTarget[0], \mTarget[0]
    add.x \mTarget[1], \mTarget[1], \mTarget[1]
    add.x \mTarget[2], \mTarget[2], \mTarget[2]
    add.x \mTarget[3], \mTarget[3], \mTarget[3]
;    loi 2048
;    add.w \mTarget[0], \mTarget[0], i
;    add.w \mTarget[1], \mTarget[1], i
;    add.w \mTarget[2], \mTarget[2], i

;   MatrixIdentity mTemp
;   add.x mTemp[0], mTemp[0], mTemp[0]

;   MatrixMultiply \mTarget, \mTransform, mTemp

.endm

.macro GIF_NOP
  sub gifnop, vf00, vf00
  loi 32768
  add.x gifnop, gifnop, i
  ftoi0 gifnop, gifnop
  sq gifnop, kSCRATCH_QWORD(vi00)
  add vf00, vf00, vf00
  KICK kSCRATCH_QWORD
.endm

.macro ComputeLights fFinalColor, fAmbientLight, mLight1, mLight2, mLight3, fNormal, fMinValue, fMaxValue
  MatrixMultiplyVector fColor, \mLight1, \fNormal
  MatrixMultiplyVector fColor2, \mLight2, \fNormal
  add fColor, fColor, fColor2
  MatrixMultiplyVector fColor3, \mLight3, \fNormal
  add fColor, fColor, fColor3
  add fColor, fColor, \fAmbientLight
  max.xyz fColor, fColor, \fMinValue
  max.xyz fColor, fColor, \fAmbientLight
  mini.xyz \fFinalColor, fColor, \fMaxValue
.endm

.macro EnableAlphaBlending
; Turn on alpha blending
	KICK kALPHA_ENABLE
.endm

.macro DisableAlphaBlending
; Turn off alpha blending
	KICK kALPHA_DISABLE
.endm

.macro EnableDepthTest
; Turn on alpha blending
	KICK kDEPTH_TEST_ENABLE
.endm

.macro DisableDepthTest
; Turn off alpha blending
	KICK kDEPTH_TEST_DISABLE
.endm

; This macro is used when fading between scene One to scene Two.
; The amount of scene One to show is given by iAlphaValue
; This same is amount is passed to SetAlphaFadeTwo which will 
; flip it for the contribution of scene two. 
.macro SetAlphaFadeOne iAlphaValue
       isw.y \iAlphaValue, kALPHA_ENABLE_ALPHA_1_REG(vi00)
       kick kALPHA_ENABLE
.endm

; This macro is used when fading between scene One to scene Two.
; The amount of scene One to show is given by iAlphaValue
; This same is amount is passed to SetAlphaFadeOne.
.macro SetAlphaFadeTwo iAlphaValue
       iaddi _SAFTiAlpha, vi00, 127
       isub _SAFTiAlpha, _SAFTiAlpha, \iAlphaValue
       isw.y _SAFTiAlpha, kALPHA_ENABLE_ALPHA_1_REG(vi00)
       kick kALPHA_ENABLE
.endm

; This will branch to a label if the specified state bit kStateNumber
; is *not* set
.macro BranchUnlessState kStateNumber, lTarget
       ilw.x _SUSiState, kSTATE(vi00)
       iaddiu _SUSiCheck, vi00, \kStateNumber
       iand _SUSiState, _SUSiState, _SUSiCheck
       ibne _SUSiState, _SUSiCheck, \lTarget
.endm


; This will branch to a label if the specified state bit kStateNumber
; *is* set
.macro BranchIfState kStateNumber, lTarget
       ilw.x _SISiState, kSTATE(vi00)
       iaddiu _SISiCheck, vi00, \kStateNumber
       iand _SISiState, _SISiState, _SISiCheck
       ibeq _SISiState, _SISiCheck, \lTarget
.endm

; Turns on a state
.macro SetState kStateNumber
       ilw.x _SSiState, kSTATE(vi00)
       iaddiu _SSiNewState, vi00, \kStateNumber
       ior _SSiState, _SSiState, _SSiNewState
       isw.x _SSiState, kSTATE(vi00)
.endm

; Turns off a state
.macro ClearState kStateNumber
       ilw.x _CSiState, kSTATE(vi00)
       isubiu _CSiNewState, vi00, \kStateNumber
       iaddi _CSiNewState, _CSiNewState, -1
       iand _CSiState, _CSiState, _CSiNewState
       isw.x _CSiState, kSTATE(vi00)
.endm
