
#include <stdio.h>
#include "defines.h"
#include "packet.h"
#include "geommath.h"
#include <math.h>

#include "model.h"

Mat44 makePerspectiveMatrix()
{
	Mat44 perspective;

	float n = 0.1f;
	float f = 1000.0f;
	float l = -0.06f * (2048.0f / (640.0f * 0.5f));
	float r =  0.06f * (2048.0f / (640.0f * 0.5f));
	float t =  0.06f * (2048.0f / (350.0f * 0.5f));
	float b = -0.06f * (2048.0f / (350.0f * 0.5f));

	LoadIdentity(&perspective);
	SetRow(&perspective,0,Vec4((2*n) / (r-l)	, 0.0f				, 0.0f				, 0.0f));
	SetRow(&perspective,1,Vec4(0.0f				, -(2*n) / (t-b)	, 0.0f				, 0.0f));
	SetRow(&perspective,2,Vec4(((r+l) / (r-l))	, ((t+b) / (t-b))	, ((f+n) / (f-n))	, -1.0f));
	SetRow(&perspective,3,Vec4(0.0f				, 0.0f				, ((2*f*n) / (f-n))	, 0.0f));
	return (perspective);
}

int main(int argc, char **argv)
{
	FILE *output;
	int bytes=0;
	int prim=0;
	qword_t colour;
	qword_t scales;
	qword_t reg;
	int i,j;

	u_long128 packet_data[1024];

	gifpacket_t gifpacket;

	for (i=0; i<1024; i++)
	{
		packet_data[i] = 0;
	}

	if (argc!=2)
	{
		fprintf(stderr, "Usage: gen_data output_file\n");
		return -1;
	}

	gifpacket_init(&gifpacket, packet_data);

	for (i=0; i<NUM_VERTS; i++)
	{
		gifpacket_addgsvec4(&gifpacket, &(texts[i]));
		gifpacket_addgsvec4(&gifpacket, &(verts[i]));
	}

	prim=GS_SET_PRIM(GS_PRIM_TRI, GOURAUD(1), TEXTURED(0), FOG(0), ABE(0), ALPHA(0), AA(0), CTXT(0), FIX(0));
	gifpacket_addgsdata(&gifpacket, GIF_SET_TAG(NLOOP(NUM_VERTS), EOP(1), PRE(1), prim, GIF_PACKED, NREG(3)));
	gifpacket_addgsdata(&gifpacket, 0x512);

	for (i=0; i<NUM_VERTS; i++)
	{
		colour.u32[0] = 127 * (verts[i].z<0);
		colour.u32[1] = 127 * (verts[i].z>0);
		colour.u32[2] = 0;
		colour.u32[3] = 0x80;
		gifpacket_addgspacked(&gifpacket, 0); // Leave a gap for transformed STQ co-ords
		gifpacket_addgspacked(&gifpacket, colour.u128);
		gifpacket_addgspacked(&gifpacket, 0); // Leave a gap for transformed vertex co-ords
	}

	Mat44 objectToWorld;
	LoadIdentity(&objectToWorld);
	gifpacket_addgsmat44(&gifpacket, &objectToWorld);

	Mat44 worldToCamera;
	LoadIdentity(&worldToCamera);
	SetRow(&worldToCamera, 3, Vec4(0,0,-5.0f,1));
	gifpacket_addgsmat44(&gifpacket, &worldToCamera);

	Mat44 cameraToScreen = makePerspectiveMatrix();
	gifpacket_addgsmat44(&gifpacket, &cameraToScreen);

	Mat44 rotate;
	LoadRotation(&rotate, 0.1f, 0.05f, 0.025f);
	gifpacket_addgsmat44(&gifpacket, &rotate);

	scales.f32[0] = 2048.0f;
	scales.f32[1] = 2048.0f;
	scales.f32[2] = (float)((2<<24)-1)/2;
	scales.f32[3] = 1.0f;
	gifpacket_addgspacked(&gifpacket,scales.u128);

	gifpacket_terminate(&gifpacket);

	output=fopen(argv[1], "wb");

	if (output==NULL)
	{
		fprintf(stderr, "Could not open file \"%s\"\n", argv[1]);
		return -1;
	}

	fwrite(packet_data, sizeof(packet_data)/sizeof(qword_t), sizeof(qword_t), output);

	bytes=ftell(output);
	fclose(output);

	printf("Written %d bytes to \"%s\"\n", bytes, argv[1]);

	return 0;
}
