// makeimage.cpp 
// creates winds.dat and winds.i files
// Copyright(c) 2002 Martin Piper (mindor@playstation2-linux.com)
// This file is subject to the terms and conditions of the GNU General
// Public License Version 2.  
//---------------------------------------------------------------------

#include <stdio.h>
#include <stdlib.h>

#include "gs.h"

#define INC_TEXT "%-20s.assign%20d ; %03x \n"

typedef struct {
  float x;
  float y;
  float z;
  int	w;
} t_vert;

typedef struct {
  float x;	//x offset
  float y;	//y offset
  int l;        //location in memory to find letter data  
  int ctl;	//ctl- 1 =  end of word/statement
} letter;

main () {
   FILE *dat;
   FILE *inc;
   int mem_loc=0;
   
   //file holding the data image for the winds.   
   if( (dat = fopen("winds.dat", "wb+")) < 0) {
     printf("Can't open winds.dat\n");
     exit(-1);
   }
   //file holding the memory map & other constatnts for the winds.
   //I write this while setting up the data image so I can rearrange what
   //order data shows up in without having to edit more than one file.

   if( (inc = fopen("winds.i", "w+")) < 0){
     printf("Can't open winds.i\n");
     exit(-1);
   }
   //int cont[4] = {0,500,0,0};
   int cont[4] = {0,300,0,0};
   fwrite(&cont[0], sizeof(int), 4,dat);
   fprintf(inc, INC_TEXT, "CONT1", mem_loc, mem_loc);
   mem_loc++;

   //float cont2[4] = {0.5, 20.0, 0.0, 0.0,};	
   float cont2[4] = {0.00, 6.0, 0.0, 0.0,};	//angle x rot, distance from 0, angle y rot, angle sun
   fwrite(&cont2[0], sizeof(float),4,dat);
   fprintf(inc, INC_TEXT, "CONT2", mem_loc, mem_loc);
   mem_loc++;

   float wind[4] = {0.f, 0.0,  1.f, .0f};
   // Wind will only travel || to ground, will be stored as normal,xyz with magnitude w
   fwrite(&wind[0], sizeof(float),4,dat);
   fprintf(inc, INC_TEXT, "WIND", mem_loc, mem_loc);
   mem_loc++;
   float gravity[4] = {0.0f, 0.0f, 0.0f, 0.0f};
   // Wind will only travel || to ground, will be stored as normal,xyz with magnitude w
   fwrite(&gravity[0], sizeof(float),4,dat);
   fprintf(inc, INC_TEXT, "GRAVITY", mem_loc, mem_loc);
   mem_loc++;

   fprintf(inc, INC_TEXT, "COLORS",mem_loc, mem_loc);
   float color[4];
   color[0] = 0.0;
   color[1] = 0.0;
   color[2] = 0.0;
   color[3] = 1.0;
   fwrite(&color[0],sizeof(float), 4,dat);
   fprintf(inc, INC_TEXT, "BLACK", mem_loc, mem_loc);
   mem_loc++;



   color[0] = 205.0f;
   color[1] = 205.0f;
   color[2] = 205.0f;
   fwrite(&color, sizeof(float), 4,dat);
   fprintf(inc, INC_TEXT, "WHITE", mem_loc, mem_loc);
   mem_loc++;

   color[0]=  150.0f;
   color[1]=  0.0f;
   color[2]=  0.0f;
   color[3]=  1.0f;
   
   fwrite(&color, sizeof(float), 4,dat);
   fprintf(inc, INC_TEXT, "RED", mem_loc, mem_loc);
   mem_loc++;
   color[0]=	100.0f;
   color[1]=	50.0f;
   color[2]=    0.0f;
   color[3]=    1.0f;
   fwrite(&color, sizeof(float), 4,dat);
   fprintf(inc, INC_TEXT, "BROWN", mem_loc, mem_loc);
   mem_loc++;
   color[0]=	50.0f;
   color[1]=	50.0f;
   color[2]=    50.0f;
   color[3]=    1.0f;
   fwrite(&color, sizeof(float), 4,dat);
   fprintf(inc, INC_TEXT, "GREY", mem_loc, mem_loc);
   mem_loc++;
   
   //////////////////////////////////////////////////////////////////
   //                             GIFTAGS                          //
   //////////////////////////////////////////////////////////////////
   int giftag[4];
   //////////////////////////////////////////////////////////////////
   //                        TRIANGLE GIFTAG                       //
   //////////////////////////////////////////////////////////////////
   giftag[0] = GIFTAG31_0(0,1);
   giftag[1] = GIFTAG63_32(4,0,(GS_PRIM_TRI|GS_PRIM_ANTA_ON),1);
   //giftag[1] = GIFTAG63_32(4,0,(GS_PRIM_TRI),1);
     giftag[2] = GIFTAG_REGS(0,0,0,0,GSR_XYZ2,GSR_XYZ2,GSR_XYZ2,GSR_RGBAQ);
   giftag[3] = 0;
   fwrite( &giftag[0], sizeof(int), 4,dat);
   fprintf(inc,INC_TEXT, "GT_TRIANGLE", mem_loc, mem_loc);
   (mem_loc++);      
   //////////////////////////////////////////////////////////////////
   //                    TRIANGLE STRIP GIFTAG                     //
   //////////////////////////////////////////////////////////////////
   giftag[0] = GIFTAG31_0(0,1);
   giftag[1] = GIFTAG63_32(2,0,(GS_PRIM_TRI_S|GS_PRIM_ANTA_ON),1);
   //giftag[1] = GIFTAG63_32(2,0,(GS_PRIM_TRI_S),1);
  
   giftag[2] = GIFTAG_REGS(0,0,0,0,0,0,GSR_XYZ2,GSR_RGBAQ);
   giftag[3] = 0;
   fwrite( &giftag[0], sizeof(int), 4,dat);
   fprintf(inc,INC_TEXT, "GT_TRIANGLE_STRIP", mem_loc, mem_loc);
   (mem_loc++);      
   //////////////////////////////////////////////////////////////////
   //                       LINE STRIP GIFTAG                      //
   //////////////////////////////////////////////////////////////////
   giftag[0] = GIFTAG31_0(0,1);
   giftag[1] = GIFTAG63_32(2,0,(GS_PRIM_LINE_S|GS_PRIM_ANTA_ON),1);
   //giftag[1] = GIFTAG63_32(2,0,(GS_PRIM_LINE_S),1);
  
   giftag[2] = GIFTAG_REGS(0,0,0,0,0,0,GSR_XYZ2,GSR_RGBAQ);
   giftag[3] = 0;
   fwrite( &giftag[0], sizeof(int), 4,dat);
   fprintf(inc,INC_TEXT, "GT_LINE_STRIP", mem_loc, mem_loc);
   (mem_loc++);      

   //////////////////////////////////////////////////////////////////
   //                           EOP   GIFTAG                       //
   //////////////////////////////////////////////////////////////////

   giftag[0] = GIFTAG31_0(1,0);
   giftag[1] =	0;
   giftag[2] =	0;
   giftag[3] =  0;

   fwrite( &giftag, sizeof(int), 4,dat);
   fprintf(inc,INC_TEXT, "GT_EOP", mem_loc, mem_loc);
   (mem_loc++); 
   
   float l2w[4][4] ={
     1.0f,	0.0f,		0.0f,	0.0f,
     0.0f,	1.0f,		0.0f,	0.0f,
     0.0,	0.0f,		1.0f,	0.0f,
     0.0f,	0.0f,		0.0f,   1.0f
   };

   fwrite( &l2w[0][0], sizeof(float),16, dat);
   fprintf(inc,INC_TEXT, "L2W0", mem_loc, mem_loc);
   (mem_loc++);
   fprintf(inc,INC_TEXT, "L2W1", mem_loc, mem_loc);
   (mem_loc++);
   fprintf(inc,INC_TEXT, "L2W2", mem_loc, mem_loc);
   (mem_loc++);
   fprintf(inc,INC_TEXT, "L2W3", mem_loc, mem_loc);
   (mem_loc++);

   int marker[4] = {0xffffffff,0xffffffff,0xffffffff,0xffffffff};
   fwrite(&marker[0], sizeof(int), 4, dat);
   mem_loc++;

   float w2c[4][4] ={
     1.0f,	0.0f,  	0.0f,	0.0f,
     0.0f,	1.0f,  	0.0f,	0.0f,
     0.0f,	0.0f,  	1.0f, 	0.0f,
     0.0f,	0.0f,  	120.0f, 1.0f
   };

   fwrite( &w2c[0][0], sizeof(float),16, dat);

   fprintf(inc,INC_TEXT, "W2C0", mem_loc, mem_loc);
   (mem_loc++);
   fprintf(inc,INC_TEXT, "W2C1", mem_loc, mem_loc);
   (mem_loc++);
   fprintf(inc,INC_TEXT, "W2C2", mem_loc, mem_loc);
   (mem_loc++);
   fprintf(inc,INC_TEXT, "W2C3", mem_loc, mem_loc);
   (mem_loc++);

   fwrite(&marker[0], sizeof(int), 4, dat);
   mem_loc++;
   

   float w2cb[4][4] ={
     1.0f,	0.0f,  	0.0f,	0.0f,
     0.0f,	1.0f,  	0.0f,	0.0f,
     0.0f,	0.0f,  	1.0f, 	0.0f,
     0.0f,	0.0f,  	0.0f,  	1.0f
   };

   fwrite( &w2cb[0][0], sizeof(float),16, dat);

   fprintf(inc,INC_TEXT, "W2CB0", mem_loc, mem_loc);
   (mem_loc++);
   fprintf(inc,INC_TEXT, "W2CB1", mem_loc, mem_loc);
   (mem_loc++);
   fprintf(inc,INC_TEXT, "W2CB2", mem_loc, mem_loc);
   (mem_loc++);
   fprintf(inc,INC_TEXT, "W2CB3", mem_loc, mem_loc);
   (mem_loc++);
  
   float w2ca[4][4] ={
     1.0f,	0.0f,  	0.0f,	0.0f,
     0.0f,	1.0f,  	0.0f,	0.0f,
     0.0f,	0.0f,  	1.0f, 	0.0f,
     0.0f,	0.0f,  	120.0f, 1.0f
   };
   fwrite(&w2ca[0][0], sizeof(float), 16, dat);

   fprintf(inc,INC_TEXT, "W2CA0", mem_loc, mem_loc);
   (mem_loc++);
   fprintf(inc,INC_TEXT, "W2CA1", mem_loc, mem_loc);
   (mem_loc++);
   fprintf(inc,INC_TEXT, "W2CA2", mem_loc, mem_loc);
   (mem_loc++);
   fprintf(inc,INC_TEXT, "W2CA3", mem_loc, mem_loc);
   (mem_loc++);

      float c2s[4][4] = {
     512.0f, 	0.0f, 		0.0f,	0.0f, 
     0.0f, 	240.6f, 	0.0f, 	0.0f, 
     2048.0f, 	2048.0f,     -255.0f,	1.0f, 
     0.0, 	0.0, 	 16777466.0f,	0.0f 
   };
   fwrite(c2s, sizeof(float), 16, dat);
   fprintf(inc,	INC_TEXT,"C2S",mem_loc, mem_loc);
   mem_loc+=4;


   int flag_height = 9;
   int flag_width = 11;
   int vert_counter= 1;
   float flag_math[4] = {0.5f, 0.866025404, 0.0, 0.0};
   float vert[4] = { 0.0f, 0.886025404, 0.0, 0.0};
   int fw,fh;
   flag_math[3] = 2* flag_math[1];
   int flag_setup2[4] = {flag_height/2,flag_height,flag_width, 0};
   
   fwrite(&flag_math[0],sizeof(float),4, dat);
   fprintf(inc,INC_TEXT, "FLAG_SETUP1", mem_loc, mem_loc);
   mem_loc++;

   fwrite(&flag_setup2[0],sizeof(int),4, dat);
   fprintf(inc,INC_TEXT, "FLAG_SETUP2", mem_loc, mem_loc);
   mem_loc++;

#define FLAG_TEST "%d: %-5f,%-10f, \n"
   for(fw = 0; fw < (flag_width); fw++){
     for(fh = 0; fh < (flag_height/2); fh++){
     
       // printf(FLAG_TEST, vert_counter, vert[0], vert[1]);
       vert[1] += 2 * flag_math[1];
       vert_counter++;
     }
     vert[0] += flag_math[0];
     vert[1] = 0.0f;
     while( fh< flag_height){
       // printf(FLAG_TEST, vert_counter, vert[0], vert[1]);
       vert[1] += 2* flag_math[1];
       fh++;
       vert_counter++;
     }
     vert[0] += flag_math[0];
     vert[1] = flag_math[1];
   }
   for(fh = 0; fh < (flag_height/2); fh++){
     
     // printf(FLAG_TEST, vert_counter, vert[0], vert[1]);
       vert[1] += 2 * flag_math[1];
       vert_counter++;
   }
   vert_counter--;  
   int num_rope_verts = 10;
   vert_counter+= num_rope_verts;
   fprintf(inc,	INC_TEXT, "NUM_FLAG_VERTS", (vert_counter), (vert_counter));
   int triangle_counter = 1;
   int triangle_number = 1;
   while(triangle_counter <= 23){
     printf("%d, ", triangle_number);
     triangle_number+= 4;
     triangle_counter++;
     printf("%d, ", triangle_number);
     triangle_number+= 5;
     triangle_counter++;
   }
   printf("\n");
   triangle_counter = 1;
   triangle_number = 1;
   while(triangle_counter <= 23){
     printf("%d, ", triangle_number);
     triangle_number+= 5;
     triangle_counter++;
     printf("%d, ", triangle_number);
     triangle_number+= 4;
     triangle_counter++;
   }
   printf("\n");
   int flag_strips[16][4] ={
     1,		23, 	1,	0,
     1, 	23, 	1,	1,     
     2,		10,	1,	0,
     2,		9,	1,	1,
     3,		8,	1,	0,
     3,		7,	1,	1,
     4,		23,	1,	0,
     4,		23,	1,	1,
     38,	10,	2,	0,
     74,	7,	1,	0,
     34,	10,	2,	0,
     70,	8,	1,	0,
     30,	10,	2,	0,
     66,	9,	1,	0,
     26,       	10,	2,	0,
     62,	10,	1,	0,
       
   };

   fwrite(&flag_strips[0], sizeof(int), 16*4, dat);
   fprintf(inc, INC_TEXT, "FLAG_STRIPS", mem_loc, mem_loc);
   mem_loc+=16;
    
   int	flag_rope_sections[3][4]={
     0,		5,	1,	0,
     9,		104,	1,	0,
     104,	112,	0,	0,	
     //     0,		5,	1,	0
   };
   


   fwrite(&flag_rope_sections[0], sizeof(int), 3*4, dat);
   fprintf(inc, INC_TEXT, "FLAG_ROPE_SECTIONS", mem_loc, mem_loc);
   mem_loc+= 3;
  
   int flag_point[102][4]={
     5  , 1  , 10 , 14 ,
     14 , 10 , 19 , 23 ,
     23 , 19 , 28 , 32 ,
     32 , 28 , 37 , 41 ,
     41 , 37 , 46 , 50 ,
     50 , 46 , 55 , 59 ,
     59 , 55 , 64 , 68 ,
     68 , 64 , 73 , 77 ,
     77 , 73 , 82 , 86 ,
     86 , 82 , 91 , 95 ,
     95 , 91 , 100, 0  ,
     1  , 6  , 10 , 0  ,
     10 , 6  , 15 , 19 ,
     19 , 15 , 24 , 28 ,
     28 , 24 , 33 , 37 ,
     37 , 33 , 42 , 46 ,
     46 , 42 , 51 , 55 ,
     55 , 51 , 60 , 64 ,
     64 , 60 , 69 , 73 ,
     73 , 69 , 78 , 82 ,
     82 , 78 , 87 , 91 ,
     91 , 87 , 96 , 100,
     100, 96 , 0  , 0  ,
     6  , 2  , 11 , 15 ,
     15 , 11 , 20 , 24 ,
     24 , 20 , 29 , 33 ,
     33 , 29 , 38 , 42 ,
     42 , 38 , 47 , 51 ,
     51 , 47 , 56 , 60 ,
     60 , 56 , 65 , 69 ,
     69 , 65 , 74 , 78 ,
     78 , 74 , 83 , 87 ,
     87 , 83 , 92 , 96 ,
     96 , 92 , 101, 0  ,
     2  , 7  , 11 , 0  ,
     11 , 7  , 16 , 20 ,
     20 , 16 , 25 , 29 ,
     29 , 25 , 34 , 38 ,
     38 , 34 , 43 , 47 ,
     47 , 43 , 52 , 56 ,
     56 , 52 , 61 , 65 ,
     65 , 61 , 70 , 74 ,
     74 , 70 , 79 , 83 ,
     83 , 79 , 88 , 92 ,
     92 , 88 , 97 , 101,
     101, 97 , 0  , 0  ,
     7  , 3  , 12 , 16 ,
     16 , 12 , 21 , 25 ,
     25 , 21 , 30 , 34 ,
     34 , 30 , 39 , 43 ,
     43 , 39 , 48 , 52 ,
     52 , 48 , 57 , 61 ,
     61 , 57 , 66 , 70 ,
     70 , 66 , 75 , 79 ,
     79 , 75 , 84 , 88 ,
     88 , 84 , 93 , 97 ,
     97 , 93 , 102, 0  ,
     3  , 8  , 12 , 0  ,
     12 , 8  , 17 , 21 ,
     21 , 17 , 26 , 30 ,
     30 , 26 , 35 , 39 ,
     39 , 35 , 44 , 48 ,
     48 , 44 , 53 , 57 ,
     57 , 53 , 62 , 66 ,
     66 , 62 , 71 , 75 ,
     75 , 71 , 80 , 84 ,
     84 , 80 , 89 , 93 ,
     93 , 89 , 98 , 102,
     102, 98 , 0  , 0  ,
     8  , 4  , 13 , 17 ,
     17 , 13 , 22 , 26 ,
     26 , 22 , 31 , 35 ,
     35 , 31 , 40 , 44 ,
     44 , 40 , 49 , 53 ,
     53 , 49 , 58 , 62 ,
     62 , 58 , 67 , 71 ,
     71 , 67 , 76 , 80 ,
     80 , 76 , 85 , 89 ,
     89 , 85 , 94 , 98 ,
     98 , 94 , 103, 0  ,
     4  , 9  , 13 , 0  ,
     13 , 9  , 18 , 22 ,
     22 , 18 , 27 , 31 ,
     31 , 27 , 36 , 40 ,
     40 , 36 , 45 , 49 ,
     49 , 45 , 54 , 58 ,
     58 , 54 , 63 , 67 ,
     67 , 63 , 72 , 76 ,
     76 , 72 , 81 , 85 ,
     85 , 81 , 90 , 94 ,
     94 , 90 , 99 , 103,
     103, 99 , 0  , 0  ,
     9, 18,	0,   0,
     18,   27,  0,  0,
     27,   36,  0,  0,
     36,   45,  0,  0,
     45,   54,  0,  0,
     54,   63,0,  0,
     63,   72, 0,  0,
     72,   81, 0,  0,
     81,   92, 0,  0,
     92,   99,0,  0,


   };
   int rope_sticks[8][4] = {
     112,111,111,110,
     110,109,109,108,
     108,107,106,105,
     104,9,9,4,
     4,8,8,3,
     3,7,7,2,
     2,6,6,1,
     1,5,5,0,
   } ; 

   fwrite(&flag_point[0], sizeof(int),102*4,dat);
   fprintf(inc, INC_TEXT, "FLAG_STICKS", mem_loc,mem_loc);
   mem_loc+=102;
   fwrite(&rope_sticks,	sizeof(int),8*4,dat);
   fprintf(inc, INC_TEXT, "ROPE_STICKS", mem_loc, mem_loc);
   mem_loc+=8;
   fprintf(inc,	INC_TEXT, "NUM_ROPE_STRIPS", 3,	3);
   fprintf(inc, INC_TEXT, "NUM_FLAG_STRIPS", 16, 16);
   fprintf(inc,	INC_TEXT, "FLAG_VERTS", mem_loc, mem_loc);   
   int	main_mem_loc = mem_loc+vert_counter;
   fprintf(inc,	INC_TEXT, "FLAG_VERTS_PREV", main_mem_loc, main_mem_loc);
   main_mem_loc+= vert_counter;
   fprintf(inc, INC_TEXT, "FLAG_VERTS_CALCS", main_mem_loc, main_mem_loc);
   main_mem_loc+= vert_counter;
   fprintf(inc,	INC_TEXT, "FLAG_TRIANGLE_DATA",	main_mem_loc,	main_mem_loc);
   main_mem_loc+= 176;
   fprintf(inc,	INC_TEXT, "NUM_FLAG_TRIANGLES",	176,	176);
   fprintf(inc, INC_TEXT, "MAIN_O_BUFFER", (main_mem_loc),(main_mem_loc));
  ;fprintf(inc, INC_TEXT, "MAIN_D_BUFFER", (main_mem_loc), (main_mem_loc));

   /////////////////////////////////////////////////////
   //                   START INTRO DATA               /
   /////////////////////////////////////////////////////
   fprintf(inc, INC_TEXT, "START_INTRO_DATA", mem_loc, mem_loc);
   float rhombus[4][4] = {
     -15.0f, 	0.0f,		0.0f,	1,
     5.0f,	0.0f,		0.0f,	1,
     -5.0f,	-17.32050808f,	0.0f,	1,
     15.0f,	-17.32050808f,	0.0f,	1,
     
   };
   

   fwrite(&rhombus[0], sizeof(float), 16, dat);
   fprintf(inc,INC_TEXT, "RHOMBUS", mem_loc, mem_loc);
   mem_loc += 4;

   int l_vsa = mem_loc;    
   float vert_stroke_a[5][4]={

//  1  2 3 4
      1.050007,  //x 1	 
     80.807979,  //y
     
     
//  5  6 7 8
     0.080112,  //x 
    77.797478,  //y

     2.130314,  //x 2	 
    71.756970,  //y

     5.160720,  //x18
    65.666261,  //y

     9.211224,  //x 3	 
    59.585452,  //y

    15.261831,  //x
    50.494544,  //y

    22.342538,  //x 4	 
    40.393534,  //y

    29.413345,  //x
    30.282423,  //y

    37.484152,  //x 5	 
    18.171311,  //y

    41.543840,  //x
    08.030603,  //y

   };   
   //storage per qword x1.x2x3x4, y1.y2y3y4, x5.x6x7x8,	y5.y6y7y8
   fwrite(&vert_stroke_a[0], sizeof(float), (5*4), dat);
   fprintf(inc,INC_TEXT, "VERTICAL_STROKE_A", mem_loc, mem_loc);
   mem_loc+= 5;
   
   int l_vsb = mem_loc;
   float vert_stroke_b[5][4]={

//  1  2 3 4
    13.261328,  //x 1	 
    77.767575,  //y
//  5  6 7 8
    13.281426,  //x 
    73.727070,  //y

    16.261826,  //x 2	 
    66.656162,  //y

    21.312433,  //x
    58.585454,  //y

    26.373141,  //x 3	 
    49.494443,  //y

    34.443848,  //x
    39.393432,  //y

    41.534556,  //x 4	 
    28.262321,  //y

    48.595262,  //x
    17.171112,  //y

    52.645465,  //x 5	 
     9.070604,  //y

    54.665363,  //x
    03.020200,  //y

   };   
   //storage per qword x1.x2x3x4, y1.y2y3y4, x5.x6x7x8,	y5.y6y7y8
   fwrite(&vert_stroke_b[0], sizeof(float), (5*4), dat);
   fprintf(inc,INC_TEXT, "VERTICAL_STROKE_B", mem_loc, mem_loc);
   mem_loc+= 5;
   
   int l_pla = mem_loc;
   float p_loop_a[5][4]={

//  1  2 3 4
    65.656767,  //x 1	 
    00.060006,  //y
//  5  6 7 8
    69.697272,  //x 
    00.050005,  //y

    77.738173,  //x 2	 
    01.060509,  //y

    83.738271,  //x
    10.131616,  //y

    81.707969,  //x 3	 
    19.192321,  //y

    76.667164,  //x
    26.243128,  //y

    65.606157,  //x 4	 
    35.323836,  //y

    57.555552,  //x
    40.374139,  //y

    52.514948,  //x 5	 
    39.434045,  //y

    47.444139,  //x
    40.454045,  //y

   };   
   //storage per qword x1.x2x3x4, y1.y2y3y4, x5.x6x7x8,	y5.y6y7y8
   fwrite(&p_loop_a[0], sizeof(float), (5*4), dat);
   fprintf(inc,INC_TEXT, "P_LOOP_A", mem_loc, mem_loc);
   mem_loc+= 5;
   
   int l_plb = mem_loc;
   float p_loop_b[5][4]={

//  1  2 3 4
    73.537986,  //x 1	 
    00.000301,  //y
//  5  6 7 8
    81.898291,  //x 
    05.040807,  //y
    //////////////////
    83.948292,  //x 2	 
    10.101316,  //y

    82.908187,  //x
    16.201924,  //y
    ////////////////////
    80.857983,  //x 3	 
    21.262328,  //y

    76.797376,  //x
    26.322935,  //y
    /////////////////////
    71.736870,  //x 4	 
    31.373339,  //y

    65.656161,  //x
    35.413843,  //y
    //////////////////
    57.585556,  //x 5	 
    40.444144,  //y

    53.545148,  //x
    42.454345,  //y

   };   
   //storage per qword x1.x2x3x4, y1.y2y3y4, x5.x6x7x8,	y5.y6y7y8
   fwrite(&p_loop_b[0], sizeof(float), (5*4), dat);
   fprintf(inc,INC_TEXT, "P_LOOP_B", mem_loc, mem_loc);
   mem_loc+= 5;
   
   int l_rd = mem_loc;
   float r_dongle[5][4]={

//  1  2 3 4
    39.354037,  //x 1	 
    45.505053,  //y
//  5  6 7 8
    40.374137,  //x 
    54.575862,  //y

    43.384540,  //x 2	 
    62.686674,  //y

    48.455152,  //x
    69.797180,  //y

    55.595863,  //x 3	 
    71.767070,  //y

    65.586255,  //x
    64.705971,  //y

    54.555148,  //x 4	 
    62.716269,  //y

    49.454843,  //x
    60.665962,  //y

    46.414540,  //x 5	 
    56.585354,  //y

    45.404439,  //x
    50.504545,  //y

   };   
   //storage per qword x1.x2x3x4, y1.y2y3y4, x5.x6x7x8,	y5.y6y7y8
   fwrite(&r_dongle[0], sizeof(float), (5*4), dat);
   fprintf(inc,INC_TEXT, "R_DONGLE", mem_loc, mem_loc);
   mem_loc+= 5;

   
   int l_tt = mem_loc;
   float t_top[5][4]={

//  1  2 3 4
    14.191522,  //x 1	 
    26.252122,  //y
//  5  6 7 8
    19.242126,  //x 
    16.191317,  //y

    23.292531,  //x 2	 
    11.140812,  //y

    27.333036,  //x
     3.100009,  //y

    38.404447,  //x 3	 
     0.080007,  //y

    54.596365,  //x
     0.070007,  //y

    70.718572,  //x 4	 
    00.070009,  //y

    86.7282721, //x
     3.121015,  //y

    78.7076701, //x 5	 
    16.182021,  //y

    74.687568,  //x
    24.232626   //y

   };   
   //storage per qword x1.x2x3x4, y1.y2y3y4, x5.x6x7x8,	y5.y6y7y8
   fwrite(&t_top[0], sizeof(float), (5*4), dat);
   fprintf(inc,INC_TEXT, "T_TOP", mem_loc, mem_loc);
   mem_loc+= 5;
   
   int l_dla  = mem_loc;
   float d_loop_a[5][4]={

//  1  2 3 4
    19.242728,  //x 1	 
    78.697872,  //y
//  5  6 7 8
    33.343837,  //x 
    77.707567,  //y

    44.435048,  //x 2	 
    70.636558,  //y

    55.526056,  //x
    60.525548,  //y

    64.606964,  //x 3	 
    50.414436,  //y

    73.667669,  //x
    36.323428,  //y

    79.718173,  //x 4	 
    29.242318,  //y

    82.748274,  //x
    17.141211,  //y

    81.747773,  //x 5	 
     9.080506,  //y

    72.696664,  //x
     0.050005   //y

   };   
   //storage per qword x1.x2x3x4, y1.y2y3y4, x5.x6x7x8,	y5.y6y7y8
   fwrite(&d_loop_a[0], sizeof(float), (5*4), dat);
   fprintf(inc,INC_TEXT, "D_LOOP_A", mem_loc, mem_loc);
   mem_loc+= 5;

   
   int l_dlb = mem_loc;
   float d_loop_b[5][4]={

//  1  2 3 4
    38.414544,  //x 1	 
    75.737370,  //y
//  5  6 7 8
    50.505655,  //x 
    70.656660,  //y

    61.576461,  //x 2	 
    63.576153,  //y

    69.647266,  //x
    56.505347,  //y

    75.697873,  //x 3	 
    50.444639,  //y

    83.768779,  //x
    41.343529,  //y

    90.819281,  //x 4	 
    29.232320,  //y

    93.829382,  //x
    18.171112,  //y

    92.818977,  //x 5	 
     8.090405,  //y

    85.748072,  //x
     2.0300001   //y

   };   
   //storage per qword x1.x2x3x4, y1.y2y3y4, x5.x6x7x8,	y5.y6y7y8
   fwrite(&d_loop_b[0], sizeof(float), (5*4), dat);
   fprintf(inc,INC_TEXT, "D_LOOP_B", mem_loc, mem_loc);
   mem_loc+= 5;

   
   int l_bla = mem_loc;
   float b_loop_a[5][4]={

//  1  2 3 4
    54.616471,  //x 1	 
    31.312424,  //y
//  5  6 7 8
    67.756978,  //x 
    21.211818,  //y

    71.817381,  //x 2	 
    15.141210,  //y

    73.817378,  //x
    10.070704,  //y

    71.696266,  //x 3	 
    05.0307001, //y

    69.787888,  //x
    03.000402,  //y

    81.928191,  //x 4	 
    07.061015,  //y

    81.817880,  //x
    14.211825,  //y

    75.777173,  //x 5	 
    21.262428,  //y

    65.686164,  //x
    27.313131    //y

   };   
   //storage per qword x1.x2x3x4, y1.y2y3y4, x5.x6x7x8,	y5.y6y7y8
   fwrite(&b_loop_a[0], sizeof(float), (5*4), dat);
   fprintf(inc,INC_TEXT, "B_LOOP_A", mem_loc, mem_loc);
   mem_loc+= 5;

   
   int l_blb = mem_loc;
   float b_loop_b[5][4]={

//  1  2 3 4
    41.464754,  //x 1	 
    41.293931,  //y
//  5  6 7 8
    52.615466,  //x 
    38.314037,  //y

    54.665464,  //x 2	 
    43.424647,  //y

    52.615458,  //x
    49.525246,  //y

    49.554752,  //x 3	 
    56.595862,  //y

    45.484243,  //x
    61.656469,  //y

    40.413638,  //x 4	 
    67.717073,  //y

    34.353132,  //x
    71.747275,  //y

    28.282524,  //x 5	 
    73.797379,  //y

    22.201814,  //x
    73.7874781   //y

   };   
   //storage per qword x1.x2x3x4, y1.y2y3y4, x5.x6x7x8,	y5.y6y7y8
   fwrite(&b_loop_b[0], sizeof(float), (5*4), dat);
   fprintf(inc,INC_TEXT, "B_LOOP_B", mem_loc, mem_loc);
   mem_loc+= 5;
   
   int l_blc = mem_loc;
   float b_loop_c[5][4]={

//  1  2 3 4
    24.323635,  //x 1	 
    79.757974,  //y
//  5  6 7 8
    39.384241,  //x 
    78.737771,  //y

    46.435145,  //x 2	 
    75.697466,  //y

    54.485752,  //x
    72.657062,  //y

    60.556458,  //x 3	 
    68.596556,  //y

    68.597161,  //x
    61.545752,  //y

    73.627464,  //x 4	 
    55.495347,  //y

    75.657666,  //x
    51.454843,  //y

    77.667766,  //x 5	 
    44.403836,  //y

    73.646654,  //x
    33.343131    //y

   };   
   //storage per qword x1.x2x3x4, y1.y2y3y4, x5.x6x7x8,	y5.y6y7y8
   fwrite(&b_loop_c[0], sizeof(float), (5*4), dat);
   fprintf(inc,INC_TEXT, "B_LOOP_C", mem_loc, mem_loc);
   mem_loc+= 5;
   
   int l_ola = mem_loc;
   float o_loop_a[5][4]={

//  1  2 3 4
    13.091205,  //x 1	 
    79.797477,  //y
//  5  6 7 8
    12.021101,  //x 
    70.746770,  //y

    11.001101,  //x 2	 
    66.666561,  //y

    11.031408,  //x
    62.755246,  //y

    18.122216,  //x 3	 
    46.404035,  //y

    26.193023,  //x
    35.313127,  //y

    33.273732,  //x 4	 
    27.232318,  //y

    40.374442,  //x
    20.151711,  //y

    48.475150,  //x 5	 
    14.081106,  //y

    53.535656,  //x
    10.051004    //y

   };   
   //storage per qword x1.x2x3x4, y1.y2y3y4, x5.x6x7x8,	y5.y6y7y8
   fwrite(&o_loop_a[0], sizeof(float), (5*4), dat);
   fprintf(inc,INC_TEXT, "O_LOOP_A", mem_loc, mem_loc);
   mem_loc+= 5;

   int l_olb = mem_loc;
   float o_loop_b[5][4]={
//  1  2 3 4
    13.241324,  //x 1	 
    79.787571,  //y
//  5  6 7 8
    12.241124,  //x 
    70.666562,  //y

    12.241324,  //x 2	 
    60.595656,  //y

    16.261428,  //x
    48.514447,  //y

    23.322836,  //x 3	 
    39.403334,  //y

    31.393442,  //x
    29.302626,  //y

    37.454048,  //x 4	 
    23.232020,  //y

    44.504653,  //x
    17.181515,  //y

    50.555257,  //x 5	 
    12.141012,  //y

    60.566063,  //x
    09.040207    //y

   };   

   //storage per qword x1.x2x3x4, y1.y2y3y4, x5.x6x7x8,	y5.y6y7y8
   fwrite(&o_loop_b[0], sizeof(float), (5*4), dat);
   fprintf(inc,INC_TEXT, "O_LOOP_B", mem_loc, mem_loc);
   mem_loc+= 5;

   int l_olc = mem_loc;
   float o_loop_c[5][4]={


//  1  2 3 4
    24.242726,  //x 1	 
    78.667668,  //y
//  5  6 7 8
    30.313635,  //x 
    76.687365,  //y

    41.404644,  //x 2	 
    70.626758,  //y

    50.495556,  //x
    64.526047,  //y

    60.626466,  //x 3	 
    55.425139,  //y

    68.707274,  //x
    47.334127,  //y

    76.768077,  //x 4	 
    35.233019,  //y

    83.788578,  //x
    23.141709,  //y

    87.788678,  //x 5	 
    12.060604,  //y

    83.768177,  //x
    02.0201001   //y

   };   
   //storage per qword x1.x2x3x4, y1.y2y3y4, x5.x6x7x8,	y5.y6y7y8
   fwrite(&o_loop_c[0], sizeof(float), (5*4), dat);
   fprintf(inc,INC_TEXT, "O_LOOP_C", mem_loc, mem_loc);
   mem_loc+= 5;

   int l_old = mem_loc;
   float o_loop_d[5][4]={

//  1  2 3 4
    60.616364,  //x 1	 
     2.080107,  //y
//  5  6 7 8
    66.6769701, //x 
     1.060006,  //y

    73.717872,  //x 2	 
     0.070008,  //y

    79.727972,  //x
     4.090913,  //y

    78.717769,  //x 3	 
    13.171722,  //y

    76.677465,  //x
    22.272731,  //y

    71.636961,  //x 4	 
    31.343437,  //y

    67.596557,  //x
    37.393942,  //y

    63.556053,  //x 5	 
    42.454548 , //y

    56.5053501, //x
    48.525152    //y

   };   
   //storage per qword x1.x2x3x4, y1.y2y3y4, x5.x6x7x8,	y5.y6y7y8
   fwrite(&o_loop_d[0], sizeof(float), (5*4), dat);
   fprintf(inc,INC_TEXT, "O_LOOP_D", mem_loc, mem_loc);
   mem_loc+= 5;


   int l_sla = mem_loc;
   float s_loop_a[5][4]={

//  1  2 3 4
    75.777579,  //x 1	 
     7.080910,  //y
//  5  6 7 8
    73.777173, //x 
    11.151318,  //y

    68.716768,  //x 2	 
    15.201823,  //y

    71.737377,  //x
    20.241822,  //y

    77.827886,  //x 3	 
    15.181212,  //y

    78.867782,  //x
     9.070802,  //y

    74.767269,  //x 4	 
     7.0005001, //y

    65.636258,  //x
     5.000601,  //y

    58.545648,  //x 5	 
     8.021005, //y

    53.454451, //x
    14.111724    //y

   };   
   //storage per qword x1.x2x3x4, y1.y2y3y4, x5.x6x7x8,	y5.y6y7y8
   fwrite(&s_loop_a[0], sizeof(float), (5*4), dat);
   fprintf(inc,INC_TEXT, "S_LOOP_A", mem_loc, mem_loc);
   mem_loc+= 5;


   int l_slb = mem_loc;
   float s_loop_b[5][4]={

//  1  2 3 4
    47.404539,  //x 1	 
     6.101116,  //y
//  5  6 7 8
    44.385139, //x 
    17.222428,  //y

    55.426047,  //x 2	 
    28.333237,  //y

    63.516553,  //x
    36.424046,  //y

    65.526049,  //x 3	 
    48.545759,  //y

    52.425339,  //x
    54.514648,  //y

    51.354730,  //x 4	 
    42.443739,  //y

    42.283931,  //x
    33.312821,  //y

    38.3539401, //x 5	 
    22.1516101, //y

    56.5053501, //x
    48.525152    //y

   };   
   //storage per qword x1.x2x3x4, y1.y2y3y4, x5.x6x7x8,	y5.y6y7y8
   fwrite(&s_loop_b[0], sizeof(float), (5*4), dat);
   fprintf(inc,INC_TEXT, "S_LOOP_B", mem_loc, mem_loc);
   mem_loc+= 5;
   int l_slc = mem_loc;
   float s_loop_c[5][4]={

//  1  2 3 4
    12.161016,  //x 1	 
    53.535557,  //y
//  5  6 7 8
     8.150614, //x 
    58.596161,  //y

     4.120012,  //x 2	 
    65.687170,  //y

    00.130214,  //x
    74.727673,  //y

     8.161318,  //x 3	 
    79.738074,  //y

    19.222524,  //x
    80.748073,  //y

    30.273337,  //x 4	 
    79.727770,  //y

    39.344337,  //x
    75.687365,  //y

    48.3953411, //x 5	 
    70.6366601, //y

    57.4261421, //x
    62.5855551   //y

   };   
   //storage per qword x1.x2x3x4, y1.y2y3y4, x5.x6x7x8,	y5.y6y7y8
   fwrite(&s_loop_c[0], sizeof(float), (5*4), dat);
   fprintf(inc,INC_TEXT, "S_LOOP_C", mem_loc, mem_loc);
   mem_loc+= 5;
   int l_ml = mem_loc;
   float m_leg[5][4]={

//  1  2 3 4
    19.0121001, //x 1	 
    76.807278,  //y
//  5  6 7 8
    17.0116041, //x 
    71.746968,  //y

    17.0819101, //x 2	 
    64.626059,  //y

    21.132315,  //x
    58.555452,  //y

    24.1827201, //x 3	 
    51.5948461, //y

    29.253125,  //x
    45.4342401, //y

    33.273429,  //x 4	 
    40.383835,  //y

    36.323937,  //x
    36.323133,  //y

    34.393743, //x 5	 
    29.312629, //y

    40.4443481, //x
    22.241814    //y

   };   
   //storage per qword x1.x2x3x4, y1.y2y3y4, x5.x6x7x8,	y5.y6y7y8
   fwrite(&m_leg[0], sizeof(float), (5*4), dat);
   fprintf(inc,INC_TEXT, "M_LEG", mem_loc, mem_loc);
   mem_loc+= 5;
   int l_mva = mem_loc;
   float m_vert_a[5][4]={

//  1  2 3 4
    45.624362,  //x 1	 
     3.020706,  //y
//  5  6 7 8
    48.614859, //x 
     9.111415,  //y

    46.584456,  //x 2	 
    18.192224,  //y

    43.544253,  //x
    25.272831,  //y

    41.524152,  //x 3	 
    30.343436,  //y

    39.503848,  //x
    36.683942,  //y

    37.473646,  //x 4	 
    41.454447,  //y

    35.453343,  //x
    47.505354,  //y

    32.4231411, //x 5	 
    56.57615901,//y

    31.4032401, //x
    56.57615901  //y

   };   
   //storage per qword x1.x2x3x4, y1.y2y3y4, x5.x6x7x8,	y5.y6y7y8
   fwrite(&m_vert_a[0], sizeof(float), (5*4), dat);
   fprintf(inc,INC_TEXT, "M_VERT_A", mem_loc, mem_loc);
   mem_loc+= 5;
   int l_mvb = mem_loc;
   float m_vert_b[5][4]={

//  1  2 3 4
    70.627062,  //x 1	 
     0.020406,  //y
//  5  6 7 8
    69.616759, //x 
     9.111516,  //y

    65.576355,  //x 2	 
    21.222627,  //y

    61.525950,  //x
    31.333638,  //y

    58.475443,  //x 3	 
    42.454952,  //y

    49.405349,  //x
    56.636156,  //y

    57.546058,  //x 4	 
    54.494842,  //y

    65.596966,  //x
    41.383630,  //y

    72.6974721, //x 5	 
    31.2629221, //y

    78.7580821, //x
    27.192212    //y

   };   
   //storage per qword x1.x2x3x4, y1.y2y3y4, x5.x6x7x8,	y5.y6y7y8
   fwrite(&m_vert_b[0], sizeof(float), (5*4), dat);
   fprintf(inc,INC_TEXT, "M_VERT_B", mem_loc, mem_loc);
   mem_loc+= 5;
   int l_mvc = mem_loc;
   float m_vert_c[5][4]={

//  1  2 3 4
    23.352134,  //x 1	 
     2.030607,  //y
//  5  6 7 8
    24.332533, //x 
     8.090912,  //y

    24.322031,  //x 2	 
    13.151718,  //y

    20.301928,  //x
    22.222627,  //y

    18.271625,  //x 3	 
    29.313536,  //y

    15.231221,  //x
    38.404346,  //y

     9.180816,  //x 4	 
    49.515356,  //y

     6.140413,  //x
    58.616365,  //y

     3.1100111, //x 5	 
    68.7174751, //y

     0.1100051, //x
    77.7780801   //y

   };   
   //storage per qword x1.x2x3x4, y1.y2y3y4, x5.x6x7x8,	y5.y6y7y8
   fwrite(&m_vert_c[0], sizeof(float), (5*4), dat);
   fprintf(inc,INC_TEXT, "M_VERT_C", mem_loc, mem_loc);
   mem_loc+= 5;
   int l_mvd = mem_loc;
   float m_vert_d[5][4]={

//  1  2 3 4
    25.1128111,  //x 1	 
    76.7774751,  //y
//  5  6 7 8
    27.1124131, //x 
    71.7169651,  //y

    25.1426161,  //x 2	 
    63.6158561,  //y

    28.1830211,  //x
    55.5150461,  //y

    32.2334251,  //x 3	 
    45.4041361,  //y

    36.2738281,  //x
    36.3132271,  //y

    39.3041311,  //x 4	 
    29.2225181,  //y

    42.3245331,  //x
    22.1515121,  //y

    46.3347341, //x 5	 
    12.0909071, //y

    48.3548451, //x
    06.03010101   //y

   };   
   //storage per qword x1.x2x3x4, y1.y2y3y4, x5.x6x7x8,	y5.y6y7y8
   fwrite(&m_vert_d[0], sizeof(float), (5*4), dat);
   fprintf(inc,INC_TEXT, "M_VERT_D", mem_loc, mem_loc);
   mem_loc+= 5;
   int l_nva = mem_loc;
   float n_vert_a[5][4]={

//  1  2 3 4
     7.2005201, //x 1	 
    02.020606,  //y
//  5  6 7 8
    09.201019, //x 
     8.101215,  //y

    10.180718,  //x 2	 
    15.191922,  //y

    07.180717,  //x
    23.272832,  //y

    07.160616,  //x 3	 
    31.353638,  //y

     4.150415,  //x
    42.424647,  //y

     3.140214,  //x 4	 
    50.515554,  //y

     1.150015,  //x
    60.596364,  //y

     0.1600231, //x 5	 
    69.6874691, //y

    00.2612261, //x
    78.717674    //y

   };   
   //storage per qword x1.x2x3x4, y1.y2y3y4, x5.x6x7x8,	y5.y6y7y8
   fwrite(&n_vert_a[0], sizeof(float), (5*4), dat);
   fprintf(inc,INC_TEXT, "N_VERT_A", mem_loc, mem_loc);
   mem_loc+= 5;
   int l_nvb = mem_loc;
   float n_vert_b[5][4]={

//  1  2 3 4
    30.2032201, //x 1	 
     0.010506,  //y
//  5  6 7 8
    31.193018, //x 
    12.131721,  //y

    28.172616,  //x 2	 
    26.30333901,  //y

    24.152413,  //x
    43.504761,  //y

    22.142222,  //x 3	 
    55.677055,  //y

    24.242827,  //x
    59.475244,  //y

    34.303834,  //x 4	 
    44.383933,  //y

    41.384643,  //x
    34.2728201, //y

    51.5057551, //x 5	 
    22.1215061, //y

    62.6070661, //x
    10.0101001   //y

   };   
   //storage per qword x1.x2x3x4, y1.y2y3y4, x5.x6x7x8,	y5.y6y7y8
   fwrite(&n_vert_b[0], sizeof(float), (5*4), dat);
   fprintf(inc,INC_TEXT, "N_VERT_B", mem_loc, mem_loc);
   
   mem_loc+= 5;
   int l_ucc = mem_loc;
   float u_curv_c[5][4]={

//  1  2 3 4
    33.4731471, //x 1	 
     2.0206061, //y
//  5  6 7 8
    34.4634431, //x 
     8.1012151, //y

    32.4030381, //x 2	 
    16.20192301,  //y

    28.36263401,//x
    22.26252901,//y

    23.301927,  //x 3	 
    29.35353901,//y

    16.241321,  //x
    39.434347,  //y

    10.170715,  //x 4	 
    47.525156,  //y

     5.130311,  //x
    55.5959621, //y

    00.1000101, //x 5	 
    64.6569701, //y

    02.1205151, //x
    74.7477791   //y

   };   
   //storage per qword x1.x2x3x4, y1.y2y3y4, x5.x6x7x8,	y5.y6y7y8
   fwrite(&u_curv_c[0], sizeof(float), (5*4), dat);
   fprintf(inc,INC_TEXT, "U_CURVE_C", mem_loc, mem_loc);
   
   mem_loc+= 5;
   int l_ucb = mem_loc;
   float u_curv_b[5][4]={

//  1  2 3 4
    58.4758471, //x 1	 
     0.020506,  //y
//  5  6 7 8
    56.455342, //x 
    10.101515,  //y

    49.394737,  //x 2	 
    21.20242301,  //y

    45.354333,  //x
    27.263029,  //y

    28.293626,  //x 3	 
    36.354039,  //y

    33.2330201, //x
    44.434847,  //y

    26.162414,  //x 4	 
    53.525756,  //y

    22.1221101, //x
    60.5963621, //y

    20.1021101, //x 5	 
    66.6569701, //y

    23.1121171, //x
    69.7479781   //y

   };   
   //storage per qword x1.x2x3x4, y1.y2y3y4, x5.x6x7x8,	y5.y6y7y8
   fwrite(&u_curv_b[0], sizeof(float), (5*4), dat);
   fprintf(inc,INC_TEXT, "U_CURVE_B", mem_loc, mem_loc);
   mem_loc+= 5;
     int l_uca = mem_loc;
   float u_curv_a[5][4]={

//  1  2 3 4
    88.8190711, //x 1	 
     0.020502,  //y
//  5  6 7 8
    89.688872, //x 
    15.141919,  //y

    85.738472,  //x 2	 
    15.14191901,  //y

    81.707868,  //x
    24.24282801,//y

    76.667364,  //x 3	 
    31.323635,  //y

    67.616356,  //x
    44.404847,  //y

    60.5355491, //x 4	 
    52.515856,  //y

    51.4447401, //x
    62.6166631, //y

    40.3533301, //x 5	 
    71.6675691, //y

    28.2620221, //x
    77.7079691   //y

   };   
   //storage per qword x1.x2x3x4, y1.y2y3y4, x5.x6x7x8,	y5.y6y7y8
   fwrite(&u_curv_a[0], sizeof(float), (5*4), dat);
   fprintf(inc,INC_TEXT, "U_CURVE_A", mem_loc, mem_loc);
   mem_loc+= 5;

   int l_eb  = mem_loc;
   float e_bottom[5][4]={

//  1  2 3 4
    64.6761681, //x 1	 
    42.374234,  //y
//  5  6 7 8
    59.655757, //x 
    40.333933,  //y

    52.524745,  //x 2	 
    39.34403401,  //y

    44.354031,  //x
    41.37454301,//y

    38.293624,  //x 3	 
    48.465153,  //y

    30.192613,  //x
    60.606769,  //y

    25.1325211, //x 4	 
    71.787378,  //y

    27.3435441, //x
    73.7971801, //y

    37.4842551, //x 5	 
    70.7567661, //y

    50.6259621, //x
    61.5753531   //y

   };   
   //storage per qword x1.x2x3x4, y1.y2y3y4, x5.x6x7x8,	y5.y6y7y8
   fwrite(&e_bottom[0], sizeof(float), (5*4), dat);
   fprintf(inc,INC_TEXT, "E_BOTTOM", mem_loc, mem_loc);
   mem_loc+= 5;

   int l_et  = mem_loc;
   float e_top[5][4]={

//  1  2 3 4
    74.7874801, //x 1	 
    27.2723251, //y
//  5  6 7 8
    76.8177831,//x 
    19.2216191, //y

    78.85798803,//x 2	 
    13.1912101 ,  //y

    79.9377891, //x
     9.0407014 ,//y

    76.7974741, //x 3	 
     6.0006011, //y

    71.6865541, //x
     6.0106031, //y

    64.5363501, //x 4	 
     9.091114,  //y

    61.4757431, //x
    15.1921251, //y

    54.4052371, //x 5	 
    25.3028341, //y

    51.3349411, //x
    31.3934381   //y

   };   
   //storage per qword x1.x2x3x4, y1.y2y3y4, x5.x6x7x8,	y5.y6y7y8
   fwrite(&e_top[0], sizeof(float), (5*4), dat);
   fprintf(inc,INC_TEXT, "E_TOP", mem_loc, mem_loc);
   mem_loc+= 5;


   int l_wva  = mem_loc;
   float w_vert_a[5][4]={

//  1  2 3 4
    30.2744321, //x 1	 
    03.0703081, //y
//  5  6 7 8
    44.3243311,//x 
    06.1312161, //y

    42.30402803,//x 2	 
    15.1919231 ,  //y

    38.2635241, //x
    23.2728304 ,//y

    33.2230201, //x 3	 
    32.3437381, //y

    28.1826161, //x
    41.4145451, //y

    24.1422121, //x 4	 
    48.495252,  //y

    20.1018081, //x
    46.4660601, //y

    16.0615031, //x 5	 
    64.6467711, //y

    13.0112011, //x
    73.7678811   //y

   };   
   //storage per qword x1.x2x3x4, y1.y2y3y4, x5.x6x7x8,	y5.y6y7y8
   fwrite(&w_vert_a[0], sizeof(float), (5*4), dat);
   fprintf(inc,INC_TEXT, "W_VERT_A", mem_loc, mem_loc);
   mem_loc+= 5;


   int l_wvb  = mem_loc;
   float w_vert_b[5][4]={

//  1  2 3 4
    50.4355431, //x 1	 
     0.0304081, //y
//  5  6 7 8
    53.4151391,//x 
    10.1216171, //y

    49.36463403,//x 2	 
    21.2327271 ,  //y

    42.2939261, //x
    34.3540424 ,//y

    35.2233181, //x 3	 
    46.4951571, //y

    29.1425121, //x
    57.6764721, //y

    23.1124231, //x 4	 
    68.787668,  //y

    28.2335311, //x
    70.6460501, //y

    39.3545461, //x 5	 
    55.5149401, //y

    50.5256651, //x
    44.3540211   //y

   };   
   //storage per qword x1.x2x3x4, y1.y2y3y4, x5.x6x7x8,	y5.y6y7y8
   fwrite(&w_vert_b[0], sizeof(float), (5*4), dat);
   fprintf(inc,INC_TEXT, "W_VERT_B", mem_loc, mem_loc);
   mem_loc+= 5;


   int l_wvc  = mem_loc;
   float w_vert_c[5][4]={

//  1  2 3 4
    55.4355431, //x 1	 
     0.0204051, //y
//  5  6 7 8
    55.4253411,//x 
     6.0810121, //y

    52.41503903,//x 2	 
    14.1517171 ,  //y

    48.3847351, //x
    21.2124254 ,//y

    45.3343311, //x 3	 
    28.2932331, //y

    41.2938271, //x
    36.3640401, //y

    36.2534231, //x 4	 
    44.444847,  //y

    33.2131181, //x
    52.5055561, //y

    30.1529131, //x 5	 
    59.6364671, //y

    27.1124111, //x
    71.7376781   //y

   };   
   //storage per qword x1.x2x3x4, y1.y2y3y4, x5.x6x7x8,	y5.y6y7y8
   fwrite(&w_vert_c[0], sizeof(float), (5*4), dat);
   fprintf(inc,INC_TEXT, "W_VERT_C", mem_loc, mem_loc);
   mem_loc+= 5;


   int l_wt  = mem_loc;
   float w_tail[5][4]={

//  1  2 3 4
    00.0400061, //x 1	 
    76.7670711, //y
//  5  6 7 8
    00.0800101,//x 
    66.6862651, //y

    04.13081503,//x 2	 
    60.6356601 ,  //y

    13.1815211, //x
    53.5651534 ,//y

    19.2523291, //x 3	 
    47.4942451, //y

    29.3635421, //x
    36.3830321, //y

    41.4846531, //x 4	 
    24.272022,  //y

    51.5856641, //x
    15.1811141, //y

    60.6865731, //x 5	 
    07.1104091, //y

    70.7774781, //x
    01.0800051   //y

   };   
   //storage per qword x1.x2x3x4, y1.y2y3y4, x5.x6x7x8,	y5.y6y7y8
   fwrite(&w_tail[0], sizeof(float), (5*4), dat);
   fprintf(inc,INC_TEXT, "W_TAIL", mem_loc, mem_loc);
   mem_loc+= 5;

   int l_hc  = mem_loc;
   letter h_cross_head = {
     -1.0,	0.0,	0,	8
   };
   float h_cross[4] ={
     39.454852,
     42.374237,

     55.596064,
     42.374236
   };
   fwrite(&h_cross_head, sizeof(float),4,  dat);
   fwrite(&h_cross[0], sizeof(float),4, dat);
   fprintf(inc,INC_TEXT,"H_CROSS", mem_loc, mem_loc);
   mem_loc+= 2;
   
   float rhombus_y = 1.0;
   float studios_y = 1.0f;
   letter letters[79] = {
     //R 5
     -66.0f,	0.0+rhombus_y, 	l_vsa, 0,//vert_stroke_a
     -66.0f,	0.0+rhombus_y,	l_vsb, 0,//vert_stroke_b
     -66.0f,	0.0+rhombus_y,	l_pla, 0,//p_loop_a
     -66.0f,	0.0+rhombus_y,	l_plb, 0, // p_loop_b
     -66.0f,    0.0+rhombus_y,   l_rd,  0,// r_dongle
     //H 4 (5)
     -58.0,	0.0+rhombus_y,	l_vsa, 0,//vert_stroke_a
     -58.0,	0.0+rhombus_y,	l_vsb, 0,//vert_stroke_b
     -58.0,	0.0+rhombus_y,	l_hc,  0,//cross
     -54.5,	0.0+rhombus_y,	l_vsa, 0,//vert_stroke_a
     -54.5,	0.0+rhombus_y,	l_vsb, 0,//vert_stroke_b
     //O 4
     -48.7,	0.0+rhombus_y,	l_ola, 0,
     -48.7,	0.0+rhombus_y,	l_olb, 0,
     -48.7,	0.0+rhombus_y,		l_olc, 0,
     -48.7,	0.0+rhombus_y,		l_old, 0,
     //M 5
     -41.0,	0.0+rhombus_y,		l_ml,	0,
     -41.0,	0.0+rhombus_y,		l_mva,	0,
     -41.0,	0.0+rhombus_y,		l_mvb,  0,
     -35.3,	0.0+rhombus_y,		l_mvc,	0,
     -35.3,	0.0+rhombus_y,		l_mvd,	0,
     //B 5
     -30.0,	0.0+rhombus_y,		l_vsa,	0,
     -30.0,	0.0+rhombus_y,		l_vsb,	0,
     -30.0,	0.0+rhombus_y,		l_bla,	0,
     -30.0,	0.0+rhombus_y,		l_blb,	0,
     -30.0,	0.0+rhombus_y,		l_blc,	0,

     //U 3
     -21.0,	0.0+rhombus_y,		l_uca,	0,
     -21.0,	0.0+rhombus_y,		l_ucb,	0,
     -21.0,	0.0+rhombus_y,		l_ucc,	0,
     //S 3
     -13.0,	0.0+rhombus_y,		l_sla,	0,
     -13.0,	0.0+rhombus_y,		l_slb,	0,
     -13.0,	0.0+rhombus_y,		l_slc,	0,

     //S 3
      06.0,	0.0+studios_y,   		l_sla,	0,
      06.0,	0.0+studios_y,		l_slb,	0,
      06.0,	0.0+studios_y,		l_slc,	0,

     //T 3
     15.0,	0.0 +studios_y,		l_tt,  0, 
     15.0,	0.0 +studios_y,		l_vsa, 0,//
     15.0,	0.0+studios_y,		l_vsb, 0,  
     
     //U 3
     23.0,	0.0+studios_y,		l_uca,	0,
     23.0,	0.0+studios_y,		l_ucb,	0,
     23.0,	0.0+studios_y,		l_ucc,	0,
     //D 4
     31.0,	0.0+studios_y,		l_vsa, 0,
     31.0,	0.0+studios_y,		l_vsb, 0,
     31.0,	0.0+studios_y,		l_dla, 0,
     31.0,	0.0+studios_y,		l_dlb, 0,
     //I 2
     40.0,	0.0+studios_y,		l_vsa, 0,
     40.0,	0.0+studios_y,		l_vsb, 0,

     //O 4
      46.0,	0.0+studios_y,		l_ola, 0,
      46.0,	0.0+studios_y,		l_olb, 0,
      46.0,	0.0+studios_y,		l_olc, 0,
      46.0,	0.0+studios_y,		l_old, 0,
     
     //S 3
     55.0,     0.0+studios_y,	    	 l_sla, 0,
     55.0,	0.0+studios_y,		l_slb,	0,
     55.0,	0.0+studios_y,		l_slc,	0,

     //PRESENTS
     //P 4
     -36.0,	15.0,		l_vsa, 0,
     -36.0,	15.0,		l_vsb, 0,
     -36.0,	15.0,		l_pla, 0,
     -36.0,	15.0,	       	l_plb, 0,

     //R  5
     -27.0f,	15.0f,	 	l_vsa, 0,//vert_stroke_a
     -27.0f,	15.0f,		l_vsb, 0,//vert_stroke_b
     -27.0f,	15.0f,		l_pla, 0,//p_loop_a
     -27.0f,	15.0f,		l_plb, 0, // p_loop_b
     -27.0f,    15.0f, 		l_rd,  0,//r_dongle

     //E 3
     -18.0f,	15.0f,		l_vsa, 0,
     -18.0f,	15.0f,		l_et,  0,
     -18.0f,	15.0f,		l_eb,  0,

     //S 3
     -09.0f,	15.0f,		l_sla, 0,
     -09.0f,	15.0f,		l_slb, 0,
     -09.0f,	15.0f,		l_slc, 0,
     //E 3
     0.0f,	15.0f,		l_vsa,	0,
     0.0f,	15.0f,		l_et,	0,
     0.0f,	15.0f,		l_eb,	0,

     //N 3
     09.0,	15.0f,		l_ml,	0,
     12.4,	15.0f,	 	 l_nva,	0,
     12.4,	15.0f,		l_nvb,  0,

     //T 3
     19.0,	15.0f,		l_tt,  0, 
     19.0,	15.0f,		l_vsa, 0,//
     19.0,	15.0f,		l_vsb, 0,  

     //S 3
     28.0,	15.0f,		l_sla,	0,
     28.0,	15.0f,		l_slb,	0,
     28.0,	15.0f,		l_slc,	1,




};


   fwrite(&letters, sizeof(float), 79 * 4, dat);
   fprintf(inc,INC_TEXT, "LETTERS_INTRO_B", mem_loc, mem_loc);
   mem_loc+= 79;
   float the_x =-10.0;
   float the_y =-0.0;
   float winds_x =-10.0;
   float winds_y= 0.0;
   
   letter letters_c[28] ={

     //T 3
     -40.2 +the_x,	00.0+the_y ,	l_tt,  0, 
     -40.2 +the_x,	00.0+the_y ,	l_vsa, 0,//
     -40.2 +the_x,	00.0+the_y ,	l_vsb, 0,  
     //H 4 (5)
     -33.0 +the_x,	0.0 +the_y,	l_vsa, 0,//vert_stroke_a
     -33.0 +the_x,	0.0+the_y ,	l_vsb, 0,//vert_stroke_b
     -33.0 +the_x,	0.0+the_y ,	l_hc,  0,//cross
     -29.5 +the_x,	0.0+the_y,	l_vsa, 0,//vert_stroke_a
     -29.5 +the_x,	0.0+the_y,	l_vsb, 0,//vert_stroke_b
     //E 3
     -23.7f +the_x,	 0.0+the_y ,	l_vsa, 0,
     -23.7f +the_x,	 0.0+the_y ,	l_et,  0,
     -23.7f +the_x,	 0.0+the_y ,	l_eb,  0,
     //w
     -1.0+winds_x,	  0.0+winds_y,	l_wva,	0,
     -1.0+winds_x,	  0.0+winds_y,	l_wvb,	0,
     2.5+winds_x,	  0.0+winds_y,	l_wva,	0,
     2.5+winds_x,	  0.0+winds_y,	l_wvc,	0,
     4.5+winds_x,	  0.0+winds_y,	l_wt,	0,	
     //I
      11.0+winds_x,	0.0+winds_y ,	l_vsa, 0,//vert_stroke_a
      11.0+winds_x,	0.0+winds_y ,	l_vsb, 0,//vert_stroke_b
     //N 3
     17.0+winds_x,	00.0+winds_y ,	l_ml,	0,
     20.4+winds_x,	00.0+winds_y ,  l_nva,	0,
     20.4+winds_x,	00.0+winds_y ,	l_nvb,  0,
     //D 4
     26.0+winds_x,	0.0+winds_y,	l_vsa, 0,
     26.0+winds_x,	0.0+winds_y,	l_vsb, 0,
     26.0+winds_x,	0.0+winds_y,	l_dla, 0,
     26.0+winds_x,	0.0+winds_y,	l_dlb, 0,
     //S 3
     35.0+winds_x,	00.0+winds_y ,	l_sla,	0,
     35.0+winds_x,	00.0+winds_y ,	l_slb,	0,
     35.0+winds_x,	00.0+winds_y ,	l_slc,	1,
   };


   fwrite(&letters_c, sizeof(float), 28 * 4, dat);
   fprintf(inc,INC_TEXT, "LETTERS_INTRO_C", mem_loc, mem_loc);
   mem_loc+= 28;


   float multiplier = 16.0f;
   float tmp_math[4] = {80.0f, 0.0f, 0.0f, 1.f};//110
   tmp_math[1] = 1.0/multiplier;
   tmp_math[2] = multiplier;
   fwrite(&tmp_math[0], sizeof(float), 4,	dat);
   fprintf(inc,INC_TEXT, "TEMP_MATH", mem_loc, mem_loc);
   (mem_loc++);
   int tmp_math2[4] = {100, 520, 0, 0 };
   fwrite(&tmp_math2[0],sizeof(int),	4,	dat);
   fprintf(inc,INC_TEXT, "TEMP_MATH2",mem_loc, mem_loc);
   mem_loc++;
   float rtv_math[4] = {0.5f,	-0.866025404f, 0.0f, 2.0f};
   fwrite(&rtv_math[0],sizeof(float),	4,	dat);
   fprintf(inc,INC_TEXT, "RTV_MATH",mem_loc, mem_loc);
   mem_loc++;
   float rt_off[4] = {-15.0f,-8.6f,0.0f,0.0f};
   fwrite(&rt_off[0],sizeof(float),	4,	dat);
   fprintf(inc,INC_TEXT, "RT_OFF",mem_loc, mem_loc);
   mem_loc++;


   fprintf(inc,INC_TEXT, "INTRO_UNPACK_BUFFER",  mem_loc, mem_loc);
   mem_loc+= 44;    
   fprintf(inc,INC_TEXT, "INTRO_O_BUFFER", mem_loc, mem_loc);
   
   fprintf(inc,INC_TEXT, "D_BUFFER", (mem_loc+200), (mem_loc+200));


   fprintf(inc,";;;\n;;; Start control constants\n;;;\n" );
   fprintf(inc,INC_TEXT, "MODE_INTRO_A", 0);
   fprintf(inc,INC_TEXT, "MODE_INTRO_B", 1);
   fprintf(inc,INC_TEXT, "MODE_INTRO_C", 2);
   fprintf(inc,INC_TEXT, "MODE_SETUP_MAIN", 3);
   fprintf(inc,INC_TEXT, "MODE_MAIN",	4);

   fprintf(inc,INC_TEXT, "MAX_FRAMES_B",  750);
   fprintf(inc,INC_TEXT, "MAX_FRAMES_C",  750);
   printf("memory usage..... intro: %-3d, main: %-3d\n", mem_loc, main_mem_loc);
   printf("free memory...... intro: %-3d, main: %-3d\n", 1023-mem_loc, 1023-main_mem_loc);

   fclose(inc);
   fclose(dat);
   
   //
}
/*

    iaddiu	dvc_vt_i,	vi00,	0
    iaddiu	cv_vt2_i,	vi00,	5
    bal		ret_constrain_rope_vts,	constrain_rope_vts

    iaddiu	dvc_ci,	vi00,	vi00

do_vert_constraints_loop:
    ilw.x	dvc_vt_i,	FLAG_STICKS(dvc_ci)
    ilw.y	cv_vt2_i,	FLAG_STICKS(dvc_ci)

    bal		ret_constrain_vts,	constrain_vts
    
    ilw.z	cv_vt2_i,	FLAG_STICKS(dvc_ci)

    ibeq	cv_vt2_i,	vi00,	next_i
    bal		ret_constrain_vts,	constrain_vts

    ilw.w	cv_vt2_i,	FLAG_STICKS(dvc_ci)

    ibeq	cv_vt2_i,	vi00,	next_i
    bal		ret_constrain_vts,	constrain_vts
next_i:
    iaddiu	dvc_ci,	dvc_ci,	1
    iaddiu	max_c,	vi00,	102
    ibne	max_c, dvc_ci,	do_vert_constraints_loop

    iaddiu	dvc_vt_i,	vi00,	9
    iaddiu	cv_vt2_i,	vi00,	104

    bal		ret_constrain_rope_vts,	constrain_rope_vts2
    
    iaddiu	dvc_vt_i,	vi00,   112
    iaddiu	dvc_vt_i,	vi00,	111
   
    bal		ret_constrain_rope_vts,	constrain_rope_vts

    iaddiu	rpct,		vi00,	7
    iaddiu	dvc_vt_i,	vi00,	104
const_rope_loop:
    iaddiu	cv_vt2_i,	dvc_vt_i,	1
    bal		ret_constrain_vts, constrain_vts
    iaddiu	dvc_vt_i,	dvc_vt_i,	1
    isubiu	rpct,		rpct,		1
    ibne	rpct,		vi00,		const_rope_loop	




    b ret_do_vert_constraints

constrain_vts:
     lq		dvc_vt1,		FLAG_VERTS(dvc_vt_i)
     lq		dvc_vt2,		FLAG_VERTS(cv_vt2_i)
    sub.xyz	dif,	dvc_vt1,	dvc_vt2
     loi	1
     mul.xyz	dot,	dif,		dif
     addy.x	dot,	dot,		dot
     addz.x	dot,	dot,		dot
     sqrt	q,	dot[x]
     addq.x	q_result,	vf00,	q
     subi.x	tmp,		q_result,	I
     div	q,		tmp[x],		q_result[x]
     mulq.xyz	delta,		dif,		q	
     add.xyz	dvc_vt2,	dvc_vt2,	delta
     sq.xyz	dvc_vt2,	FLAG_VERTS(cv_vt2_i)

     jr		ret_constrain_vts


constrain_rope_vts:
     lq		dvc_vt1,	FLAG_VERTS(dvc_vt_i)
     lq		dvc_vt2,	FLAG_VERTS(cv_vt2_i)
     sub.xyz	dif,	dvc_vt1,	dvc_vt2
     loi	1.0
     mul.xyz	dot,	dif,		dif
     addy.x	dot,	dot,		dot
     addz.x	dot,	dot,		dot
     sqrt	q,	dot[x]
     addq.x	q_result,	vf00,	q
     subi.x	tmp,		q_result,	I
     div	q,		tmp[x],		q_result[x]
     mulq.xyz	delta,		dif,		q	
     add.xyz	dvc_vt2,	dvc_vt2,	delta
     sq.xyz	dvc_vt2,	FLAG_VERTS(cv_vt2_i)

     jr		ret_constrain_rope_vts
 
*/
