//-----------------------------------------------------------
// Copyright (C)2002 Sony Computer Entertainment America Inc.
//
// This file is subject to the terms and conditions of the GNU
// General Public Licence Version 2.  See the file "COPYING"
// in the main directory of this archive for more details.
//-----------------------------------------------------------

/*
08/28/99
Description:
rfont.c for use with ee
This font system uses no vram.  
The font is stored in main ram.
The font is in the ramfont.s assembler file. 
When assembled the font data is less than 2kb (58*5*70)
The assembler source file is kind of readable/editable.
No I didn't make it by hand!
This system uses the point primitive and renders each character in 
the string in turn using the scratch pad as a buffer area.

17/8/2000 found bug in dma tag of add_rfont and fixed

*/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>
#include <malloc.h>
#include <assert.h>
#include <ps2gs.h>
#include <ps2vpu.h>
#include <ps2vpufile.h>

#include "os.h"
#include "ps2_dma.h"
#include "ps2_vif.h"

#include "dlib_rfont.h"

//-----------------------------------------------------------------------------------------
#define  SCREEN_WIDTH          640
#define  SCREEN_HEIGHT         224

#define  OFFX                  (((4096 - SCREEN_WIDTH) / 2) << 4)
#define  OFFY                  (((4096 - SCREEN_HEIGHT) / 2) << 4)

// take a coord in the valid range (0..screen width or height and convert to device coord.
#define  SCREEN_TO_DEVICEX(p)  (((p) << 4) + OFFX)
#define  SCREEN_TO_DEVICEY(p)  (((p) << 4) + OFFY)

//this is the font data compiled in as an object module
extern  u_int ramfont[]       __attribute__((aligned(16)));
extern  u_int ramfonts[]      __attribute__((aligned(16)));
extern  u_int ramfontb[]      __attribute__((aligned(16)));


//=====================================================================================
//=====================================================================================
//=====================================================================================
void add_rfont2 (unsigned short xpos,  // x pos in screen coordinates (0..SCREEN_WIDTH)
                 unsigned short ypos,  // y pos in screen coordinates (0..SCREEN_HEIGHT)
                 unsigned int   zpos,  // z depth
                 unsigned char  r,     // font color
                 unsigned char  g,     // font color
                 unsigned char  b,     // font color
                 unsigned char  s,     // size ( 0 for small font 1 for big font)
                 char           *cp)   // pointer to string to display
{
   unsigned short  x, y, xStart;
   char            *ram_font_pointer;
   unsigned short  font_height;
   unsigned short  font_width;
   OS_Qword        *buffer;         // ptr to memory where dma packet for gs will be created
   OS_Qword        *startbuffer;
   ps2_giftag      *giftag;         // ptr to current giftag in dma packet
   OS_Qword        *directviftag;   // ptr to DIRECT VIFTag in packet

   xStart = xpos;

   // set the font size
   if (s == 0) 
   {
      font_height = RFONTS_H;
      font_width  = RFONTS_W;
   }
   else
   {
      font_height = RFONTB_H;
      font_width  = RFONTB_W;
   }

   //----- allocate some memory for the dma packet (on spr)
   startbuffer = (OS_Qword *)memalign (128, 16384);
   buffer = startbuffer;

   if (buffer == NULL)
   {
      printf ("Couldn't allocate a buffer\n");
      return;
   }

   // Set up the DIRECT VIFTag
   buffer->ui32[0] = VIF_CMD_NOP << 24;
   buffer->ui32[1] = VIF_CMD_NOP << 24;
   buffer->ui32[2] = VIF_CMD_NOP << 24;
   buffer->ui32[3] = VIF_CMD_DIRECT << 24;

   directviftag = buffer;
   buffer++;

   // set up the giftag to send the color
   buffer->ui64[0] = PS2_GIFTAG_SET_TOPHALF (1,                     // nloop
                                             0,                     // eop
                                             0,                     // pre
                                             0,                     // prim
                                             PS2_GIFTAG_FLG_PACKED, // flag
                                             1);                    // number of registers
   buffer->ui64[1] = 0x1L;          // single rgba
   giftag = (ps2_giftag *)buffer;   // address of giftag
   buffer++;

   buffer->ui32[0] = r;
   buffer->ui32[1] = g;
   buffer->ui32[2] = b;
   buffer->ui32[3] = 128;
   buffer++;

   directviftag->ui32[3] += 2; //giftag and single color register setting

   // set up the giftag to send the point
   buffer->ui64[0] = PS2_GIFTAG_SET_TOPHALF (0,                     // nloop
                                             1,                     // eop
                                             1,                     // pre
                                             PS2_GS_SETREG_PRIM (PS2_GS_PRIM_PRIM_POINT, 0, 0, 0, 0, 0, 0, 0, 0),  //prim
                                             PS2_GIFTAG_FLG_PACKED, // flag
                                             1);                    // number of registers
   buffer->ui64[1] = 0x5L;          // single xyzf2
   giftag = (ps2_giftag *)buffer;   // address of giftag
   buffer++;
   directviftag->ui32[3]++;         // update DIRECT VIFTag because added a giftag

   while (*cp != 0)
   {
      if (*cp == '\n')
      {
         ypos += font_height;
         xpos = xStart;
         cp++;
         continue;
      }

      if (*cp != ' ')
      {
         //set the rfp to point to the start of the data for the character
         //ram_font_pointer=((char*)ramfont)+(((*cp)-'!')*5*7);
         if (s == 0) 
            ram_font_pointer = ((char *)ramfonts + ((*cp) * font_height * font_width));
         else
            ram_font_pointer = ((char *)ramfontb + ((*cp) * font_height * font_width));

         // actually render the character, a pixel at a time....
         for (y = 0; y < font_height; y++)
         {
            for (x = 0; x < font_width; x++)
            {
               if (*ram_font_pointer)
               {
                  buffer->ui32[0] = SCREEN_TO_DEVICEX ((xpos + x));
                  buffer->ui32[1] = SCREEN_TO_DEVICEY (ypos + y);
                  buffer->ui32[2] = zpos;    // set the z
                  buffer->ui32[3] = 0;
                  buffer++;
                  giftag->NLOOP++;           // update giftag
                  directviftag->ui32[3]++;   // update DIRECT VIFTag

                  if (((int)buffer - (int)startbuffer) > 16380)  // if the scratch pad gets full kick it out and start filling again
                  {
                     VIF_AddData (startbuffer, buffer - startbuffer, VIF_ATTACHMENT_INLINE, VIF_DEST_VIF1);
                     buffer = startbuffer + 4;
                     directviftag->ui32[3] = (VIF_CMD_DIRECT << 24) + 3;  // reset DIRECT VIFTag
                     giftag->NLOOP = 0;
                  }
               }

               ram_font_pointer++;     // increment the position in the font data
            } // end for x
         } // end for y
      } // end printable character
 
      xpos += font_width;              // update the cursor position
      cp++;
   }

   VIF_AddData (startbuffer, buffer - startbuffer, VIF_ATTACHMENT_INLINE, VIF_DEST_VIF1);

   // We don't need the buffer anymore...
   free (startbuffer);
}
//=====================================================================================

