//-------------------------------------------------------------------------------
// Copyright (C)2003 Sony Computer Entertainment America Inc.
//
// This file is subject to the terms and conditions of the GNU
// General Public Licence Version 2.  See the file "COPYING"
// in the main directory of this archive for more details.
//-------------------------------------------------------------------------------
//
// VU1 Demo Coding Competition harness SCEA 2003
//                                
 
// Author: David Coombes, SCEA

// Big thanks to StewartS for his help with the invite demo 
// and also for saving me from death by dma.
//
// Note: 
// This is a work in progress... going to add a menu for manually selecting demos
// as well as a random play function.
// Going to use the merge circuit to add the demo title so that demo programs
// can do anything to the frame buffer and the demo title will still display
// correctly.
// I looked at this to get things at least half way decent on xga
//-------------------------------------------------------------------------------



/*
Changes
==================
  4/24/2003 KAB  SCEA , changed text for SCEA 2003, added captions on/off option  
*/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>
#include <malloc.h>
#include <assert.h>
#include <ps2gs.h>
#include <ps2vpu.h>
#include <ps2vpufile.h>
	   
#include "os.h"
#include "ps2_dma.h"
#include "ps2_vif.h"
#include "sjoy.h"

#include "main.h"
#include "dlib_rfont.h"
#include "dlib_file.h"

#define  DATA_PATH   "./demodata/"

#define  NEAR_Z     ((1 << 24) - 1)

#define CONTROL_MODE_RANDOM		1
#define CONTROL_MODE_USER		0
#define BUTTON_HELP_USER "L1:next, L2:prev, L1+L2:menu, select:help "	  // button help user interactive mode
#define BUTTON_HELP_RAND "random demo mode  L1+L2:menu"		  // button help random demo mode

#define MENULMARGIN 80		 // left margin for menu text
//---------------------------------------------------------------------------
// Globals
static int  gHelpOn = 0;

//---------------------------------------------------------------------------
//
//---------------------------------------------------------------------------
void error_exit (char *c)
{
   printf ("********************************************************\n");
   printf ("********************************************************\n");
   printf ("Fatal Error\n");
   printf ("%s\n",c);
   printf ("********************************************************\n");
   printf ("********************************************************\n");
}

//---------------------------------------------------------------------------
// Returns ptr to end of data record recovered
//---------------------------------------------------------------------------
char *get_ini_record (char *ini_data,  // ini data file (pointer to start of record to recover)
                      char *path,      // path where demos are stored
                      char *name,      // name of demo
                      char *code,      // name of code file 
                      char *data,      // name of data file
                      char *help)      // name of help file
{
   char  *c;
   char  temp_s[MAX_STRING];
   char  *s;

   c = ini_data;

   // A record starts with an open brace;
   while (*c != '(')
      c++;
   c++; //skip over the open brace

   // Read the demo name and convert to a string
   s = temp_s;
   while (*c != ',')
      *s++ = *c++;
   *s = 0;  // null terminate the string
   c++;     // skip over the comma
   strcpy (name, temp_s);

   // Read the code name and convert to a string
   s = temp_s;
   while (*c != ',')
      *s++ = *c++;
   *s = 0;  // null terminate the string
   c++;     // skip over the comma
   strcpy (code, path);
   strcat (code, temp_s);

   // Read the data name and convert to a string
   s = temp_s;
   while (*c != ',')
      *s++ = *c++;
   *s = 0;  // null terminate the string
   c++;     // skip over the comma
   printf (">>%s<< len %d\n", temp_s, (s + 1) - temp_s);
   strcpy (data, path);
   strcat (data, temp_s);

   // Read the help file name and convert to a string
   s = temp_s;
   while (*c != ')')
      *s++ = *c++;
   *s = 0;  // null terminate the string
   c++;     // skip over the close brace
   strcpy (help, path);
   strcat (help, temp_s);

   return (c);
}

//---------------------------------------------------------------------------
// Returns the created node
//---------------------------------------------------------------------------
DEMO_CODE_NODE *insert_demo_node (DEMO_CODE_LIST *dcl,
                                  char           *name)  // Name of demo
{
   DEMO_CODE_NODE  *node;

   // Allocate memory for this node
   node = malloc (sizeof (DEMO_CODE_NODE));
   if (node == NULL)
   {
      // Failed to allocate memory for node
      return (NULL);
   }

   // Give it a name
   strcpy (node->name, name);

   // Insert into node list
   node->next = dcl->head.next;
   node->prev = &dcl->head;

   dcl->head.next->prev = node;
   dcl->head.next = node;

   dcl->current = node;
   return (node);
}

//---------------------------------------------------------------------------
// Loads a vu1 program from disk and stores in the node.
// Returns NULL on failure or address of node on successs
//---------------------------------------------------------------------------
DEMO_CODE_NODE *load_vu1_code_from_disk (DEMO_CODE_NODE *node,          // Node to attach code to
                                         char           *code_filename) // File where code is stored
{
   __u64  *code_buffer;
   int    code_bytes_loaded;

   if (node == NULL)
      return (NULL);

   // Load the files
   code_buffer = (__u64 *)dc_read_file (code_filename, &code_bytes_loaded);
   if (code_buffer == NULL)
   {
      // Failed to load file
      return (NULL);
   }

   // Check its a multiple number of __u64 in length
   if (code_bytes_loaded % (sizeof (__u64)) != 0)
   {
       printf ("Error: code size %d\n", code_bytes_loaded);
       return (NULL);
   }

   node->code_size = code_bytes_loaded / (sizeof (__u64));
   node->code = code_buffer;

   return (node);
}

//---------------------------------------------------------------------------
// loads vu1 data from disk and stores in the node
// returns null on failure or address of node on successs
//---------------------------------------------------------------------------
DEMO_CODE_NODE *load_vu1_data_from_disk (DEMO_CODE_NODE *node,          // Node to attach data to
                                         char*          data_filename)  // File where data is stored
{
   OS_Qword  *data_buffer;
   int       data_bytes_loaded;

   if (node == NULL)
      return (NULL);

   // Load the files
   data_buffer = (OS_Qword *)dc_read_file (data_filename, &data_bytes_loaded);
   if (data_buffer == NULL)
   {
      // Failed to load file
      return (NULL);
   }

   // Check its a multiple number of __u64 in length
   if (data_bytes_loaded % (sizeof (OS_Qword)) != 0)
   {
      printf ("Error: data size %d\n", data_bytes_loaded);
      return (NULL);
   }

   node->data_size = data_bytes_loaded / (sizeof (OS_Qword));
   node->data = data_buffer;

   return (node);
}

//---------------------------------------------------------------------------
//
//---------------------------------------------------------------------------
void load_demo_list (char           *filename,  // name of inifile
                     char           *path,      // path to inifile and all data files
                     DEMO_CODE_LIST *dcl)       // demo code structure
{
   char            *ini_data;
   char            *c;
   char            name[MAX_STRING];
   char            code[2 * MAX_STRING];     // filename can be maxstring + path which can also be maxstring
   char            data[2 * MAX_STRING];
   char            hlp[2 * MAX_STRING];
   char            help[2 * MAX_STRING];
   FILE            *helpFile;
   int             len;
   char            ini_filename[2 * MAX_STRING];
   char            mess[MAX_STRING];
   u_int           bytes_loaded;
   DEMO_CODE_NODE  *new_node;

   // make the full path name to the inifile
   strcpy (ini_filename, path);
   strcat (ini_filename, filename);

   // load the ini file from the disk
   ini_data = (char *)dc_read_file (ini_filename, &bytes_loaded);

   if (!ini_data)
   {
      sprintf (mess, "Could not load %s", ini_filename);
      error_exit (mess);
   }

   c = ini_data;

   while (*c != '#')
   {
      c = get_ini_record (c, path, name, code, data, hlp);

      new_node = insert_demo_node (dcl, name);
      if (!new_node)
         error_exit ("node creation failed\n");

      if (!load_vu1_code_from_disk (new_node, code))
      {
         sprintf (mess, "Could not load %s (code)", code);
         error_exit (mess);
      }

      if (!load_vu1_data_from_disk (new_node, data))
      {
         sprintf (mess, "Could not load %s (data)", data);
         error_exit (mess);
      }

      // add help text
      strcpy (help, hlp);

      helpFile = fopen (help, "rb");

      if (helpFile == NULL)
      {
         // no help file found
         printf ("No help file found for %s, skipping\n", help);
         continue;
      }

      if (fseek (helpFile, 0, SEEK_END) != 0)
      {
         printf ("Couldn't get help file's size '%s', skipping\n", help);
         fclose (helpFile);
         continue;
      }

      len = ftell (helpFile);

      if (fseek (helpFile, 0, SEEK_SET) != 0)
      {
         printf ("Couldn't get help file's size '%s', skipping\n", help);
         fclose (helpFile);
         continue;
      }

      if (fread (dcl->current->helpText, 1, len, helpFile) != len )
         printf ("Couldn't read the full help file '%s', skipping\n", help);

      fclose (helpFile);
   }

   //don't need this anymore
   free (ini_data);
}

//---------------------------------------------------------------------------
//
//---------------------------------------------------------------------------
void init_demo_list (DEMO_CODE_LIST *dcl)
{
   // Just set the dummy nodes head and tail to link together
   dcl->head.next = &dcl->tail;
   dcl->head.prev = &dcl->head;

   // There is no current node as the list is empty
   dcl->current = NULL;
}

//---------------------------------------------------------------------------
//
//---------------------------------------------------------------------------
void next_demo_node (DEMO_CODE_LIST *dcl,
                     int            dir)     // Greater than 0 next else prev
{
   // If the list isn't empty
   if (&dcl->head != &dcl->tail)
   {
      if (dir > 0)
      {
         if (dcl->current->next != &dcl->tail) 
            dcl->current = dcl->current->next;
         else
            dcl->current = dcl->head.next;    
      }
      else if (dcl->current->prev != &dcl->head) 
         dcl->current = dcl->current->prev;
      else
         dcl->current = dcl->tail.prev;    
   }
}

//---------------------------------------------------------------------------
//
//---------------------------------------------------------------------------
unsigned int UploadVUCodeAndData (DEMO_CODE_LIST *dcl)
{
   OS_Qword      *codebuffer = (OS_Qword *)dcl->current->code;
   unsigned int  codesize = dcl->current->code_size;
   unsigned int  codedest = 0;
   OS_Qword      *emptydata;

   // First clear VU1 data memory
   if ((emptydata = (OS_Qword *)memalign (128, 16384)) != NULL)
   {
      unsigned int  i;

      for (i = 0; i < 1024; i++)
      {
         emptydata[i].ui32[0] = 0;
         emptydata[i].ui32[1] = 0;
         emptydata[i].ui32[2] = 0;
         emptydata[i].ui32[3] = 0;
      }

      if (VIF_AddVUData (emptydata, 1024, 0, VIF_ATTACHMENT_INLINE, VIF_DEST_VIF1) != VIF_RET_OK)
         return (1);

      free (emptydata);
   }

   // Wait for completion of current VU1 code...
   if (VIF_AddTag (VIF_CMD_FLUSHA, 0, 0, VIF_DEST_VIF1) != VIF_RET_OK)
      return (1);

   while (codesize)
   {
      if (codesize > 254)
      {
         // We're trying to send too much code in one go...  Cut it in smaller chunks
         if (VIF_AddTag (VIF_CMD_MPG, 254, codedest, VIF_DEST_VIF1) != VIF_RET_OK)
            return (1);
         if (VIF_AddData (codebuffer, 254 / 2, VIF_ATTACHMENT_INLINE, VIF_DEST_VIF1) != VIF_RET_OK)
            return (1);

         codebuffer += 254 / 2;
         codesize -= 254;
         codedest += 254;
      }
      else
      {
         if (VIF_AddTag (VIF_CMD_MPG, codesize, codedest, VIF_DEST_VIF1) != VIF_RET_OK)
            return (1);
         if (VIF_AddData (codebuffer, codesize / 2, VIF_ATTACHMENT_INLINE, VIF_DEST_VIF1) != VIF_RET_OK)
            return (1);

         codesize = 0;
      }
   }

   if (VIF_AddVUData ((OS_Qword *)dcl->current->data, dcl->current->data_size, 0, VIF_ATTACHMENT_INLINE, VIF_DEST_VIF1) != VIF_RET_OK)
      return (1);

   // Everything was fine
   return (0);
}

//---------------------------------------------------------------------------
// VU coding entries expect the joypad info to be in a different format from
// what we get it in...  Convert that
//---------------------------------------------------------------------------
unsigned int ConvertPadInfo (unsigned int paddata)
{
   unsigned int  newpaddata = 0;

   if (paddata & SJOY_PS2_R_LEFT)
      newpaddata |= 0x8000;
   if (paddata & SJOY_PS2_R_DOWN)
      newpaddata |= 0x4000;
   if (paddata & SJOY_PS2_R_UP)
      newpaddata |= 0x1000;
   if (paddata & SJOY_PS2_R_RIGHT)
      newpaddata |= 0x2000;

   if (paddata & SJOY_PS2_L1)
      newpaddata |= 0x0400;
   if (paddata & SJOY_PS2_R1)
      newpaddata |= 0x0800;
   if (paddata & SJOY_PS2_L2)
      newpaddata |= 0x0100;
   if (paddata & SJOY_PS2_R2)
      newpaddata |= 0x0200;

   if (paddata & SJOY_PS2_SELECT)
      newpaddata |= 0x0001;
   if (paddata & SJOY_PS2_START)
      newpaddata |= 0x0008;

   if (paddata & SJOY_PS2_L_LEFT)
      newpaddata |= 0x0080;
   if (paddata & SJOY_PS2_L_DOWN)
      newpaddata |= 0x0040;
   if (paddata & SJOY_PS2_L_UP)
      newpaddata |= 0x0010;
   if (paddata & SJOY_PS2_L_RIGHT)
      newpaddata |= 0x0020;

   newpaddata = ~newpaddata;

   return (newpaddata);
}

//---------------------------------------------------------------------------
// Set the GS registers to values we know to be valid and working...
//---------------------------------------------------------------------------
unsigned int ResetGSRegisters (void)
{
   struct
   {
      ps2_giftag            giftag;

      ps2_gsreg_alpha       alpha1;
      ps2_gsreg_addr        alpha1addr;

      ps2_gsreg_clamp       clamp1;
      ps2_gsreg_addr        clamp1addr;

      ps2_gsreg_colclamp    colclamp;
      ps2_gsreg_addr        colclampaddr;

      ps2_gsreg_dimx        dimx;
      ps2_gsreg_addr        dimxaddr;

      ps2_gsreg_dthe        dthe;
      ps2_gsreg_addr        dtheaddr;

      ps2_gsreg_fba         fba1;
      ps2_gsreg_addr        fba1addr;

      ps2_gsreg_fogcol      fogcol;
      ps2_gsreg_addr        fogcoladdr;

      ps2_gsreg_pabe        pabe;
      ps2_gsreg_addr        pabeaddr;

      ps2_gsreg_prmodecont  prmodecont;
      ps2_gsreg_addr        prmodecontaddr;

      ps2_gsreg_test        test1;
      ps2_gsreg_addr        test1addr;

      ps2_gsreg_texa        texa;
      ps2_gsreg_addr        texaaddr;

      ps2_gsreg_zbuf        zbuf1;
      ps2_gsreg_addr        zbuf1addr;
   } DefaultSettings;

   PS2_GIFTAG_CLEAR_TAG (&DefaultSettings.giftag);
   DefaultSettings.giftag.NLOOP = sizeof (DefaultSettings) / 16 - 1;
   DefaultSettings.giftag.EOP = 1;
   DefaultSettings.giftag.PRE = 0;
   DefaultSettings.giftag.NREG = 1;
   DefaultSettings.giftag.REGS0 = PS2_GIFTAG_REGS_AD;

   DefaultSettings.alpha1addr = PS2_GS_ALPHA_1;
   DefaultSettings.clamp1addr = PS2_GS_CLAMP_1;
   DefaultSettings.colclampaddr = PS2_GS_COLCLAMP;
   DefaultSettings.dimxaddr = PS2_GS_DIMX;
   DefaultSettings.dtheaddr = PS2_GS_DTHE;
   DefaultSettings.fba1addr = PS2_GS_FBA_1;
   DefaultSettings.fogcoladdr = PS2_GS_FOGCOL;
   DefaultSettings.pabeaddr = PS2_GS_PABE;
   DefaultSettings.prmodecontaddr = PS2_GS_PRMODECONT;
   DefaultSettings.test1addr = PS2_GS_TEST_1;
   DefaultSettings.texaaddr = PS2_GS_TEXA;
   DefaultSettings.zbuf1addr = PS2_GS_ZBUF_1;

   *(__u64 *)&DefaultSettings.alpha1 = PS2_GS_SETREG_ALPHA (0, 1, 0, 1, 0);
   *(__u64 *)&DefaultSettings.clamp1 = PS2_GS_SETREG_CLAMP (0, 0, 0, 256, 0, 256);
   *(__u64 *)&DefaultSettings.colclamp = PS2_GS_SETREG_COLCLAMP (1);
   *(__u64 *)&DefaultSettings.dimx = PS2_GS_SETREG_DIMX (-4, 2, -3, 3, 0, -2, 1, -1, -3, 3, -4, 2, 1, -1, 0, -2);
   *(__u64 *)&DefaultSettings.dthe = PS2_GS_SETREG_DTHE (0);
   *(__u64 *)&DefaultSettings.fba1 = PS2_GS_SETREG_FBA (0);
   *(__u64 *)&DefaultSettings.fogcol = PS2_GS_SETREG_FOGCOL (0x0, 0x0, 0x0);
   *(__u64 *)&DefaultSettings.pabe = PS2_GS_SETREG_PABE (0);
   *(__u64 *)&DefaultSettings.prmodecont = PS2_GS_SETREG_PRMODECONT (1);
   *(__u64 *)&DefaultSettings.test1 = PS2_GS_SETREG_TEST (0, 0, 0, 0, 0, 0, 1, PS2_GS_ZALWAYS);
   *(__u64 *)&DefaultSettings.texa = PS2_GS_SETREG_TEXA (0x7F, 1, 0x81);
   *(__u64 *)&DefaultSettings.zbuf1 = PS2_GS_SETREG_ZBUF (0, 0, 0);

   // Add the registers setting to the VIF chain...
   if (VIF_AddTag (VIF_CMD_DIRECT, 0, sizeof (DefaultSettings) / sizeof (OS_Qword), VIF_DEST_VIF1) != VIF_RET_OK)
      return (1);
   if (VIF_AddData ((OS_Qword *)&DefaultSettings, sizeof (DefaultSettings) / sizeof (OS_Qword), VIF_ATTACHMENT_INLINE, VIF_DEST_VIF1) != VIF_RET_OK)
      return (1);

   // Everything was fine
   return (0);
}

//---------------------------------------------------------------------------
//
//---------------------------------------------------------------------------
int main (int argc, char *argv[])
{
   int             isok = -15;
   int             odev, frame;
   char            test_string[MAX_STRING];
   OS_Qword        gameinfo;
   DEMO_CODE_LIST  dcl;
   int             mode = CONTROL_MODE_USER;   // demo mode
   int				captions=1;         // captions: title and help text on/off
   int             random_counter = 15 * 60;
   ps2_gs_dbuff    db;
   ps2_gs_finish   finish;
   unsigned int    paddata = 0, prevpaddata = 0;

   // Initialization
   
      struct OS_Data  *osdata;
      ps2_gs_gparam   *gparam;



      // OS_Init will config the GS for the mode we're interested in
      // all I'm going to do is then use that mode, but letterbox
      // whatever it is ..

      if (OS_Init (argc, argv) != OS_RET_OK)
      {
         printf ("MAIN ERROR: Couldn't initialize the low-level systems\n");
         exit (1);
      }
      else
         printf ("MAIN: Initialization of low-level systems complete\n");

      OS_GetOSData (&osdata);
      gparam = ps2_gs_get_gparam ();  	// get parameter from teh gs

    //  ps2_gs_set_dbuff (&db, osdata->FrameDepth, gparam->width, gparam->height, (osdata->ZBit == 0) ? 0 : PS2_GS_ZGEQUAL, osdata->ZDepth, 1);
      ps2_gs_set_dbuff (&db, osdata->FrameDepth,640, 224,PS2_GS_ZGEQUAL, osdata->ZDepth, 1);

      *(__u64 *)&db.clear0.rgbaq = PS2_GS_SETREG_RGBAQ (0x0, 0x0, 0x0, 0x80, 0x3f800000);
      *(__u64 *)&db.clear1.rgbaq = PS2_GS_SETREG_RGBAQ (0x0, 0x0, 0x0, 0x80, 0x3f800000);

      ps2_gs_put_drawenv (&db.giftag1);

      ps2_gs_set_finish (&finish);
      ps2_gs_wait_finish (&finish);
   

   	// so - now let's get the old display register

	



   init_demo_list (&dcl);
   load_demo_list ("demotest.ini", DATA_PATH, &dcl);

   // Reinitialize GS registers
   if (ResetGSRegisters ())
   {
      printf ("MAIN ERROR: Couldn't reinitialize GS registers\n");
      exit (1);
   }

   // Load the initial demo code and data into VU1
   if (UploadVUCodeAndData (&dcl))
   {
      printf ("Couldn't add VU Data\n");
      exit (1);
   }

   // The only way to get out of this loop is to press the SELECT button
   while (1)
   {
      // Wait for VSync
      odev = !ps2_gs_sync_v (0);
      frame = 0;

      // Mode select menu
      while (1)
      {
         OS_Update ();

         ps2_gs_swap_dbuff (&db, frame);

         prevpaddata = paddata;
         paddata = sjoy_get_ps2_button (0);
/*
         add_rfont2 (10,  80, NEAR_Z, 150, 150, 100, 1, "*****************");
         add_rfont2 (10,  90, NEAR_Z, 150, 150, 100, 1, "*DEVSTATION 2003*");
         add_rfont2 (10, 100, NEAR_Z, 150, 150, 100, 1, "*****************");
*/
		add_rfont2(MENULMARGIN,50,NEAR_Z,150,150,100,2, "*****************************");
		add_rfont2(MENULMARGIN,60,NEAR_Z,150,150,100,2, "   Linux for PlayStation\x01""2     ");
		add_rfont2(MENULMARGIN,70,NEAR_Z,150,150,100,2, "   VU Coding Contest 2003         ");
		add_rfont2(MENULMARGIN,80,NEAR_Z,150,150,100,2, "*****************************");
/* 
		 if (mode == 0)
            sprintf (test_string, "Control Mode: Manual (L1 l2)");
         else
            sprintf (test_string, "Control Mode: Random Play");
*/
		if (mode== CONTROL_MODE_USER) //display the current control mode
		    add_rfont2(MENULMARGIN,130,NEAR_Z,150,150,100,2, "    Control Mode: Manual ");    
		else                            // random
		    add_rfont2(MENULMARGIN,130,NEAR_Z,150,150,100,2, "    Control Mode: Random");

		if (captions== 1) //display the current control mode
		    add_rfont2(MENULMARGIN,140,NEAR_Z,150,150,100,2, "        Captions: On ");    
		else                            // random
		    add_rfont2(MENULMARGIN,140,NEAR_Z,150,150,100,2, "        Captions: Off");

		 
		add_rfont2(MENULMARGIN,160,NEAR_Z,150,150,100,0,"         L1,L2: control mode -- R1,R2: captions");    
		add_rfont2(MENULMARGIN,180,NEAR_Z,150,150,100,2,     "    press START to begin ");    


/*
         add_rfont2 (10, 130, NEAR_Z, 150, 150, 100, 1, test_string);
         add_rfont2 (10, 150, NEAR_Z, 150, 150, 100, 1, "Use L1 and L2 to Toggle Manual/Random");
         add_rfont2 (10, 180, NEAR_Z, 150, 150, 100, 1, "START to begin");
*/
         // Exit this loop?
         if (((paddata & ~prevpaddata) & SJOY_PS2_START) && (isok > 0))
         {
            isok = -15;
            break;
         }

         if (((paddata & ~prevpaddata) & SJOY_PS2_L1) && (isok > 0))
         {
            isok = -15;
            mode = 1 - mode;
         }

         if (((paddata & ~prevpaddata) & SJOY_PS2_L2) && (isok > 0))
         {
            isok = -15;
            mode = 1 - mode;
         }

         if (((paddata & ~prevpaddata) & SJOY_PS2_R1) && (isok > 0))
         {
            isok = -15;
            captions = 1 - captions;
         }

         if (((paddata & ~prevpaddata) & SJOY_PS2_R2) && (isok > 0))
         {
            isok = -15;
            captions = 1 - captions;
         }


         // Pad debounce
         isok++;

         VIF_SwapBuffers (VIF_DEST_VIF1);
         VIF_TransferWait (VIF_DEST_VIF1);

         ps2_gs_set_half_offset ((frame & 1) ? &db.draw1 : &db.draw0, odev);
         frame++;

         // Wait for VSync
         odev = !ps2_gs_sync_v (0);
      }

      printf ("use l1 l2 buttons on pad 1 to toggle between demos\n");
      odev = !ps2_gs_sync_v (0);

	 //ps2_gs_set_center(20,20);

      while (1)
      {
         OS_Update ();

         ps2_gs_swap_dbuff (&db, frame);
         prevpaddata = paddata;
         paddata = sjoy_get_ps2_button (0);


         // Put some handy information into the last qword of VU1 memory
         {

            if (frame & 1)
            {
               gameinfo.ui32[0] = db.draw0.frame1.FBP;
               gameinfo.ui32[1] = db.draw1.frame1.FBP;
            }
            else
            {
               gameinfo.ui32[0] = db.draw1.frame1.FBP;
               gameinfo.ui32[1] = db.draw0.frame1.FBP;
            }

            //--- was gonna give you these... but its easier just to tell you...
            //--- zbp is 0x8c and the start of free gs mem is zbp+0x46+0x46 ( NOTE I'M KEEPING SOME FOR THE MENU AND THE SCROLLING TEXT MESSAGE THAT WILL BE MERGED IN LATER)
            //gameinfo.ui32[2]=db.draw1.zbuf1.ZBP;           //the start of the z buffer
            //gameinfo.ui32[3]=db.draw1.zbuf1.ZBP+0x46+0x46;   //the start of free memory ( NOTE I'M KEEPING SOME FOR THE MENU AND THE SCROLLING TEXT MESSAGE THAT WILL BE MERGED IN LATER)

            //--- so instead.. have the pad data and a random number...
            gameinfo.fl32[2] = rand () / 432143.21787f;
            gameinfo.ui32[3] = ConvertPadInfo (paddata);

            VIF_AddVUData ((OS_Qword *)&gameinfo, 1, 1023, VIF_ATTACHMENT_INLINE, VIF_DEST_VIF1);
         }

         // Start execution of code currently in VU1
         VIF_Add4Tags (VIF_CMD_MSCAL,  0, 0,
                       VIF_CMD_NOP,    0, 0,
                       VIF_CMD_FLUSHE, 0, 0,
                       VIF_CMD_NOP,    0, 0, VIF_DEST_VIF1);

         if (mode == CONTROL_MODE_USER)
         {
            if ((paddata & SJOY_PS2_L1) && (paddata & SJOY_PS2_L2))
            {
               // Exit the current while loop
               break;
            }

            if (((paddata & ~prevpaddata) & SJOY_PS2_L2) && (isok > 0))
            {
               isok = -15;
               next_demo_node (&dcl, 1);
            }

            if (((paddata & ~prevpaddata) & SJOY_PS2_L1) && (isok > 0))
            {
               isok = -15;
               next_demo_node (&dcl, -1);
            }

            if (((paddata & ~prevpaddata) & SJOY_PS2_SELECT) && (isok > 0))
            {
               isok = -14;

               gHelpOn = gHelpOn ? 0 : 1;
            }
         }
         else
         {
            // random play mode
            if ((paddata & SJOY_PS2_L1) && (paddata & SJOY_PS2_L2))
            {
               // Exit the current while loop
               break;
            }

            if (random_counter <= 0) 
            {
               next_demo_node (&dcl, rand () % 10);
               random_counter = (5 + (rand () % 5)) * 60;
               isok = -15;
            }

            random_counter--;
         }

         if (isok == -15)
         {

            // Reinitialize GS registers
            if (ResetGSRegisters ())
            {
               printf ("MAIN ERROR: Couldn't reinitialize GS registers\n");
               exit (1);
            }

            // Load in the newly selected demo code and data
            if (UploadVUCodeAndData (&dcl))
            {
               printf ("Couldn't add VU Data\n");
               exit (1);
            }
         }
         else if(captions)
         {
            // Only bother writing this if we're not switching demo...
            if (mode == CONTROL_MODE_USER)
            {
               if (gHelpOn)
                  add_rfont2 (10, 25, NEAR_Z, 100, 100, 100, 0, dcl.current->helpText);

               sprintf (test_string, "%s", dcl.current->name);
               add_rfont2 (10, 200, NEAR_Z, 100, 100, 100, 0, test_string);

               sprintf (test_string, BUTTON_HELP_USER);
               add_rfont2 (10, 210, NEAR_Z, 100, 100, 100, 0, test_string);
            }
            else                        // random
            {
               sprintf (test_string, "%s", dcl.current->name);
               add_rfont2 (10, 200, NEAR_Z, 100, 100, 100, 0, test_string);

               sprintf (test_string, BUTTON_HELP_RAND);
               add_rfont2 (10, 210, NEAR_Z, 100, 100, 100, 0, test_string);
            }
         }

         // Pad debounce
         isok++;

         VIF_SwapBuffers (VIF_DEST_VIF1);
         VIF_TransferWait (VIF_DEST_VIF1);

         ps2_gs_set_half_offset ((frame & 1) ? &db.draw1 : &db.draw0, odev);
         frame++;

         // Wait for VSync
         odev = !ps2_gs_sync_v (0);
      }

      // Exit this loop?
      if ((paddata & ~prevpaddata) & SJOY_PS2_SELECT)
         break;
   }

   // Uninitialization
   if (OS_Uninit () != OS_RET_OK)
   {
      printf ("MAIN ERROR: Couldn't uninitialize the low-level systems\n");
      exit (1);
   }
   else
      printf ("MAIN: Uninitialization of low-level systems complete\n");

   return (0);
}
