//------------------------------------------------------------
// PS2Linux specific OS-level functionality
//
// Copyright (C)2002 Sony Computer Entertainment America Inc.
//
// This file is subject to the terms and conditions of the GNU
// General Public Licence Version 2.  See the file "COPYING"
// in the main directory of this archive for more details.
//------------------------------------------------------------

#include <stdlib.h>              // Standard C Headers
#include <stdio.h>               //

#include <linux/ps2/gs.h>        // PS2 Linux
#include <ps2gs.h>               //
#include <ps2dma.h>              //
#include <ps2vpu.h>              //
#include <linux/ps2/dev.h>

#include "os.h"                  // Project Headers
#include "debug.h"               //
#include "ps2_dma.h"             //
#include "ps2_vif.h"             //
#include "sjoy.h"                //

void	hack_disp2(); 		// hack display settings if need s be

//-------------------------------------
IMPORT_VPU_SYMBOL (OS_Qword *, DMA_Chain_VIF0, 0)        // From dmachainhook.dsm
IMPORT_VPU_SYMBOL (OS_Qword *, DMA_Chain_VIF1, 0)        //

//-------------------------------------
static VPUFILE         *OS_vfd;
static ps2_vpu         *OS_vpu[2];
static int             OS_fd_gs;
static struct OS_Data  OS_OSData;
//static char            *OS_ProgramName;
static	int  		dMode=-1;


//-------------------------------------
//
//-------------------------------------
static int OS_Release (void)
{
   sjoy_close ();

   if (OS_vpu[1])
   {
      ps2_vpu_close (OS_vpu[1]);
      OS_vpu[1] = NULL;
   }

   if (OS_vpu[0])
   {
      ps2_vpu_close (OS_vpu[0]);
      OS_vpu[0] = NULL;
   }

   if (OS_fd_gs >= 0)
   {
      ps2_gs_close ();
      OS_fd_gs = -1;
   }

   return (PS2_GS_VC_ACQ_SUCCESS);
}

//-------------------------------------
//
//-------------------------------------
int OS_Acquire (void)
{
   OS_fd_gs = ps2_gs_open (-1);
   OS_vpu[0] = ps2_vpu_open (0);
   OS_vpu[1] = ps2_vpu_open (1);

   if ((OS_fd_gs < 0) || (OS_vpu[0] == NULL) || (OS_vpu[1] == NULL))
   {
       OS_Release ();
       return (PS2_GS_VC_ACQ_FAILURE);
   }

   ps2_vpu_reset (OS_vpu[1]);
   ps2_vpu_reset (OS_vpu[0]);

   ps2_gs_reset (0, OS_OSData.Interlace, OS_OSData.VideoMode, OS_OSData.FrameOrField, OS_OSData.ScreenResolution, OS_OSData.RefreshRate);
   ps2_gs_start_display (1);

   // hook in display 
   hack_disp2();
   sjoy_open ();

   // Everything was fine
   return (PS2_GS_VC_ACQ_SUCCESS);
}

//-------------------------------------
//
//-------------------------------------
unsigned int OS_Init (int argc, char *argv[])
{
	int	narg=1;
	while( (narg<argc) && (argv[narg][0]=='-') )
	{
		switch(argv[narg][1])
		{
			case	'n':
			case	'N':
				dMode=0;
			break;
			
			case	'p':
			case	'P':
				dMode=1;
			break;

			case	'w':
			case	'W':
				dMode=2;
			break;

			case	's':
			case	'S':
				dMode=3;
			break;

			case	'h':
			case	'H':
				printf("options are:\n");
				printf("            -h 	 help\n");
				printf("            -n   tv- ntsc interlace\n");
				printf("            -p   tv- pal interlace\n");
				printf("            -w   pc- 640x480 widescreen\n");
				printf("            -s   pc- 640x480 full screen stretched\n");
				printf("best viewed in ntsc..\n");
			exit(0);

			default:
				printf("bad options - try -h for help\n");
			exit(0);
		}
		narg++;
	}

	if( (argc!=narg) || (dMode==-1) )
	{
	 	printf("bad, or no options specified - try -h for help\n");
		exit(0);
	}

	// common settings
   	OS_OSData.FrameDepth = PS2_GS_PSMCT32;
 	OS_OSData.FrameOrField = PS2_GS_FRAME;	 
    	OS_OSData.RefreshRate = PS2_GS_60Hz;		// only vesa?
 	OS_OSData.ScreenResolution = PS2_GS_640x480; 	// only applies for vesa mode?
   	OS_OSData.ZDepth = PS2_GS_PSMZ24;
   	OS_OSData.ZBit = 24;
	
	switch(dMode)
	{
		case	0: 	// NTSC
		         OS_OSData.Interlace = PS2_GS_INTERLACE;
		         OS_OSData.VideoMode = PS2_GS_NTSC;
		break;

		case	1:	// pal
		         OS_OSData.Interlace = PS2_GS_INTERLACE;
		         OS_OSData.VideoMode = PS2_GS_PAL;
		break;


		default:	// both letterbox + full height here - will hook magv etc later
		         OS_OSData.Interlace = PS2_GS_NOINTERLACE;
		         OS_OSData.VideoMode = PS2_GS_VESA;
		break;
	}

	// ok - ready to go!
	// 

	


   // Effectively allocate all static memory blocks which requested being in the DMA memory space
   ps2mem_alloc_dmamem ();

   // Initialize the GS library in graphics mode (i.e. non text mode)
   ps2_gs_vc_graphicsmode ();

   OS_fd_gs = ps2_gs_open (-1);
   OS_vpu[0] = ps2_vpu_open (0);
   OS_vpu[1] = ps2_vpu_open (1);

   if ((OS_fd_gs < 0) || (OS_vpu[0] == NULL) || (OS_vpu[1] == NULL))
   {
      if (OS_vpu[1])
      {
         ps2_vpu_close (OS_vpu[1]);
         OS_vpu[1] = NULL;
      }

      if (OS_vpu[0])
      {
         ps2_vpu_close (OS_vpu[0]);
         OS_vpu[0] = NULL;
      }

      if (OS_fd_gs >= 0)
      {
         ps2_gs_close ();
         OS_fd_gs = -1;
      }

      return (PS2_GS_VC_ACQ_FAILURE);
   }

   // Open and read the DMA chain, which is contained in an ELF file
   if ((OS_vfd = vpuobj_open ("dmachainhook.elf", O_DATA_PS2MEM)) == NULL)
   {
      // Couldn't open or read the file.  Fail and exit
      return (OS_RET_ERROR);
   }

   // Initialize some low-level PS2 modules
   {
      if (DMA_ModuleInit (OS_vfd, OS_vpu[0], OS_vpu[1], &OS_OSData) != DMA_RET_OK)
      {
         DEBUG_Print ("ERROR: OS Couldn't initialize DMA module\n");

         return (OS_RET_ERROR);
      }

      if (VIF_ModuleInit (DMA_Chain_VIF0, 1024 * 2,
                          DMA_Chain_VIF1, 1024 * 1024) != VIF_RET_OK)
      {
         DEBUG_Print ("ERROR: Couldn't initialize VIF module\n");

         DMA_ModuleUninit ();

         vpuobj_close (OS_vfd);

         // Close GS and VU1 and VU0
         ps2_gs_close ();
         ps2_vpu_close (OS_vpu[1]);
         ps2_vpu_close (OS_vpu[0]);

         return (OS_RET_ERROR);
      }
   }

   // Initialize the GS CRTC
   ps2_gs_reset (0, OS_OSData.Interlace, OS_OSData.VideoMode, OS_OSData.FrameOrField, OS_OSData.ScreenResolution, OS_OSData.RefreshRate); 

   // Set the GS registers to values we know to be valid and working
   {
      struct
      {
         ps2_giftag            giftag;

         ps2_gsreg_alpha       alpha1;
         ps2_gsreg_addr        alpha1addr;

         ps2_gsreg_clamp       clamp1;
         ps2_gsreg_addr        clamp1addr;

         ps2_gsreg_colclamp    colclamp;
         ps2_gsreg_addr        colclampaddr;

         ps2_gsreg_dimx        dimx;
         ps2_gsreg_addr        dimxaddr;

         ps2_gsreg_dthe        dthe;
         ps2_gsreg_addr        dtheaddr;

         ps2_gsreg_fba         fba1;
         ps2_gsreg_addr        fba1addr;

         ps2_gsreg_fogcol      fogcol;
         ps2_gsreg_addr        fogcoladdr;

         ps2_gsreg_pabe        pabe;
         ps2_gsreg_addr        pabeaddr;

         ps2_gsreg_prmodecont  prmodecont;
         ps2_gsreg_addr        prmodecontaddr;

         ps2_gsreg_test        test1;
         ps2_gsreg_addr        test1addr;

         ps2_gsreg_texa        texa;
         ps2_gsreg_addr        texaaddr;

         ps2_gsreg_zbuf        zbuf1;
         ps2_gsreg_addr        zbuf1addr;
      } DefaultSettings;

      PS2_GIFTAG_CLEAR_TAG (&DefaultSettings.giftag);
      DefaultSettings.giftag.NLOOP = sizeof (DefaultSettings) / 16 - 1;
      DefaultSettings.giftag.EOP = 1;
      DefaultSettings.giftag.PRE = 0;
      DefaultSettings.giftag.NREG = 1;
      DefaultSettings.giftag.REGS0 = PS2_GIFTAG_REGS_AD;

      DefaultSettings.alpha1addr = PS2_GS_ALPHA_1;
      DefaultSettings.clamp1addr = PS2_GS_CLAMP_1;
      DefaultSettings.colclampaddr = PS2_GS_COLCLAMP;
      DefaultSettings.dimxaddr = PS2_GS_DIMX;
      DefaultSettings.dtheaddr = PS2_GS_DTHE;
      DefaultSettings.fba1addr = PS2_GS_FBA_1;
      DefaultSettings.fogcoladdr = PS2_GS_FOGCOL;
      DefaultSettings.pabeaddr = PS2_GS_PABE;
      DefaultSettings.prmodecontaddr = PS2_GS_PRMODECONT;
      DefaultSettings.test1addr = PS2_GS_TEST_1;
      DefaultSettings.texaaddr = PS2_GS_TEXA;
      DefaultSettings.zbuf1addr = PS2_GS_ZBUF_1;

      *(__u64 *)&DefaultSettings.alpha1 = PS2_GS_SETREG_ALPHA (0, 1, 0, 1, 0);
      *(__u64 *)&DefaultSettings.clamp1 = PS2_GS_SETREG_CLAMP (0, 0, 0, 256, 0, 256);
      *(__u64 *)&DefaultSettings.colclamp = PS2_GS_SETREG_COLCLAMP (1);
      *(__u64 *)&DefaultSettings.dimx = PS2_GS_SETREG_DIMX (-4, 2, -3, 3, 0, -2, 1, -1, -3, 3, -4, 2, 1, -1, 0, -2);
      *(__u64 *)&DefaultSettings.dthe = PS2_GS_SETREG_DTHE (0);
      *(__u64 *)&DefaultSettings.fba1 = PS2_GS_SETREG_FBA (0);
      *(__u64 *)&DefaultSettings.fogcol = PS2_GS_SETREG_FOGCOL (0x0, 0x0, 0x0);
      *(__u64 *)&DefaultSettings.pabe = PS2_GS_SETREG_PABE (0);
      *(__u64 *)&DefaultSettings.prmodecont = PS2_GS_SETREG_PRMODECONT (1);
      *(__u64 *)&DefaultSettings.test1 = PS2_GS_SETREG_TEST (0, 0, 0, 0, 0, 0, 1, PS2_GS_ZALWAYS);
      *(__u64 *)&DefaultSettings.texa = PS2_GS_SETREG_TEXA (0x7F, 1, 0x81);
      *(__u64 *)&DefaultSettings.zbuf1 = PS2_GS_SETREG_ZBUF (0, 0, 0);

      ps2_gs_put_drawenv (&DefaultSettings.giftag);
   }

   // Turn the GS display ON
   ps2_gs_start_display (1);

   
   // From this point, enable GS mode switch between applications
   ps2_gs_vc_enablevcswitch (OS_Acquire, OS_Release);

   // Prevent switching within a frame
   ps2_gs_vc_lock ();


   // hook in display 
   hack_disp2();
   sjoy_open ();

   // Everything was fine
   return (OS_RET_OK);
}

//-------------------------------------
//
//-------------------------------------
unsigned int OS_Uninit (void)
{
   sjoy_close ();

   ps2_gs_vc_unlock ();

   VIF_ModuleUninit ();
   DMA_ModuleUninit ();

   vpuobj_close (OS_vfd);

   // Close GS and VU1 and VU0
   ps2_gs_close ();
   ps2_vpu_close (OS_vpu[1]);
   ps2_vpu_close (OS_vpu[0]);

   // Everything was fine
   return (OS_RET_OK);
}

//-------------------------------------
//
//-------------------------------------
unsigned int OS_Update (void)
{
   // Permit switching to a different console
   ps2_gs_vc_unlock ();
   ps2_gs_vc_lock ();

   // Read controllers
   sjoy_poll ();

   // Everything was fine
   return (OS_RET_OK);
}

//-------------------------------------
//
//-------------------------------------
unsigned int OS_GetOSData (struct OS_Data **osdata)
{
   if (osdata == NULL)
      return (OS_RET_INVALIDPARAMETER);

   *osdata = &OS_OSData;

   // Everything was fine
   return (OS_RET_OK);
}


// hack the display2 register.. 
// only do this if in a pc display mode - and coz I don't have
// a pal telly to hand..


void	hack_disp2()
{
	
	if (dMode==1)
	{
		struct	ps2_gssreg		R;
		ps2_gssreg_display 		*dsp=(ps2_gssreg_display *)&R.val;

		R.reg=PS2_GSSREG_DISPLAY2;	// display 2 - linux seems to use this one for output

		if(ioctl(OS_fd_gs,PS2IOC_GGSSREG,(void *)&R)<0)
		{
		 	printf("ioctl failed to get gs display2\n");
			OS_Uninit();
		}

		dsp->DX=700;
		dsp->DY=90 ;
		dsp->DH=447;

		if(ioctl(OS_fd_gs,PS2IOC_SGSSREG,(void *)&R)<0)
		{
		 	printf("ioctl failed to get gs display2\n");
			OS_Uninit();
		}
	}
	
	if( (dMode==2) || (dMode==3) )
	{
		struct	ps2_gssreg		R;
		ps2_gssreg_display 		*dsp=(ps2_gssreg_display *)&R.val;

		R.reg=PS2_GSSREG_DISPLAY2;	// display 2 - linux seems to use this one for output


		// set x values
		
		dsp->DX=276;
		dsp->MAGH=1;
		dsp->DW=1279; 		
		
		// now set Y values depending on mode

		if(dMode==2)		// letterbox
		{
			dsp->DY=34 + ((480-224)>>1);	 // center
			dsp->MAGV=0;
			dsp->DH=223;
		}
		else  	// fullscreen stretched
		{
			dsp->DY=34;
			dsp->MAGV=1;
			dsp->DH=(224*2)-1; 	
		}


		if(ioctl(OS_fd_gs,PS2IOC_SGSSREG,(void *)&R)<0)
		{
		 	printf("ioctl failed to get gs display2\n");
			OS_Uninit();
		}
	}
}
