//-----------------------------------------------------------------
// PS2 DMA related functionality
// Low-level dma functions.  For setting up and building dma lists.
//
// Geoff Audy, July 31st 2002
//
// Copyright (C)2002 Sony Computer Entertainment America Inc.
//
// This file is subject to the terms and conditions of the GNU
// General Public Licence Version 2.  See the file "COPYING"
// in the main directory of this archive for more details.
//-----------------------------------------------------------------

#ifndef __PS2_DMA_H__
#define __PS2_DMA_H__

#ifdef __cplusplus
extern "C" {
#endif

//-------------------------------------
enum
{
   DMA_RET_OK = 0,
   DMA_RET_INVALIDPARAMETER,
   DMA_RET_WRONGALIGNMENT,
   DMA_RET_DATATOOBIGTOFIT,
   DMA_RET_ERROR,

   DMA_RET_MAX
};

enum
{
   DMA_BLOCKING_BLOCKING = 0,
   DMA_BLOCKING_NONBLOCKING,

   DMA_BLOCKING_MAX
};

#define DMA_CHANNEL_VIF0     0x00000001
#define DMA_CHANNEL_VIF1     0x00000002
#define DMA_CHANNEL_MAX      0x00000004

enum
{
   DMA_ATTACHMENT_INLINE = 0,
   DMA_ATTACHMENT_REFERENCE,
   DMA_ATTACHMENT_CALL,

   DMA_ATTACHMENT_MAX
};

// Allocates a variable "sym", which is a "nbrelements"-element array of type "type", aligned on "aval" bits
#define DMA_ALLOC_STATIC(sym, type, nbrelements, aval) typeof(type) *sym = 0; __asm__ (".section __dmamem_symtab,\"a\"\n.previous"); const struct __dmamem_symbol __dmamem_symtab_##sym __attribute__ ((section ("__dmamem_symtab"))) = {DMAMEM_SYMTAB_MAGIC, (void **)&sym, sizeof (type) * nbrelements, aval, DMAMEM_SYMTAB_SIGNATURE (&sym)};

//-------------------------------------
unsigned int DMA_ModuleInit (VPUFILE *vfd, ps2_vpu *vpu0, ps2_vpu *vpu1, struct OS_Data *osdata);
unsigned int DMA_ModuleUninit (void);

unsigned int DMA_ChainInit (OS_Qword *chainbuffer, unsigned int buffersize,
                            unsigned int channel);
unsigned int DMA_ChainSend (OS_Qword *chainbuffer);
unsigned int DMA_ChainTransferWait (OS_Qword *chainbuffer);
unsigned int DMA_ChainAdd (OS_Qword *chainbuffer, OS_Qword *buffer,
                           unsigned int buffersizeinqw, unsigned int attachment);

unsigned int DMA_GetChainSize (OS_Qword *chainbuffer, unsigned int *chainsize);

// I can't believe this function prototype was messed up in the "ps2mem.h"
// header file... It shows they never even tested it!!!
int ps2mem_alloc_dmamem (void);

#ifdef __cplusplus
}
#endif

#endif   // __PS2_DMA_H__
